<?php


/*
   This program computes the rectangular XYZ-coordinates of any point on
   the surface of a spheroid based on the ellipsoid parameters and the
   coordinates and elevation of the point.

Several historical spheroid models are available:

IERS 2003
WGS 84
WGS 72
SouthAmerican 1969
Everest 1830 (1967 Definition)
Everest 1830 (Modified 1967)
Australian National 1966
Krassovsky
International 1924 | Hayford 1910
Clarke 1880
Clarke 1878
Clarke 1866

Clarke 1858

Bessel 1841
Airy 1830 (Modified)
Airy 1830
Everest 1830
Plessis 1817
 -- Maupertuis 1738 -- REMOVED

*/



// ----------------------------------
// Program version and revision date.

   $_ProgramVersion_ = 'v1.02';
   $_BaseScriptName_ = BaseName($_SERVER['SCRIPT_FILENAME']);
   $at = "&#97;&#116;";
   $_RevisionDate_ = gmdate("Y F d  l $at h:i:s A", FileMTime($_BaseScriptName_)).' GMT';

// GET NAME OF PROGRAM SCRIPT TO RUN
// WHEN THE SUBMIT BUTTON IS CLICKED.
// IN THIS CASE, IT MEANS THIS SCRIPT.
   $_RUN_ = $_SERVER['SCRIPT_NAME'];

   $ButtonClicked  = '';
   $ButtonClicked .= @$_POST['WGS84Button']; // DEFAULT BUTTON IS ALWAYS THE FIRST BUTTON
   $ButtonClicked .= @$_POST['IERS2003Button'];
   $ButtonClicked .= @$_POST['WGS72Button'];
   $ButtonClicked .= @$_POST['SouthAmerican1969Button'];
   $ButtonClicked .= @$_POST['Everest18301967DefinitionButton'];
   $ButtonClicked .= @$_POST['Everest1830Modified1967Button'];

   $ButtonClicked .= @$_POST['AustralianNational1966Button'];
   $ButtonClicked .= @$_POST['KrassovskyButton'];
   $ButtonClicked .= @$_POST['International1924Hayford1910Button'];

   $ButtonClicked .= @$_POST['Clarke1880Button'];
   $ButtonClicked .= @$_POST['Clarke1878Button'];
   $ButtonClicked .= @$_POST['Clarke1866Button'];
   $ButtonClicked .= @$_POST['Clarke1858Button'];

   $ButtonClicked .= @$_POST['Bessel1841Button'];
   $ButtonClicked .= @$_POST['Airy1830ModifiedButton'];
   $ButtonClicked .= @$_POST['Airy1830Button'];
   $ButtonClicked .= @$_POST['Everest1830Button'];
   $ButtonClicked .= @$_POST['Plessis1817Button'];
   $ButtonClicked .= @$_POST['Maupertuis1738Button'];
   $ButtonClicked  = trim($ButtonClicked);



// SET INITIAL DEFAULT STARTUP MODEL TO GRS80/WGS84
   if ($ButtonClicked == '') {$ButtonClicked = "GRS 80 | WGS 84";}

   $LatLon    = @trim($_POST['LatLon']);
   $elevation = @strtolower(trim($_POST['Elevation']));


// SET INITIAL DEFAULT STARTUP COORDINATES
   if ($LatLon == '' and $elevation == '')
      {
        $LatLon    = '+42.6764593056 ,  -76.8230111111'; // Set home defaults
        $Elevation = '+261.823 m'; // Set home default

   // DEFAULT OVERRIDES FOR TESTING AND CALIBRATION.
        $LatLon    = '+53.6119903611 ,  -1.6644422222'; // Control test defaults
        $Elevation = '+299.800 m'; // Control Test dfault

        $LatLon = Parse_Lat_Lon_String($LatLon);

        $elevation = floatval($Elevation);
      }

   $LatLon = str_replace("°",      " ", $LatLon);
   $LatLon = str_replace("&#176;", " ", $LatLon);
   $LatLon = str_replace("&deg;",  " ", $LatLon);

   $LatLon = Parse_Lat_Lon_String($LatLon);

   list($LatDeg, $LonDeg) = preg_split("[,]", $LatLon);
   $LonSign  = ($LonDeg < 0)? '-' : '+';
   $LonSignV = ($LonDeg < 0)?  -1  : 1;

   $LatDeg = DMS_to_Deg($LatDeg);
   $LonDeg = DMS_to_Deg($LonDeg);
   $LatDMS = Deg_to_DMS($LatDeg, 4, '+', 'd');
   $LatDeg = sprintf("%+3.10f", $LatDeg);

   $LonDMS = Deg_to_DMS($LonDeg, 4, '+', 'd');
   $LonDMS = Deg_to_DMS(floatval($LonSignV*$LonDeg), 4, '+', 'd');
   $LonDeg = sprintf("%+3.10f", floatval($LonSignV*$LonDeg));

   $LatLon = "$LatDeg ,  $LonDeg";

// CHECK ELEVATION ARGUMENT.  IF FEET (ft) INDICATED,
// THEN CONVERT VALUE INTO EQUIVALENT METERS.
   $w = (strpos($elevation, 'f') === FALSE)? 1 : 0.3048;
   $elev = floatval($elevation) * $w;

// SET N/S/E/W indicators for lat/lon values
   $SN = ($LatDeg < 0)? 'S':'N';  if ($LatDeg == 0){$SN = ' ';}
   $WE = ($LonDeg < 0)? 'W':'E';  if ($LonDeg == 0){$WE = ' ';}


   $StatsStr = $description = '';

   if ($ButtonClicked == "WGS 72")
      {$StatsStr = XYZ_For_Ellipsoid (6378135, 6356750.52, $LatDeg, $LonDeg, $elev);  $description = "Was Used by the NASA | U.S.A. Department of Defense";}

   if ($ButtonClicked == "GRS 80 | WGS 84")
      {$StatsStr = XYZ_For_Ellipsoid (6378137, 6356752.314245, $LatDeg, $LonDeg, $elev); $description = "Used worldwide by the ITRS - International Terrestrial Reference\nSystem and the GPS - Global Positioning System";}

   if ($ButtonClicked == "International 1924 | Hayford 1910")
      {$StatsStr = XYZ_For_Ellipsoid (6378388, 6356911.946, $LatDeg, $LonDeg, $elev); $description = "Used for the USA and Europe";}

   if ($ButtonClicked == "Airy 1830")
      {$StatsStr = XYZ_For_Ellipsoid (6377563.396, 6356256.909, $LatDeg, $LonDeg, $elev); $description = "Used for Great Britain";}

   if ($ButtonClicked == "Airy 1830 (Modified)")
      {$StatsStr = XYZ_For_Ellipsoid (6377340.189, 6356034.447, $LatDeg, $LonDeg, $elev); $description = "Used for Great Britain";}

   if ($ButtonClicked == "Clarke 1858")
      {$StatsStr = XYZ_For_Ellipsoid (6378293.645, 6356617.938, $LatDeg, $LonDeg, $elev); $description = "Used for Great Britain and Europe";}

// 1 metre (1866) value = 3.280869367 feet (as given in the 1866 Clarke paper)
// 1 meter (2017) value = 3.280839895 ft (NIST, 1 m = EXACTLY 0.3048 ft)
// Difference 3.280839895 - 3.280869367 = -0.0000294720 inch (shorter)
// Modern meter value = 0.0000294720 inch shorter than Clarke's (1866) value.

   if ($ButtonClicked == "Clarke 1866")
      {$StatsStr = XYZ_For_Ellipsoid (6378206.4, 6356583.8, $LatDeg, $LonDeg, $elev); $description = "Used for North America";}

   if ($ButtonClicked == "Clarke 1878")
      {$StatsStr = XYZ_For_Ellipsoid (6378190, 6356456, $LatDeg, $LonDeg, $elev); $description = "Used for North America";}

   if ($ButtonClicked == "Clarke 1880")
      {$StatsStr = XYZ_For_Ellipsoid (6378249.145, 6356514.87, $LatDeg, $LonDeg, $elev); $description = "Used for France and Africa";}

   if ($ButtonClicked == "IERS 2003")
      {$StatsStr = XYZ_For_Ellipsoid (6378136.6, 6356751.9, $LatDeg,$LonDeg, $elev); $description = "Used by the IERS - International Earth Rotation Service";}

   if ($ButtonClicked == "Australian National 1966")
      {$StatsStr = XYZ_For_Ellipsoid (6378160, 6356774.719, $LatDeg,$LonDeg, $elev); $description = "";}

   if ($ButtonClicked == "South American 1969")
      {$StatsStr = XYZ_For_Ellipsoid (6378160, 6356774.719, $LatDeg,$LonDeg, $elev); $description = "";}

   if ($ButtonClicked == "Krassovsky 1940")
      {$StatsStr = XYZ_For_Ellipsoid (6378245, 6356863.019, $LatDeg,$LonDeg, $elev); $description = "Used for USSR, Russia and Romania";}

   if ($ButtonClicked == "Everest 1830")
      {$StatsStr = XYZ_For_Ellipsoid (6377299.365, 6356098.359, $LatDeg,$LonDeg, $elev); $description = "Used for India";}

/*
   if ($ButtonClicked == "Maupertuis 1738")
      {$StatsStr = XYZ_For_Ellipsoid (6397300, 6363806.283, $LatDeg,$LonDeg, $elev); $description = "Used for France";}
*/

   if ($ButtonClicked == "Plessis 1817")
      {$StatsStr = XYZ_For_Ellipsoid (6376523.0, 6355862.9333, $LatDeg,$LonDeg, $elev); $description = "Used for France";}

   if ($ButtonClicked == "Bessel 1841")
      {$StatsStr = XYZ_For_Ellipsoid (6377397.155, 6356078.963, $LatDeg,$LonDeg, $elev); $description = "Used for Europe and Japan";}

   if ($ButtonClicked == "Everest 1830 (Modified 1967)")
      {$StatsStr = XYZ_For_Ellipsoid (6377304.063, 6356103.0390, $LatDeg,$LonDeg, $elev); $description = "Used for West Malaysia and Singapore";}

   if ($ButtonClicked == "Everest 1830 (1967 Definition)")
      {$StatsStr = XYZ_For_Ellipsoid (6377298.556, 6356097.550, $LatDeg,$LonDeg, $elev); $description = "Used for Brunei and East Malaysia";}



       $elevation = sprintf("%+1.3f", $elev).' m';















// PARSE RETURNED ELLIPSOID STATS STRING

   list($x, $y, $z, $a, $b, $e, $H, $EquatorCircum, $PolarCircum) = preg_split("[ ]", $StatsStr);

   $f = ($a - $b) / $a;

   $e2 = $e*$e;



   $invf = "1/" . sprintf("%1.10f", 1/$f);

   $Hft = $H / 0.3048;  $w = ($H < 0)? '-':'+';
   $hft = floor(abs($Hft));
   $hin = sprintf("%1.1f", 12*(abs($Hft) - $hft));
   if ($hin == 12){$hft++; $hin = '0.0';}
   $Hftin = "$w$hft ft and $hin in";

   $w   = 1609.344;
   $ami = $a / $w;
   $bmi = $b / $w;
   $ami = sprintf("%1.6f", $ami);
   $bmi = sprintf("%1.6f", $bmi);

   $a   = sprintf("%1.4f",  $a);
   $b   = sprintf("%1.4f",  $b);
   $e2  = sprintf("%1.12f", $e2);
   $OneMinuse = sprintf("%1.12f",  1 - $e);
   $OneMinuse2 = sprintf("%1.12f", 1 - $e2);
   $e   = sprintf("%1.12f", $e);

   $f   = sprintf("%1.12f", $f);
   $H   = sprintf("%+1.4f", $H);
   $Hft = sprintf("%+1.2f", $Hft);

   $xmi = $x / $w;
   $ymi = $y / $w;
   $zmi = $z / $w;

   $x   = sprintf("%+1.4f", $x);
   $y   = sprintf("%+1.4f", $y);
   $z   = sprintf("%+1.4f", $z);
   $xmi = sprintf("%+1.6f", $xmi);
   $ymi = sprintf("%+1.6f", $ymi);
   $zmi = sprintf("%+1.6f", $zmi);

   $x   = substr("             $x", -13);
   $y   = substr("             $y", -13);
   $z   = substr("             $z", -13);

   $xmi = substr("              $xmi", -12);
   $ymi = substr("              $ymi", -12);
   $zmi = substr("              $zmi", -12);

   $CircumDiff = $EquatorCircum - $PolarCircum;

   $EquatorCircumMi = sprintf("%1.6f", $EquatorCircum / 1609.344);
   $PolarCircumMi   = sprintf("%1.6f", $PolarCircum / 1609.344);
   $CircumDiffMi    = sprintf("%1.6f", $CircumDiff / 1609.344);

   $QECircumMi = bcdiv($EquatorCircumMi, '4', 4);
   $QPCircumMi = bcdiv($PolarCircumMi,   '4', 4);

   $QECircum = bcdiv($EquatorCircum, '4', 4);
   $QPCircum = bcdiv($PolarCircum,   '4', 4);

   $EquatorCircum = sprintf("%1.4f", $EquatorCircum);
   $PolarCircum   = sprintf("%1.4f", $PolarCircum);
   $CircumDiff    = sprintf("%1.4f", $CircumDiff);






/*

f = (1 - b/a)

1     b    a - b
-  -  - = ------
1     a      a

(a - b)² = a² + b² - 2*a*b

      a² + b² - 2*a*b
f² =  ---------------
             a²

2*f - f²
                          a² + b² - 2*a*b
e = SqRoot(2*(1 - b/a) - -----------------)
                                 a²

2*(1 - b/a) - 1 + (b/a)² - 2*b/a

*/


// COMPUTE PARALLEL CIRCLES TABLE FOR MODEL (a, b) PARAMETERS.
   $ParallelCirclesTable = Parallel_Circles_Table($a, $b);




   $OutputTextArea =
"$ButtonClicked Ellipsoid Model Parameters

$description

Equatorial radius   R  =  $a m  =  $ami mi
Polar radius        r  =  $b m  =  $bmi mi
Flattening          f  =  $f  =  $invf
Eccentricity        e  =  $e
                    e&#178; =  $e2

Equatorial Quarter = $QECircum m  =  $QECircumMi mi
Meridional Quarter = $QPCircum m  =  $QPCircumMi mi

Circumference at Equator   =  $EquatorCircum m  =  $EquatorCircumMi mi
Circumference of Meridian  =  $PolarCircum m  =  $PolarCircumMi mi
Circumference Difference   =     $CircumDiff m  =     $CircumDiffMi mi

__________________________________________________________________
Location Coordinates and Sea Level Elevation of Point

Latitude    = $LatDMS $SN  =  $LatDeg&deg;
Longitude   = $LonDMS $WE  =  $LonDeg&deg;

Height/Elev = $H m  = $Hft ft  = $Hftin

__________________________________________________________________
Corresponding Rectangular 3D XYZ-Coordinates of Point at Given
Location and Elevation.

x = $x m  = $xmi mi
y = $y m  = $ymi mi
z = $z m  = $zmi mi
_________________________________________________________________
";



// _______________________________________________________
// DETERMINE HOW MANY ROWS TO USE FOR THE OUTPUT TEXTAREA.
   $TextAreaRows = 2 + substr_count($OutputTextArea, "\n");




print <<< _HTML


<!doctype html>

<html lang='en'>

<head>

<title>Historical Spheroid 3D XYZ-Coordinates Calculator</title>
<meta http-equiv="content-type" content="text/html; charset=UTF-8">
<meta name='viewport' content='width=device-width, initial-scale=1'>
<meta http-equiv='content-type' content='text/html; charset=UTF-8'>
<meta name='description' content='This program computes the rectangular XYZ-coordinates of any point on the surface of an historical spheroid based on the spheroid model parameters and the coordinates and elevation of the point.'>
<meta name='keywords' content='IERS 2003,WGS 84,WGS 72,SouthAmerican 1969,Everest 1830 (1967 Definition),Everest 1830 (Modified 1967),Australian National 1966,Krassovsky,International 1924 | Hayford 1910,Clarke 1880,Clarke 1878,Clarke 1866,Clarke 1858,Bessel 1841,Airy 1830,Everest 1830,Plessis 1817'>
<meta name='author' content='Jay Tanner - PHPScienceLabs.com'>
<meta http-equiv='pragma'  content='no-cache'>
<meta http-equiv='expires' content='-1'>
<meta name='robots'    content='index,follow'>
<meta name='googlebot' content='index,follow'>
<link rel="canonical" href="https://www.phpsciencelabs.com/earth-historical-oblate-spheroid-parameters-calculator/">

<style type='text/css'>
BODY
{color:silver; background-color:black;  font-family:Verdana; font-size:15px}

TABLE
{font-size:12px; border: 1px solid black;}

TD
{color:black; background-color:white;  line-height:150%; font-size:12px; padding:6px; text-align:center;}

DIV
{color:black; background-color:white;  font-family:Verdana; text-align:justify; padding:4px; font-size:16px;}

IMG
{border:1px solid gray; border-radius:8px; box-shadow:4px 4px 3px #888888;}

PRE
{color:black; background-color:white;  font-family:monospace; font-size:14px; font-weight:bold; text-align:left; padding:4px;}

CODE
{font-family:monospace; font-size:15px; font-weight:normal;}

TEXTAREA
{color:black; background:white; font-family:monospace; text-align:left; font-size:18px; font-weight:bold; box-shadow:4px 4px 3px #888888; border-radius: 8px; border:1px solid black; padding:8px; white-space:pre;}

INPUT[type="text"]::-ms-clear {width:0; height:0;}

INPUT[type="text"]
{color:black; background-color:white; font-size:18px; font-family:monospace; font-weight:bold; text-align:center; box-shadow:2px 2px 3px #888888; border:2px solid black; border-radius:4px;}

INPUT[type="text"]:focus
{background-color:white; box-shadow:2px 2px 3px #666666; font-family:monospace; font-size:18px; border:2px solid red; text-align:center; font-weight:bold; border-radius:4px;}


INPUT[type="submit"]
{color:black; background:#BBBBBB; font-size:100%; font-weight:bold; border:1px solid black; border-radius:4px;}

INPUT[type="submit"]:hover
{color:black; background:cyan; font-size:100%; font-weight:bold; box-shadow:2px 2px 5px #222222; border:1px solid black; border-radius:4px;}


INPUT[type="checkbox"]:checked
{color:black; background:#D8D8D8; box-shadow:2px 2px 3px #222222;}

HR {background:black; height:2px; border:0px;}

A:link,  A:visited {background-color:transparent; color:blue; font-family:Verdana; font-size:12px; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}
A:hover, A:active  {background-color:transparent; color:red;  font-family:Verdana; font-size:12px; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}

::selection      {background-color:yellow; color:black;}  /* Only affects IE9+ and Safari     */
::-moz-selection {background-color:yellow; color:black;}  /* Only affects FireFox and Mozilla */
</style>

<script type="text/x-mathjax-config">MathJax.Hub.Config({tex2jax: {inlineMath: [['$','$'], ['\\(','\\)']], displayMath: [['\\[','\\]'], ['$$','$$']]}});</script>
<script type="text/javascript" src="../MathJax/MathJax.js?config=TeX-AMS-MML_HTMLorMML"></script>

</head>

<body>

<form name="EllipsoidForm" method="post" action="$_RUN_">

<table width="715" border="0" cellspacing="1" cellpadding="3">

<tr><td colspan="2" style='color:white; background-color:#000066; border:2px solid white; border-radius:8px 8px 0px 0px;'><b style='font-size:18px; line-height:150%;'>Historical Oblate Spheroid Parameters Calculator</b><br><b>Given the Geographic Coordinates and Elevation of Any Point Relative<br>to the Surface of Any of Several Historical Reference Spheroid Models<br></b></td></tr>

<!--
<tr><td colspan="2">&nbsp;</td></tr>
-->

<tr>
<td width="193"><b style='font-size:16px;'>Latitude</b>&nbsp;&nbsp;<z>(</z>&minus;Neg = South<z>)</z>&nbsp;&nbsp; <span style='font-size:18px;'>,</span>&nbsp; <b style='font-size:16px;'>Longitude</b>&nbsp;&nbsp;<z>(</z>&minus;Neg = West<z>)</z><br>
±Deg.dddddddddd
&nbsp; <b>or</b> &nbsp; &plusmn;Deg &nbsp;min&nbsp; sec.ssss
<input name="LatLon" type="text" value="$LatLon" size="39" maxlength="38"></td>
<td width="130"><b style='font-size:14px;'>Sea-Level Elevation</b>&nbsp;<br>in meters or feet (<b>m</b>, &nbsp; <b>ft</b>)
<input name="Elevation" type="text" value="$elevation" size="17" maxlength="16"></td>
</tr>

<tr><td colspan="2">Select a spheroid model to compute the rectangular <b>3D</b> &nbsp;<b>XYZ</b>-coordinates of the point based<br>on the geographic coordinates and elevation given above and the equations following below.</td></tr>

<tr>
<td colspan='2' style='line-height:300%; background-color:#002200;'>
<input name='WGS84Button'                        type='submit' value='GRS 80 | WGS 84' title='Global ITRS International Terrestrial Reference System | GPS Global Positioning System'>
<input name='IERS2003Button'                     type='submit' value='IERS 2003' title='International Earth Rotation and Reference Systems'>
<input name='WGS72Button'                        type='submit' value='WGS 72' title='USA Department of Defense'>
<input name='SouthAmerican1969Button'            type='submit' value='South American 1969' title='Same as Australian National 1966'>
<input name='Everest18301967DefinitionButton'    type='submit' value='Everest 1830 (1967 Definition)' title='Brunei and East Malaysia'>
<input name='Everest1830Modified1967Button'      type='submit' value='Everest 1830 (Modified 1967)' title='West Malaysia and Singapore'>
<br>
<input name='AustralianNational1966Button'       type='submit' value='Australian National 1966' title='Same as South American 1969'>
<input name='Krassovsky1940Button'               type='submit' value='Krassovsky 1940' title='USSR, Russia, Romania'>
<input name='International1924Hayford1910Button' type='submit' value='International 1924 | Hayford 1910' title ='International 1924 | Hayford 1910'>
<br>
<input name='Clarke1880Button'                   type='submit' value='Clarke 1880' title='France, Africa'>
<input name='Clarke1878Button'                   type='submit' value='Clarke 1878' title='North America'>
<input name='Clarke1866Button'                   type='submit' value='Clarke 1866' title='North America'>
<input name='Clarke1858Button'                   type='submit' value='Clarke 1858' title='Great Britain'>
<br>
<input name='Bessel1841Button'                   type='submit' value='Bessel 1841' title='Europe and Japan'>
<input name='Airy1830Button'                     type='submit' value='Airy 1830' title='Great Britain'>
<input name='Airy1830ModifiedButton'             type='submit' value='Airy 1830 (Modified)' title='Great Britain'>
<input name='Everest1830Button'                  type='submit' value='Everest 1830' title='India'>
<input name='Plessis1817Button'                  type='submit' value='Plessis 1817' title='France'>

<!--
<input name='Maupertuis1738Button'               type='submit' value='Maupertuis 1738' title='France'>
-->

</td>
</tr>

<tr><td colspan="2">&nbsp;</td></tr>

<tr><td colspan="2" style='background-color:GreenYellow; padding:16px;'>
Double-Click Within Text Area to Select ALL Text<br>
<textarea name="OutputTextArea" cols="70" rows="$TextAreaRows" ReadOnly OnDblClick='this.select();' OnMouseUp='return true;'>$OutputTextArea</textarea>
</td></tr>

<!-- Begin table within a table -->
<!-- <tr><td colsspan='2' align='center'>$ParallelCirclesTable</td></tr> -->
<!-- End table within a table -->

<tr><td colspan="2" style='font-size:14px; text-align:justify;'>

<b>BASIC ELLIPSOID FORMULAS AND RELATIONSHIPS APPLIED HERE</b>
<br><br>

<img src='images/basic_ellipsoid_diagram.png' align='bottom' alt=''>

<br><br>
<b>Given the general static ellipsoid parameters:</b><br>
<ul>
<li><b>R</b> = Equatorial radius of ellipsoid = Length of semi-major axis<br></li>
<li><b>r</b> = Polar radius of ellipsoid = Length of semi-minor axis<br></li>
<li>where,&nbsp; <b>r</b> &le; <b>R</b><br><br></li>
<li>If ((R = r)), then the form is a perfect sphere, otherwise it is an ellipsoid or oblate<br>spheroid to some degree, generally wider between two opposite equatorial points<br>
due to a slight equatorial bulge, than between the opposite polar points.<br><br></li>
<li><b style='color:red;'>NOTE:</b><br>
In the equations, any units of measure may be used for (<b>R</b>, <b>r</b>), such as feet, meters,<br>kilometers, miles, etc., just as long as the same units are used consistently throughout.<br>The values of (<b>x</b>, <b>y</b>, <b>z</b>) will be expressed in the same units as the parameters (<b>R</b>, <b>r</b>).</li>
</ul>

<br>
<hr>
<b>Computing 3D Rectangular XYZ-Coordinates For a Point on a Reference Ellipsoid Surface</b>
<br><br><br>
<b>The reference ellipsoid parameters, location and elevation of the point are</b>:
<br>
<i>R</i> = Equatorial radius.
<br>
<i>r</i> = Polar radius.
<br><br>
<i>Lat</i> = Geographic latitude of point on ellipsoid surface.<br>
<i>Lon</i> = Geographic longtitude of point on ellipsoid surface <z>(</z>Negative = West<z>)</z>.
<br><br>
<math><mi>h</mi></math> = Height or elevation of point relative to ellipsoid surface.

<br><br>

<div style='text-align:center; font-size:100%;'>
From the given parameters, the 3D XYZ-coordinates of the point, relative to the surface<br>of the reference ellipsoid, are computed from the equations given below.<br>
<img src='images/spheroid-equations-and-diagram.png'></div>


<br><br><br>

</td></tr>

<tr><td colspan='2' style='text-align:left;'><b>Reference:</b>
<br><br>
The Earth Ellipsoid</b><br>
<a href='https://en.wikipedia.org/wiki/Earth_ellipsoid' target='_blank'>https://en.wikipedia.org/wiki/Earth_ellipsoid</a>

<br><br>

</td></tr>

<tr><td colspan="2" style='font-size:10px;'>A PHP Science Program by Jay Tanner - Revised: $_RevisionDate_</td></tr>

</table>
</form>

</body>
</html>


_HTML;



/*

   This is a local utility function to parse the
   (LatLonString) argument string before using it.
*/

   function Parse_Lat_Lon_String($LatLonString)
{

// READ INPUT ARGUMENT STRING AND STRIP ALL WHITE SPACE.
   $LatLonStr = strip_wspace($LatLonString);

   $LatLonStr = str_replace(',', ' , ', $LatLonStr);

// IF EMPTY INPUT, THEN SET DEFAULT COORDINATES TO (0 , 0).
   if ($LatLonStr == '')
      {return '0.0000000000 , 0.0000000000';}


// COUNT ALL SPACES.  AFTER STRIPPING OUT ALL WHITE SPACE, A
// SINGLE SPACE REMAINING INDICATES TWO NUMERICAL VALUES.
   if (substr_count($LatLonStr, ' ') == 1)
      {$LatLonStr = str_replace(' ', ' , ', $LatLonStr); return $LatLonStr;}

// AT THIS POINT, ARGUMENT STRING MUST BE A SINGLE VALUE (LATITUDE ONLY).
// SO, SET LONGITUDE VALUE TO (0).
   $LatLonStr = "$LatLonStr , 0.0000000000";

// SEPARATE THE LATITUDE AND LONGITUDE VALUES AT THE COMMA.
   list($lat, $lon) = preg_split("[,]", $LatLonStr);
   $lat = trim($lat);
   $lon = trim($lon);

// CONVERT LATITUDE AND LONGITUDE INTO SIGNED DECIMAL VALUES.
   $LatDeg = DMS_to_Deg($lat);
   $LonDeg = DMS_to_Deg($lon);
   $LatDeg = sprintf("%+1.10f", $LatDeg);
   $LonDeg = sprintf("%+1.10f", $LonDeg);

// DONE.
   return "$LatDeg , $LonDeg";
}









/*
===========================================================================

NEW PHP SCIENCE LABS INTERNAL UNIVERSAL CORE MODULE

TOTAL NEW GENERATION REMAKE DESIGNED FOR FUTURE EVOLUTION.

FOR TRANSPORTABILITY, COPY UP TO TERMINUS LINE.

START DATE: 2017 Mar 03 Fri

Author    : Jay Tanner - 2017
Language  : PHP v5.6.x - v7.1.8
License   : The PHP source code is released under the provisions of
            the Creative Commons Attribution-ShareAlike 3.0 license.
            https://creativecommons.org/licenses/by-sa/3.0/us/legalcode
===========================================================================
*/

// Define 78-character horizontal bars. These
// can be called globally as block separators.

   $_HR1_ = str_repeat("&#183;", 78);
   $_HR2_ = str_repeat('-', 78);
   $_HR3_ = str_repeat('=', 78);
   $_HR4_ = str_repeat('*', 78);
   $_HR5_ = str_repeat("&#151;", 78);

   $_HR_ = array();

   for($i=1;   $i < 80;   $i++)
  {
   $chars = str_repeat("&#151;", $i);
   $_HR_[$i] = $chars;
  }


// CONSTANTS
//
// ------------------------------------
// Speed of light in meters per second.
// Taken from NASA/JPL DE430 constants.

// ---------------------------------------------
// Due to the way the length of a second and the
// length of the meter are EXACTLY defined, this
// speed of light is also an EXACT definition.

   define ("CLIGHT", "299792458");

// -----------------------------------------------
// Standard Earth gravity (1 G) acceleration (SI).

   define ("G_STD", "9.80665");

// --------------------------------
// Universal gravity constant (SI).

   define ("G_UNI", "0.000000000066742");

// -------------------------------------------------------------------
// Define some EXACT standard metric length/distance interconversion
// factors in terms of meters per unit (M_PER_XXX).  This is the
// factor by which to multiply the units to obtain equivalent meters.

   define ("M_PER_A"   , "0.0000000001");    // m per angstrom
   define ("M_PER_NM"  , "0.000000001");     // m per nanometer
   define ("M_PER_UM"  , "0.000001");        // m per micrometer (micron)
   define ("M_PER_MM"  , "0.001");           // m per millimeter
   define ("M_PER_CM"  , "0.01");            // m per centimeter
   define ("M_PER_IN"  , "0.0254");          // m per inch
   define ("M_PER_FT"  , "0.3048");          // m per foot
   define ("M_PER_YD"  , "0.9144");          // m per yard
   define ("M_PER_M"   , "1");               // m per meter
   define ("M_PER_KM"  , "1000");            // m per kilometer
   define ("M_PER_MI"  , "1609.344");        // m per mile (statute)
   define ("M_PER_NMI" , "1852");            // m per nautical mile
   define ("M_PER_LS"  , "299792458");       // m per Light Second
   define ("M_PER_LM"  , "17987547480");     // Light Minute
   define ("M_PER_LH"  , "1079252848800");   // Light Hour
   define ("M_PER_LD"  , "25902068371200");  // Light Day

   define ("KM_PER_NMI", "1.852");           // km per nautical mile
   define ("KM_PER_MI" , "1.609344");        // kilometers per statute mile

// -----------------------------
// Define the Astronomical Unit.

   define ("M_PER_AU" ,  "149597870700");    // NASA/IAU/DE430 as of 2012
   define ("KM_PER_AU",  "149597870.700");
   define ("MI_PER_AU",  "92955807.273");

// -----------------------------------------------------
// Derived factors based on the above adopted standards.
// The AU used is the IAU value and the year value used
// is the standard Julian year of exactly 365.25 days.

   define ("M_PER_LY"  , "9460730472580800"); // EXACT
   define ("M_PER_PC"  , "30856775813057289.532915512381557");
   define ("AU_PER_LY" , "63241.0770880709446742");
   define ("AU_PER_PC" , "206264.8062470963551565");
   define ("LY_PER_PC" , "3.2615637769712137");

   define ("LT_1AU"    , "0.00577551830441213");

// ----------------------------------
// DEFINE SOME WEIGHT/MASS STANDARDS.

   define ("KG_PER_OZ" , "0.028349523125"); // EXACT
   define ("KG_PER_LB" , "0.45959237");     // EXACT
   define ("G_PER_OZ"  , "28.349523125");   // EXACT

   define ("N_PER_LB"  , "4.4482216152605");

// ------------------------------------------
// LENGTH OF JULIAN CALENDAR YEAR IN SECONDS.
// EXACTLY 365.25 DAYS.

   define ("S_PER_JYR", "31557600");

// ----------------------------------
// DEFINE NUMBER OF SECONDS IN 1 DAY.

   define ("S_PER_DAY" , "86400");

// ---------------------------------------------
// LENGTH OF GREGORIAN CALENDAR YEAR IN SECONDS.
// EXACTLY 365.2425 DAYS.

   define ("S_PER_GYR", "31556952");

// ---------------------------
// Define Planck constant (h).
// 6.62606896E-34

   define ("PLANCK_h", "0.000000000000000000000000000000000662606896"); //  J/s

// -----------------------------------------
// Define Stefan-Boltzmann constant (sigma).
//
// Radiated energy = sigma * K**4 Watts per sq. meter per second
// 5.670373E-8
// Where K = temperature in kelvins

   define ("STEF_BOLT_sigma", "0.00000005670373"); // Watts/sq. meter/k^4

// ------------------------------
// Define Boltzmann constant (k).
//
// k = Joules / deg Kelvin

   define ("BOLTZ_k", "0.000000000000000000000013806504"); //  J/s/K = Watts/K

// __________________________________________________________________
// DEFINE SPECIAL PI-RELATED CONSTANTS TO 110 DECIMALS (NOT ROUNDED).
// THESE ARE FOR ARBITRARY-PRECISION USE.
   define ("PI", "3.141592653589793238462643383279502884197169399375105820974944592307816406286208998628034825342117067982148");

   define ("PI_OVER_2", "1.57079632679489661923132169163975144209858469968755291048747229615390820314310449931401741267105853399107404325");

   define ("PIX2", "6.28318530717958647692528676655900576839433879875021164194988918461563281257241799725606965068423413596429617302");

   define ("PI2", "9.86960440108935861883449099987615113531369940724079062641334937622004482241920524300177340371855223182402591377");

   define ("RAD_PER_DEG", "0.01745329251994329576923690768488612713442871888541725456097191440171009114603449443682241569634509482212304492");

   define ("DEG_PER_RAD", "57.29577951308232087679815481410517033240547246656432154916024386120284714832155263244096899585111094418622338163");

   define ("SQRT_PI", "1.77245385090551602729816748334114518279754945612238712821380778985291128459103218137495065673854466541622682362");

   define ("LN_10", "2.30258509299404568401799145468436420760110148862877297603332790096757260967735248023599720508959829834196778403");
/*
   ____________________________________________________________________
   DEFINE THE AVAILABLE SOLAR SYSTEM BODY NAMES. THESE DEFINITIONS WILL
   BE GLOBAL TO ALL FUNCTION MODULES.  ALL THE NAMES ARE RIGHT-PADDED
   WITH WHITE SPACE TO A UNIFORM WIDTH OF 10 CHARACTERS.
*/
   define ("BODY_NAMES", "Sun       Moon      Mercury   Venus     Mars      Jupiter   Saturn    Uranus    Neptune   Pluto");
   define ("ASTEROID_NAMES", "Ceres     Cybele    Davida    Eunomia   Europa    Flora     Hebe      Hygiea    InteramniaIris      Juno      Metis     Pallas    Psyche    Vesta");

// ___________________________________________________________________
// DEFINE THE NAMES OF MONTHS AND DAYS OF THE WEEK.  ALL THE NAMES ARE
// RIGHT-PADDED WITH WHITE SPACE TO A UNIFORM WIDTH OF 10 CHARACTERS.

   define ("MONTH_NAMES", "January   February  March     April     May       June      July      August    September October   November  December");
   define ("DOW_NAMES", "Sunday    Monday    Tuesday   Wednesday Thursday  Friday    Saturday");



// FUNCTIONS
// -------------------------------------------------------------------------
// Strips all leading, trailing and internal white space from within a text
// string.  All occurrences of multiple spaces within the text string will
// be replaced by single spaces and all leading and trailing white space
// and NewLine end-codes will also be removed.
//
// This helps facilitate reading sequential space-delimited items from
// a given text-data string by normalizing all spacing between words
// to a single space, for easier use with the preg_split() function.

   function Strip_WSpace ($TextString)
{
   return preg_replace("/\s+/", " ", trim($TextString));
}




/*
   -------------------------------------------------------------------------
   This function simply swaps the contents
   of two simple variable arguments.
*/

   function Swap (&$var1, &$var2)
 {
   $w=$var1;  $var1=$var2;  $var2=$w;
 }



/*
   This boolean function checks if a text string
   is NOT purely numeric.   Leading and trailing
   spaces in the text string are ignored.

   Returns:
   TRUE  = Non-numeric
   FALSE = Purely numeric string
*/

   function Is_Not_Numeric ($TextString)
{
   return (!is_numeric(trim($TextString)))? TRUE:FALSE;
}



/*
   =====================================================================
   This boolean function indicates whether or not
   the given character is a vowel.  It only reads
   the first character of the argument string.
   This function treats 'Y' as a vowel.

   Returns:
   TRUE  = Character is a vowel.
   FALSE = Character is NOT a vowel.
*/
   function Is_Vowel ($char)
{
   $x = strtoupper(substr($char, 0,1));
   return (strpos('AEIOUY', $x) !== FALSE)? TRUE:FALSE;
}







/*
   =====================================================================
   This boolean function indicates whether or not the
   given character is a consonant.  It only reads the
   first character of the argument string.

   Returns:
   TRUE  = Craracter is a consonant
   FALSE = Character is NOT a consonant.
*/
   function Is_Consonant ($char)
{
   $x = strtoupper(substr($char, 0,1));
   return (strpos('BCDFGHJKLMNPQRSTVWXZ', $x) !== FALSE)? TRUE:FALSE;

}





/*
   =====================================================================
   This boolean function indicates whether or not the
   the given character is a numerical digit.  It only
   reads the first character of the argument string.

   Returns:
   TRUE  = A character is a digit
   FALSE = Character is NOT a digit.
*/
   function Is_Digit($TextChar)
{
   $OfDigit = trim(substr(trim($TextChar), 0,1));

   return (strpos('0123456789', $OfDigit) !== FALSE)? TRUE:FALSE;
}


/*
   =====================================================================
   This boolean function indicates whether or not the given character is
   a letter of the English alphabet.  It only reads the first character
   of the argument string.

   Returns:
   TRUE  = A character is a letter (a-z or A-Z)
   FALSE = Character is NOT a digit.
*/
   function Is_Letter($TextChar)
{
   $OfLetter = trim(substr(trim($TextChar), 0,1));

   if ($OfLetter == '') {return FALSE;}

   return (strpos('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ', $OfLetter) !== FALSE)? TRUE:FALSE;
}

/*
   =====================================================================
   This boolean function indicates whether or not the given character is
   a symbol character.   A symbol character is any character that is not
   a digit or a letter of the English alphabet.  It only reads the first
   character of the argument string.

   This function excludes the escape symbol (backslash \) as a recognized
   symbol character.  A backslash must NOT be used in the argument.

   Returns:
   TRUE  = Character is NEITHER a letter (a-z or A-Z) nor a digit (0-9)
   FALSE = Character is either a letter or a digit.
*/
   function Is_Symbol($TextChar)
{
   $OfSymbol = trim(substr(trim($TextChar), 0,1));

   if ($OfSymbol == '') {return FALSE;}

   return (strpos(htmlentities('~!@#$%^&*()-_=+[{]}|;:"'."',<.>/?"), $OfSymbol) !== FALSE)? TRUE:FALSE;
}






/*
   Sort an alpha or numeric table given as a text string argument
   delimited by NewLine codes.

   Numerical sort is in ascending order from the left and assumes that
   all strings begin with a number and end with a NewLine (\n) code.

   Otherwise, it performs a simple alphabetic text line sort.
*/

   function Sort_Table ($TableTextString)
{
   $wArray = preg_split("[\n]", RTrim($TableTextString, "\n"));
   $wCount = count($wArray);

// Sort table numerically from left.
   sort($wArray);

// Reconstruct and return table as text string.
   $w = '';
   for ($i=0;   $i < $wCount;   $i++)
   {
   $CurrLine = RTrim($wArray[$i], "\n");
   $w .= "$CurrLine\n";
   }
   return RTrim($w, "\n");

} // End of  Sort_Table(...)








/*
   ------------------------------------------------------------------
   Pads a text string with any padding character (default = space) on
   the left or right end to make XPadCount chracters in total length.
*/
   FUNCTION LR_X_Pad ($TextString, $LorRStr='L', $XPadChar=' ', $XPadCount=0)
{
   $wLorRStr   = (strtoupper(trim($LorRStr)) == 'L')? 'L':'R';
   $wTextStr   = substr($TextString, 0, $XPadCount);
   $wXPadChar  = substr($XPadChar, 0,1);
   $wXPadCount = intval($XPadCount);

   if ($wLorRStr == 'L')
  {
   $w = str_repeat($wXPadChar, $wXPadCount) . $wTextStr;
   $w = substr($w, -$wXPadCount);
  }
   else
  {
   $w = substr($wTextStr, 0, $wXPadCount) . str_repeat($wXPadChar, $wXPadCount);
   $w = substr($w, 0, $wXPadCount);
  }

   return $w;
}







/*
   This function rotates a plain text string by ±n characters.

   -N = Rotate N characters left
   +N = Rotate N characters right
*/


   function str_rotate($TextString, $RotNumChars=1)

   {
   $TextStr = str_replace("\n", '', $TextString);

   $L = strlen($TextStr);
   $N = intval($RotNumChars);  if ($N == 0){return $TextString;}
   $n = abs($N) % $L;

   if ($N < 0)
  {

// Rotate left by (-n) characters.
   $LeftStr  = substr($TextStr, 0, $n);
   $RightStr = substr($TextStr, $n, $L);
   return $RightStr.$LeftStr;
  }

// Rotate right by (+n) characters.
   $LeftStr  = substr($TextStr, -$n);
   $RightStr = substr($TextStr, 0, $L - $n);
   return $LeftStr.$RightStr;

} // End of  str_rotate(...)



/*
   These two functions padd a string on the left or right ends
   with any chosen padding character (default = space).


   TextString = String to be padded
   PadLen     = Padding length

   PadChar    = Padding character to use (default = space).
                For numbers, a zero can be used instead of a space.

*/
   FUNCTION L_Pad_Text ($TextString, $PadLen=0, $PadChar=' ')
{
   return substr(str_repeat(substr($PadChar, 0,1), $PadLen).$TextString, -$PadLen);
}


   function R_Pad_Text ($TextString, $PadLen=0, $PadChar=' ')
{
   return $TextString.substr(str_repeat(substr($PadChar, 0,1), $PadLen), 0,$PadLen);
}




   function Rand_Letters($LettersCount)
{
   $letters = "abcdefghijklmnopqrstuvwzyzABCDEFGHIJKLMNOPQRSTUVWZYZ";

   $n = intval(trim($LettersCount));

   $RandLetters = '';

   for($i=0;   $i < $n;   $i++)
  {
   $RandLetters .= substr($letters, mt_rand(0, strlen($letters))-1, 1);
  }
   return $RandLetters;
}





   function Rand_Digits($DigitsCount)
{
   $digits = "0123456789";

   $n = intval(trim($DigitsCount));

   $RandDigits = '';

   for($i=0;   $i < $n;   $i++)
  {
   $RandDigits .= substr($digits, mt_rand(0, strlen($digits)-1), 1);
  }
   return $RandDigits;
}





   function Rand_Text_String($StringLength)
{
   $chars = "abcdefghijkmnpqrstuvwzyzABCDEFGHIJKLMNPQRSTUVWZYZ!@#$%^&*)2345?6789(+-=][}{><./";

   $n = intval(trim($StringLength));

   $RandTextString = '';

   for($i=0;   $i < $n;   $i++)
  {
   $RandTextString .= HTMLEntities(substr($chars, mt_rand(0, strlen($chars)-1), 1));
  }
   return $RandTextString;
}



   function Rand_Symbols($StringLength)
{
   $chars = "!@#%^*?(+-=[{><./";

   $n = intval(trim($StringLength));

   $RandSymbolsString = '';

   for($i=0;   $i < $n;   $i++)
  {
   $RandSymbolsString .= substr($chars, mt_rand(0, strlen($chars)-1), 1);
  }
   return $RandSymbolsString;
}














/*
   ========================================================================
   Generates a string of any number of random digits in any given base
   from 2 to 36.

   All letters (A to Z) used as digits will be returned in uppercase.

   ERRORS:
   On error, FALSE is returned for invalid argument(s).
   ========================================================================
*/

   function Rand_Digits_Base_B ($NumDigits=1, $BaseB=10, $LZeroSuppressed=FALSE)
{

// Read arguments.
   $N = intval($NumDigits);
   $B = intval($BaseB);

// Check for invalid argument(s).
   if ($B < 2 or $B > 36 or $N < 1) {return FALSE;}

// Define digits spectrum for the given base.
   $DigitsSpectrum = substr("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ",0,$B);

// Construct an N-digit random sequence in base B, one digit at
// a time, using a random 50/50 digit alternation method. There
// will be at least one random base B digit returned.
   $RandomDigits = "";

   for ($i=0;   $i < $N;   $i++)
  {
   if (mt_rand() % 2 == 0)
  {$RandomDigits .= substr($DigitsSpectrum, mt_rand(0,$B-1),1);}
   else
  {$RandomDigits = substr($DigitsSpectrum, mt_rand(0,$B-1),1) . $RandomDigits;}
  }

// If LZeroSuppressed flag === TRUE, then leftmost digit of returned
// random digits string must NOT be zero.  If it is, then another
// random non-zero digit will be attached to the left of it and
// the rightmost digit dropped.  The result will be a normal
// integer value with the required number of digits, but
// never starting with 0 (zero) as the first digit.
   if ($LZeroSuppressed === TRUE)
  {
   $RandomDigits = substr($DigitsSpectrum, mt_rand(1,$B-1),1) . $RandomDigits;
  }
   return substr("$RandomDigits", 0, $N);

} // End of  Rand_Digits_Base_B(...)




   function Rand_yyyymndn_Date()
{
   list ($m, $d, $yyyy) =
   preg_split("[\/]", JDtoGregorian(mt_rand(2305448, 2524958)));
   $mn = sprintf("%02d", $m);
   $dn = sprintf("%02d", $d);
   return "$yyyy$mn$dn";

} // End of  Rand_Int_Enc_Date()



   function Rand_Date_String()
  {
   $w = trim(JD_Z_to_DT (mt_rand(2305448, 2524958), '+00:00', 2));
   $DateStr = substr($w, 0,11);
   $DoWStr  = substr($w, -3);

   return "$DateStr  $DoWStr";

  } // End of  Ran_Date_String()



   function Rand_Ymndn()
{
   list ($m, $d, $yyyy) =
   preg_split("[\/]", JDtoGregorian(mt_rand(2305448, 2524958)));
   $mn = sprintf("%02d", $m);
   $dn = sprintf("%02d", $d);
   return "$yyyy$mn$dn";

}





   function Rand_hhmmss()
  {
   $hh = sprintf("%02d", mt_rand(0, 23));
   $mm = sprintf("%02d", mt_rand(0, 59));
   $ss = sprintf("%02d", mt_rand(0, 59));

   return "$hh:$mm:$ss";

   }




   function Rand_Date_Time_String()
  {
   $RandDateStr = Rand_Date_String();
   $DoWStr      = substr($RandDateStr, -3);
   $RandDateStr = substr($RandDateStr, 0,11);
   $RandTimeStr = Rand_Time_String();

   return "$RandDateStr  $DoWStr  $RandTimeStr";

  } // End of  Rand_Date_Time_String()








/*
  This function simply generates a random negative or positive
  fractional value of any number of given number of decimals.
  The returned value will always be between ±1 exclusive.

  Sometimes a function that simply returns a signed, random
  decimal value can be very useful.

  When used, the numerical sign is 50:50 or equally probable.

  This function reinstates the MT_Rand() function rather than
  using the new PHP 7 cryptographic randomization functions.

  Cryptographic randomization really isn't needed for simple
  things like randomization within a game.

  The MT_Rand() function is sufficient here.
*/

   function Rand_Decimal($Digits=1, $RandSignFlag=FALSE)
  {
   $DigitsStr = $RandSign = '';
   $NumDigits = floor($Digits);
   $RandSFlag = ($RandSignFlag === TRUE)? TRUE:FALSE;

   if ($RandSFlag === TRUE) {$RandSign = (mt_rand(0, 100) % 2 == 0)? '-':'+';}

   for($i=0;   $i < $NumDigits;   $i++) {$DigitsStr .= mt_rand(0, 9);}

   return $RandSign . '0.' . $DigitsStr;

  } // End of  Rand_Decimal()












// _______________________________________________
// Double-precision basic trigonometric functions
// for degree aguments rather than radians.

   function Sind ($AngDMS)
  {
   return Sin (deg2rad(DMS_to_Deg($AngDMS)));
  }

   function Cosd ($AngDMS)
  {
   return cos (deg2rad(DMS_to_Deg($AngDMS)));
  }

   function Tand ($AngDMS)
  {
   return Tan(deg2rad(DMS_to_Deg($AngDMS)));
  }

   function ASind ($x)
  {
   return rad2deg(asin(floatval($x)));
  }

   function ACosd ($x)
  {
   return rad2deg(acos(floatval($x)));
  }

   function ATand ($x)
  {
   return rad2deg(atan(floatval($x)));
  }

   function ATand2 ($y, $x)
  {
   $w = rad2deg(atan2(floatval($y), floatval($x)));

   return $w + (($w < 0)? 360:0); // 0 to +360 degrees
  }



/*
============================================================
TEMPERATURE SCALE INTERCONVERSION FUNCTION.

This function converts from any given temperature
scale into the equivalent temperature on any one
of four other recognized (F,C,K,R) scales.

The scale symbols are:
F = Fahrenheit
C = Celsius
K = Kelvin  = Degrees Celsius above absolute zero.
R = Rankine = Degrees Fahrenheit above absolute zero.

The scale symbols are NOT case sensitive.

The conversion is performed by first converting the
input temperature value into its equivalent on the K
scale, then the K scale value is converted into the
given output scale.

-------
EXAMPLE
To convert from 123.45 C into its equivalent
on the R scale:

print Temp_To_Temp("-123.45 C", "R"); // Result = 269.46


------
ERRORS
An error returns FALSE if a scale symbol is not recognized.
============================================================
*/


   function Temp_to_Temp ($FromTempStr, $ToScaleStr)
{
// -------------------------------------
// Read input argument value and scales.

   $w = trim($FromTempStr);   if ($w == '') {$w = "0";}
   $T = floatval($w);

   $FromScale = strtoupper(trim(substr($w, -1)));
   $ToScale = strtoupper(trim($ToScaleStr));

// ---------------------------------------------------
// Convert input ($FromScale) into K scale equivalent.

   switch ($FromScale)
  {
   case "F" : $K = (5 * ($T - 32) / 9) + 273.15; break;
   case "C" : $K = $T + 273.15; break;
   case "K" : $K = $T; break;
   case "R" : $K = (5 * ($T - 491.67) / 9) + 273.15; break;
   default  : $K = FALSE; break; // On ERROR
  }

// --------------------------
// ERROR if invalid FromScale

   if ($K === FALSE) {return $K;}

// --------------------------------------------------
// Convert K value into output ($ToScale) equivalent.

   switch (strtoupper(trim($ToScale)))
  {
   case "F" : $T = (9 * ($K - 273.15) / 5) + 32; break;
   case "C" : $T = $K - 273.15; break;
   case "K" : $T = $K; break;
   case "R" : $T = (9 * ($K - 273.15) / 5) + 491.67; break;
   default  : $T = FALSE; break;
  }

   return $T;

} // End of Temp_To_Temp(...)




/*
 ===========================================================================
 This function interconverts between any of eighteen length or
 distance units.  The units symbols are NOT case sensitive.

 ----------
 ARGUMENTS:
 $LengthAndUnits = Numerical value and units symbol of length to convert.
                   NOTE: There MUST be at least one space between the
                   number value and the units symbol in the string.
                   The unit symbols are NOT case sensitive.

 $ToUnits = The units symbol corresponding to the computed output value.

 -------
 ERRORS:
 On error FALSE is returned.
 An error occurs if the length value is non-numeric or either
 of the input or output length units symbols are unrecognized.

 ------------------------------------------------------------
 THE 18 LENGTH UNITS SYMBOLS RECOGNIZED BY THIS FUNCTION ARE:

 SYMBOL      UNITS

   A         Angstroms
   nm        Nanometers
 um or u     Micrometers (or Microns)
   mm        Millimeters
   cm        Centimeters
   in        Inches
   ft        Feet
   yd        Yards
   m         Meters
   km        Kilometers
   mi        Miles (statute)
   nmi       Nautical miles
   LS        Light Seconds
   LM        Light Minutes
   LH        Light Hours
   AU        Astronomical Units
   LY        Light Years (Based on Julian year of 365.25 days)
   pc        Parsecs

   ===========================================================================
*/

   function Leng_to_Leng ($LengthAndUnits, $ToUnits)
{
// Define conversion constants for internal use only.
// These constants could also be defined externally
// for universal use by any programs that need them.
//
// This function was designed to be transportable
// and fully self-contained, constants and all.

   $M_PER_A   = '0.0000000001';
   $M_PER_NM  = '0.000000001';
   $M_PER_UM  = '0.000001';
   $M_PER_MM  = '0.001';
   $M_PER_CM  = '0.01';
   $M_PER_IN  = '0.0254';
   $M_PER_FT  = '0.3048';
   $M_PER_YD  = '0.9144';
   $M_PER_M   =  1;
   $M_PER_KM  = '1000';
   $M_PER_MI  = '1609.344';
   $M_PER_NMI = '1852';
   $CLIGHT    = '299792458';        // Light Second
   $M_PER_LM  = '17987547480';      // Light Minute
   $M_PER_AU  = '149597870700';
   $M_PER_LH  = '1079252848800';    // Light Hour
   $M_PER_LY  = '9460730472580800'; // Light Year
   $M_PER_PC  = '30856775876793114.6633';

// Parse input value and separate the numerical value from the symbol.
// There MUST be at least one space between the number and the symbol.
// FALSE is returned on error if no space character detected at all or
// the given numeric value is not a valid number.
   $w = trim($LengthAndUnits); if (strpos($w, " ") === FALSE){return FALSE;}

   list($x, $FromUnits) = preg_split("[ ]", preg_replace("/\s+/", ' ', $w));

   if (!is_numeric($x)) {return FALSE;}

// Get input units symbol.  Unit symbols are NOT case sensitive.
   $FromUnitsSymbol = strtoupper(trim($FromUnits));

// Determine which conversion factor to use
// according to the units symbol argument.

   switch ($FromUnitsSymbol)
  {
   case "A"  : $f = $M_PER_A;   break;
   case "NM" : $f = $M_PER_NM;  break;
   case "UM" : $f = $M_PER_UM;  break;
   case "U"  : $f = $M_PER_UM;  break;
   case "MM" : $f = $M_PER_MM;  break;
   case "CM" : $f = $M_PER_CM;  break;
   case "IN" : $f = $M_PER_IN;  break;
   case "FT" : $f = $M_PER_FT;  break;
   case "YD" : $f = $M_PER_YD;  break;
   case "M"  : $f = $M_PER_M;   break;
   case "KM" : $f = $M_PER_KM;  break;
   case "MI" : $f = $M_PER_MI;  break;
   case "NMI": $f = $M_PER_NMI; break;
   case "LS" : $f = $CLIGHT;    break;
   case "LM" : $f = $M_PER_LM;  break;
   case "LH" : $f = $M_PER_LH;  break;
   case "AU" : $f = $M_PER_AU;  break;
   case "LY" : $f = $M_PER_LY;  break;
   case "PC" : $f = $M_PER_PC;  break;
   default   : $f = FALSE;      break;
  }

// Error if conversion factor is FALSE, which
// means that the units symbol was not found
// in the table of valid symbols.
   if ($f === FALSE) {return $f;}

// Otherwise, compute the equivalent meters value
// by multiplying the value of the argument by
// the conversion factor $f.
   $m = $x * $f;

// *********************************************************
// AT THIS POINT INPUT VALUE HAS BEEN CONVERTED INTO METERS.
// NEXT STEP IS TO CONVERT THE METERS INTO THE OUTPUT UNITS.
// *********************************************************

// Read the output units symbol argument.  The unit symbols
// are NOT case sensitive.
   $ToUnitsSymbol = strtoupper(trim($ToUnits));

// Determine the conversion factor to use from the units
// symbol argument.

   switch ($ToUnitsSymbol)
  {
   case "A"  : $f = $M_PER_A;   break;
   case "NM" : $f = $M_PER_NM;  break;
   case "UM" : $f = $M_PER_UM;  break;
   case "U"  : $f = $M_PER_UM;  break;
   case "MM" : $f = $M_PER_MM;  break;
   case "CM" : $f = $M_PER_CM;  break;
   case "IN" : $f = $M_PER_IN;  break;
   case "FT" : $f = $M_PER_FT;  break;
   case "YD" : $f = $M_PER_YD;  break;
   case "M"  : $f = $M_PER_M;   break;
   case "KM" : $f = $M_PER_KM;  break;
   case "MI" : $f = $M_PER_MI;  break;
   case "NMI": $f = $M_PER_NMI; break;
   case "LS" : $f = $CLIGHT;    break;
   case "AU" : $f = $M_PER_AU;  break;
   case "LY" : $f = $M_PER_LY;  break;
   case "PC" : $f = $M_PER_PC;  break;
   default   : $f = FALSE;      break;
  }

// Error if conversion factor is FALSE, which
// means that the units symbol was not found
// in the valid symbols table.
   if ($f == FALSE) {return FALSE;}

// Otherwise, return the equivalent of the given
// meters value by dividing the value of the
// argument by the conversion factor $f.
   return $m / $f;

} // End of  Length_To_Length(...)







/*
   ------------------------------------------------------------------
   This function computes the arbitrary-precision Nth root of a given
   numerical string X, rounded to any specified number of decimals.

   ARGUMENTS:
   N = Root to be iterated = Integer 2, 3, 4, ..., etc.
   X = Numerical argument string for which we seek the Rth root

   This function is based on the master root iteration algorithm.

   MASTER ROOT ITERATION FORMULA

   Let:
   N = Root to be iterated (2=Square root, 3=Cube root, etc. ...)
   a = Current approximation to Rth root of X
   b = Next approximation descended from a



   Given (N, X, a):

   n = N-1
   START:
   b = (a + X/(a^n)) / n

   For next approximation
   a = b
   Repeat from START until required accuracy reached.

   -----------------------------------------------------------------
   NumDecimals = Number of decimals at which to round off the result
                 if exact resolution is not possible.

   ERRORS:
   NO special error checking is done in this version of the function.
   ------------------------------------------------------------------
*/

   function bcNth_Root_Of_X ($N, $X, $Decimals=16)
{
   $x = trim($X);
   $a = sprintf("%1.16f", pow($x, 1/$N)); // Compute first approximation
   $n = intval($N) - 1;
   $d = abs(intval($Decimals));
   $D = $d + 8;

   for ($i=1;  $i <= 50;   $i++)
  {
   $b = bcdiv(bcadd(bcmul($n,$a,$D),bcdiv($x,bcpow($a,$n,$D),$D),$D),$N,$D);

   if (bccomp($a,$b,$D) == 0) {break;} else {$a = $b;}
  }
   return rtrim(rtrim(bcadd($b, '0.'.str_repeat(0,$d).'5', $d), '0'), '.');
}





/*
   ===========================================================================
   This function returns the arbitrary-precision factorial of any non-negative
   integer value.  This function should be used with caution, since factorial
   values can be extremely large for relatively small arguments.  It may take
   several seconds to compute extremely large factorials (e.g. > 10000!), so
   a safety limit of 9999 is built in, but it can be easily removed.

   Input argument must be a positive integer.  Any decimal argument will be
   silently truncated to an integer, without rounding, rather than causing
   an error.

   ERRORS:
   FALSE is returned on error, if argument is non-numeric, negative or the
   argument exceeds the maximum set limit (default = 9999).
   ===========================================================================
*/

   function bcFact ($IntN='0')
{
   $N = trim($IntN);

// ERROR IF ARGUMENT N IS NON-NUMERIC, NEGATIVE OR > 9999.
   if (!is_numeric($N) or $N < 0 or $N > 9999) {return FALSE;}

// TRUNCATE ANY DECIMAL ARGUMENT TO AN
// INTEGER VALUE, WITHOUT ROUNDING.
   $n = bcadd($N, '0');

// EXECUTE LOOP TO COMPUTE (N!) PRODUCT (P).
   $P=1;  for ($i=1;  $i <= $n;  $i++) {$P = bcmul($P, $i);}

// Done.
   return $P;
}





/*
   This function computes the double factorial value for a
   positive integer argument using arbitrary-precision.

   The double factorial of N is the product of every other
   positive integer with the same parity as N in the range
   from 1 up to and including N.

   The parity simply refers to whether an integer is an odd
   number or an even number.

   The double factorial values of 0 and -1 are equated to 1
   by definition, but double factorial values for integers
   that are less than -1 are not defined.

   Returns FALSE on ERROR if (N) is non-numeric or (N < -1).

*/
   function bcDouble_Fact ($IntN)
{
   $N = trim($IntN);   if (!is_numeric($N) or $N < -1) {return FALSE;}
   $j = ceil($N/2) - 1;
   $P = 1;  for($n=0;  $n <= $j;  $n++) {$P = bcmul($P, $N-2*$n);}

   return $P;
}














/*
  ========================================================================
  This function returns the permutations of (N) items taken (R) at a time.
  Like the digits in a number, the sequential order of items is important.

  Input arguments must be positive integers.  Any decimal argument will be
  truncated to an integer, without rounding, rather than returning an error.

   ERRORS:
   FALSE is returned on error, if either argument is non-numeric or out
   of valid range.
  ========================================================================
*/

  function bcPerm_N_R ($IntNStr, $IntRStr)
{
// Read numeric string arguments. Must be
// non-negative integer strings only. Any
// decimal part will be ignored and lost.
   $N = trim($IntNStr);
   $R = trim($IntRStr);

// Error if either argument is non-numeric.
   if (!is_numeric($N) or !is_numeric($R)) {return FALSE;}

// Error if either argument is out of valid range.
   if ($N < 0 or $R < 0 or $R > $N) {return FALSE;}

// Truncate argument(s) to integer value(s) without rounding.
   $N = bcadd($N, 0);  $R = bcadd($R, 0);

// Execute loop to compute P(N, r).
   $P=1;  for ($i=0;  $i < $R;  $i++) {$P = bcmul($P, $N-$i);}

// Done.
  return $P;
}




/*
  ========================================================================
  This function returns an arbitrary precision count of all total possible
  combinations or mixtures of (N) items taken (R) at a time.  Like a hand
  dealt in a poker game, the sequential order in which the cards were
  received is NOT important, but only which cards are held in the hand,
  regardless of their arrangement or order.

  ERRORS:
  FALSE is returned on error, if either argument is non-numeric or out
  of valid range.
  ========================================================================
*/

  function bcComb_N_R ($N, $R)
{
// Error if either argument is non-numeric.
   if (!is_numeric($N) or !is_numeric($R)) {return FALSE;}

// Truncate argument(s) to integer value(s) without rounding.
   $N = bcadd($N, 0);
   $R = bcadd($R, 0);

// Error if either argument is out of valid range.
   if ($N < 0 or $R < 0 or $R > $N) {return FALSE;}

// Execute loop to compute C(N, r).
   $C=1;  for ($i=0;  $i < $N-$R;  $i++) {$C = bcdiv(bcmul($C, $N-$i), $i+1);}

// Done.
   return $C;
}



/*
   This function is numerically identical to the bcComb() function,
   but using different variable names.

*/

  function bcBinom_n_k ($N, $K)
{
   $n = trim($N);
   $k = trim($K);

// Error if either argument is non-numeric.
   if (!is_numeric($n) or !is_numeric($k)) {return FALSE;}

// Truncate argument(s) to integer value(s) without rounding.
   $n = bcadd($n, 0);
   $k = bcadd($k, 0);

// Error if either argument is out of valid range.
   if ($n < 0 or $k < 0 or $k > $n) {return FALSE;}

// Execute loop to compute Binom(n, k).
   $C=1; for ($i=0; $i < $n-$k; $i++) {$C = bcdiv(bcmul($C, $n-$i), $i+1);}

// Done.
   return $C;
}










/*
   =========================================================================
   This function returns the greatest common divisor for any two integer
   value strings.  The numerical signs of the numbers are ignored.

   Arbitrary-precision arithmetic is used, so the integers should be given
   as strings, like "1234", so as not to be mistaken for standard floating-
   point values.

*/

   function bcGCD ($IntAStr, $IntBStr)
{
// Read integer string arguments.
   $A = trim($IntAStr);
   $B = trim($IntBStr);

// ERROR if either argument is non-numeric.
   if (!is_numeric($A) or !is_numeric($B)) {return FALSE;}

// Remove any negative signs and work with absolute values.
   $A = str_replace("-", "", $A);
   $B = str_replace("-", "", $B);

// Truncate any non-integer to an integer value WITHOUT rounding.
// This means that anything following the decimal point will
// simply be ignored and lost without rounding the integer part.
   $A = bcadd($A,0);     $B = bcadd($B,0);

// Return 1 if either integer equates to zero.
   if (bccomp($A,0) == 0 or bccomp($B,0) == 0) {return 1;}

// Swap (A,B) values, if necessary, to ensure that B >= A initially.
   if (bccomp($B,$A) < 0) {$x = $B;     $B = $A;     $GCD = $x;}
   else {$GCD = $A;}

// Compute and return the final GCD of the given arguments.
   while ($B != 0) {$x = $B;     $B = bcmod($GCD, $B);     $GCD = $x;}

   return $GCD; // Done

} // End of  bcGCD(...)



/*
   ========================================================================
   This PHP function performs arbitrary-precision
   integer fraction addition without reduction.

   (A/B) + (C/D) = E/F
   ========================================================================
*/

   function bcAdd_Frac ($A_B, $C_D)
{
   list($A,$B) = preg_split("[\/]", preg_replace("/\s+/", '', trim($A_B)));
   list($C,$D) = preg_split("[\/]", preg_replace("/\s+/", '', trim($C_D)));

   $E = bcadd(bcmul($A,$D), bcmul($B,$C));
   $F = bcmul($B,$D);

   return "$E/$F";
}





/*
   ========================================================================
   This PHP function performs arbitrary-precision
   integer fraction subtraction without reduction.

   (A/B) - (C/D) = E/F
   ========================================================================
*/

   function bcSub_Frac ($A_B, $C_D)
{
   list($A,$B) = preg_split("[\/]", preg_replace("/\s+/", '', trim($A_B)));
   list($C,$D) = preg_split("[\/]", preg_replace("/\s+/", '', trim($C_D)));

   $E = bcsub(bcmul($A,$D), bcmul($B,$C));
   $F = bcmul($B,$D);

   return "$E/$F";
}



/*
   ========================================================================
   This PHP function performs arbitrary-precision
   integer fraction multiplication without reduction.

   (A/B) * (C/D) = E/F
   ========================================================================
*/

   function bcMul_Frac ($A_B, $C_D)
{
   list($A,$B) = preg_split("[\/]", preg_replace("/\s+/", '', trim($A_B)));
   list($C,$D) = preg_split("[\/]", preg_replace("/\s+/", '', trim($C_D)));

   $E = bcmul($A,$C);     $F = bcmul($B,$D);

   return "$E/$F";
}




/*
   ========================================================================
   This function performs arbitrary-precision
   integer fraction division without reduction.

   (A/B) / (C/D) = (E/F)
   ========================================================================
*/

   function bcDiv_Frac ($A_B, $C_D)
{
   list($A,$B) = preg_split("[\/]", preg_replace("/\s+/", '', trim($A_B)));
   list($C,$D) = preg_split("[\/]", preg_replace("/\s+/", '', trim($C_D)));

   $E = bcmul($A,$D);     $F = bcmul($B,$C);

   return "$E/$F";
}

/*
   ========================================================================
   This function reduces arbitrary-precision integer fraction (A/B)
   to its lowest terms (a/b), if possible.

   Argument must be an integer fraction string in 'A/B' format.

   ERRORS:
   No special error checking is done.
   ========================================================================
*/

   function bcReduce_Frac ($A_B)
{
// -----------------------------------------------
// Extract the individual fraction elements (A, B)
// from the input argument string ("A/B").

   list($A,$B) = preg_split("[\/]", str_replace(' ','', trim($A_B)));

// -------------------------------------------------
// To find the GCD, we work with the absolute values
// of E and F, calling those working values wE and
// wF respectively.

   $wA = bcmul(((bccomp($A, 0) < 0)? -1:1), $A);
   $wB = bcmul(((bccomp($B, 0) < 0)? -1:1), $B);

   if (bccomp($wA,$wB) > 0) {$w=$wA;   $wA=$wB;   $wB=$w;}

// -------------------------------------
// Equate 1st approximation of GCD to wA

   $GCD = $wA;

// ------------------------------------------------
// Perform Euclid's algorithm loop while (wB != 0).
// The value of wB decreases after each cycle and
// eventually reaches zero, at which point we have
// reached the GCD value.  This will always equate
// to at least 1.

   while (bccomp($wB, 0) != 0)
         {$w=$wB;     $wB=bcmod($GCD, $wB);     $GCD=$w;}

// ------------------------------------------
// Compute and return reduced fraction (a/b).
// If GCD == 1, then the fraction cannot be
// reduced and is returned unchanged.

   $a = bcdiv($A, $GCD);
   $b = bcdiv($B, $GCD);

   return "$a/$b";

} // End of  bcReduce_Frac (...)




/*
   This function adds a string of BC decimal values together.
   The individual values are space delimited integers or
   fractional values.

   RetModeStr = Return mode (D = Decimal fraction  or  I = Integer fraction)

   String of Numbers = "1 2 1/3 4 5 6 7 8 9 10 11"
   decimals = 32
   sum  = 63.33333333333333333333333333333333

*/
   function bcAdd_Num_String ($StringOfNumbers, $RetModeStr="Dec", $decimals=16)
{
// ------------------------------------------------------------
// Read argument string of numerical expressions.  These can be
// integers or fractions, including decimal values.

   $w = preg_replace("/\s+/", " ", trim($StringOfNumbers));

// ------------------------------------------------
// Create 'rounding mask' to apply for rounding off
// the final result to the given number of decimals.

   $RoundingMaskStr = "0." . str_repeat("0", $decimals) . "5";

// -------------------------------------------------
// Read return mode argument and force to uppercase.
// Only 1st character is read (Dec or Int fraction).

   $RModeStr = substr(strtoupper(trim($RetModeStr)), 0,1);

// -----------------------------------
// Store expressions in working array.

   $NumsArray = preg_split("[ ]", $w);

// ----------------------
// Count the expressions.

   $NumsCount = count($NumsArray);

// ----------------------------------------------
// Cycle through the expressions, converting them
// all into exactly equivalent integer fractions.

   $sum = "0/1";

   for($i=0;   $i < $NumsCount;   $i++)
  {
// Get current number from work array and then
// convert any decimal fraction into integer
// fraction.
   $w = bcDec_to_Int_Frac ($NumsArray[$i]);

// Split current sum into A,B and current
// fraction to C,D and
   list($A,$B) = preg_split("[\/]", $sum);
   list($C,$D) = preg_split("[\/]", $w);

// Compute new A/B to replace previous A/B and
// repeat until all numbers have been summed.
   $AD = bcmul($A,$D);   $BC = bcmul($B,$C);   $BD = bcmul($B,$D);

   $A = bcadd($AD,$BC);  $B = $BD;

   $sum = "$A/$B"; // Dec_to_Int_Frac("$A/$B");
  }

// Reduce fraction to its lowest
// terms if possible.
// $sum = Dec_to_Int_Frac("$A/$B");

 $NegSign = "";


  if ($RModeStr == "D")
 {
  list($A,$B) = preg_split("[\/]", $sum);

  $sum = bcdiv($A,$B, $decimals);

// Temporarily remove any negative sign (-).  It will
// be re-attached to the returned result.
   if (substr($sum,0,1) == "-")
      {
       $NegSign = "-";   $sum = substr($sum, 1, strlen($sum));
      }

// Round off absolute sum value to given number of decimals
// and then trim off any redundant zeros and/or decimal point.
   $sum = rtrim(rtrim(bcadd($sum, $RoundingMaskStr, $decimals), "0"), ".");
 }
// Reattach any negative sign.  Done.
   return $NegSign . $sum;

} // End of  bcAdd_Num_String(...)







/*
   This function multiplies a string of BC decimal values together.
   The individual values are space delimited integers or fractional
   values in any combination.

   RetModeStr = Return mode (D = Decimal fraction  or  I = Integer fraction)

   PROD

    A     C     A*C     E
   --- * --- = ----- = ---
    B     D     B*D     F


   1/3.15  2  2.90/1.3  4  5  6  7  8  9  10  13  10  11
   =
   1/3.15 * 2 * 2.90/1.3 * 4 * 5 * 6 * 7 * 8 * 9 * 10 * 13 * 10 * 11

*/
   function bcMul_Num_String ($StringOfNumbers, $RetModeStr="Dec", $decimals=16)
{
// Read argument string of numerical expressions.  These can be
// integers or fractions, including decimal values.
   $w = preg_replace("/\s+/", " ", trim($StringOfNumbers));

// Create 'rounding mask' to apply for rounding off
// the final result to the given number of decimals.
   $RoundingMaskStr = "0." . str_repeat("0", $decimals) . "5";

// Read return mode argument and force to uppercase.
// Only 1st character is read (Dec or Int fraction).
   $RModeStr = substr(strtoupper(trim($RetModeStr)), 0,1);

// Store expressions in working array.
   $NumsArray = preg_split("[ ]", $w);

// Count the expressions.
   $NumsCount = count($NumsArray);

// Cycle through the expressions, converting them
// all into exactly equivalent integer fractions.
   $prod = "1/1";

   for($i=0;   $i < $NumsCount;   $i++)
  {
// Get current number from work array and then
// convert any decimal fraction into integer
// fraction.
   $w = Dec_to_Int_Frac ($NumsArray[$i]);

// Split current product into A,B and current
// working number (w) string into to C,D
   list($A,$B) = preg_split("[\/]", $prod);
   list($C,$D) = preg_split("[\/]", $w);

// Compute new A/B to replace previous A/B and
// repeat until all numbers have been multiplied.
   $AC = bcmul($A,$C);   $BD = bcmul($B,$D);

   $A = $AC;  $B = $BD;

   $prod = "$A/$B";
  }

// Reduce fraction to its lowest
// terms if possible.
 $prod = Dec_to_Int_Frac("$A/$B");

 $NegSign = "";


  if ($RModeStr == "D")
 {
  list($A,$B) = preg_split("[\/]", $prod);

  $prod = bcdiv($A,$B, $decimals);

// Temporarily remove any negative sign (-).  It will
// be re-attached to the returned result.
   if (substr($prod,0,1) == "-")
      {
       $NegSign = "-";   $prod = substr($prod, 1, strlen($prod));
      }

// Round off absolute product value to given number of decimals
// and then trim off any redundant zeros and/or decimal point.
   $prod = rtrim(rtrim(bcadd($prod, $RoundingMaskStr, $decimals), "0"), ".");
 }
// Reattach any negative sign.  Done.
   return $NegSign . $prod;

} // End of  bcMul_Num_String(...)










/*
   This BC function multiplies a delimited series of decimal numbers
   to a specified number of decimals.

   Default assumes numbers are integers unless decimals > 0

   The numbers can be delimited by commas or spaces between numbers.

   In the case of non-integers, the product result is NOT rounded off.
*/

   function bcMul_Num_Series($NumSeriesStr, $NumDecimals=0)
{
// CONVERT ALL COMMAS INTO SPACES.
   $NumStr = str_replace(',', ' ', $NumSeriesStr);

// STRIP OUT ALL WHITE SPACE.
   $NumStr =  preg_replace("/\s+/", " ", trim($NumStr));

// ERROR IF EMPTY STRING.  RETURN FALSE.
   if ($NumStr == '') {return FALSE;}

// STORE AND COUNT THE NUMBERS IN AN ARRAY.
   $NumArray = preg_split("[ ]", $NumStr);
   $NumCount = count($NumArray);

// IF ONLY A SINGLE NUMBER, THEN RETURN IT AS-IS.
   if ($NumCount == 1) {return $NumStr;}

// MULTIPLY THE SERIES OF NUMBERS TOGETHER.
   $product = '1';

   for ($i=0;   $i < $NumCount;   $i++)
   {
    $product = bcmul($product, trim($NumArray[$i]), $NumDecimals);
   }
    return $product;
}






/*
   This function computes the perimeter of an ellipse with the given
   parameters (R, r).  It is a numerical solution to the complete
   elliptic integral of the second kind.

   Author  : Jay Tanner
   License : Public Domain
*/

   function Ellipse_Circum ($RParam='1', $rParam='1')
{
// ______________________________________________
// read input (R, r) ellipse parameter arguments.
// they are assumed to be purely numeric strings.

   $R = trim($RParam);
   $r = trim($rParam);

// _____________________________________________________
// Return ERROR message if either (R, r) is non-numeric.

   if (!is_numeric($R) or !is_numeric($r))
      {return 'ERROR: Invalid argument. Both arguments must be purely numeric.';}

// _______________________________________________
// Define internal constant (2*pi) to 50 decimals.

   $TwoPi = '6.28318530717958647692528676655900576839433879875';

// ___________________________________
// Set to 16 decimals final precision.

   $decimals = 16;

// ______________________________________________
// Use 5 extra decimals for internal computations
// to reduce accumulative rounding errors in the
// final result.

   $d  = $decimals + 5;

// _____________________________________________
// If (r > R), then swap values so that (R > r).

   if (bccomp($R, $r, $d) < 0)  {$w=$R;  $R=$r;  $r=$w;}

// ______________________________________________________
// Compute the square of the elliptic eccentricity value.

   $e2 = bcsub('1', bcdiv(bcmul($r,$r, $d), bcmul($R,$R, $d),$d),$d);

// ______________________________________________
// Perform power series summation loop while the
// nth term is > the 16-decimal (PrecisionLimit).
// This limit could be changed, but 16 decimals
// should generally be sufficient.

  $sum = 0;   $n = $ePow2n = $PrecisionLevel = 1;

   $PrecisionLimit = '0.' . str_repeat('0', $d-1) . '1';

   while (abs($PrecisionLevel) > $PrecisionLimit)
  {
   $N = 2*$n-1;  $X=1;  for($n=0;  $n <= ceil($N/2) - 1;  $n++)
  {$X = bcmul($X, $N - 2*$n);}  $X2 = bcmul($X,$X, $d); // (2n-1)!!²

   $N = 2*$n;    $Y=1;  for($n=0;  $n <= ceil($N/2) - 1;  $n++)
  {$Y = bcmul($Y, $N - 2*$n);} $Y2 = bcmul($Y,$Y,$d); // (2n)!!²

   $ePow2n      = bcmul($ePow2n, $e2, $d); // e^2n
   $E           = bcmul($X2, $ePow2n, $d); // (2n-1)!!² * e^2n
   $F           = bcmul(2*$n-1, $Y2,  $d); // (2n)!!² * 2n
   $CurrSumTerm = bcdiv($E, $F, $d);
   $sum         = bcadd($sum, $CurrSumTerm, $d);
   $n++;
   $PrecisionLevel  = bcsub($CurrSumTerm, $PrecisionLimit, $d);
  }
   $CurrSumTerm = bcdiv(bcmul($X2, $ePow2n, $d), bcmul(2*$n-1, $Y2, $d),$d);

// ____________________________________________________
// Finish computing the full elliptic circumference and
// round-off result to the given number of  decimals.

   $w = bcmul($TwoPi, bcmul($R, bcsub('1', $sum, $d), $d),$d);

   return bcadd($w, '0.'.str_repeat('0', $decimals).'5', $decimals);

} // END OF  Ellipse_Circum (...)











/*
   This function computes the surface area of an ellipsoid
   of given equatorial and polar parameters (a, b).

   This function is NOT yet arbitrary-precision.
*/

   function Ellipsoid_Surf_Area ($aEqu, $bPol)
{
   $a = trim($aEqu);
   $b = trim($bPol);
   $d = 32; // Precision limit

   if (bccomp($a, $b, $d+3) < 0) {$w=$a; $a=$b; $b=$w;}

   $a2 = bcmul($a,$a, $d+3);
   $b2 = bcmul($b,$b, $d+3);

   if (bccomp($a, $b, $d+3) == 0) {return 4 * M_PI * $a2;}

   $e = bcsqrt(bcsub('1', bcdiv($b2, $a2, $d+3), $d+3), $d+3);

   return 2 * M_PI * $a*$a *(1 + (1 - $e*$e)/$e * atanh($e));
}










   function Parallel_Circles_Table ($aParam='6378137.0', $bParam='6356752.3142')
{

$a = trim($aParam);
$b = trim($bParam);
$e = sqrt(1 - $b*$b/$a/$a);

// COMPUTE EQUATORIAL MERIDIAN RADIUS AND CIRCUMFERENCE.
   $MeridCircum   = Ellipse_Circum ($a, $b);
   $MeridCircumMi = sprintf("%1.5f", $MeridCircum / 1609.344);
   $MeridCircum   = sprintf("%1.4f", $MeridCircum);


$out = '';

for($lat=0;   $lat < 90;   $lat++)
{
$SinLat = sin(deg2rad($lat));
$CosLat = cos(deg2rad($lat));
$Rp = ($a * $CosLat) / sqrt(1 - $e*$e*$SinLat*$SinLat);
$Cp = 2*M_PI*$Rp;

$LatTab = sprintf("%d",  $lat);
$RpTab = sprintf("%1.4f", $Rp);
$CpTab = sprintf("%1.4f", $Cp);

$RpTabMi = sprintf("%1.5f", $Rp / 1609.344);
$CpTabMi = sprintf("%1.5f", $Cp / 1609.344 );
$e = sprintf("%1.16f", $e);

$out .= "<tr> <td align='center'>$LatTab&deg;</td> <td align='center'>$RpTab m  =  $RpTabMi mi</td> <td align='center'>$CpTab m  =  $CpTabMi mi</td> </tr>\n";
}

return
"
<table border='1' cellpadding='4' style='font-size:15px; font-family:monospace; font-weight:bold; white-space:pre;'>

<tr><td colspan='3' align='center'>Table of Ellipsoid Latitude Circles at 1-Degree Intervals<br><br>a = $a,   b = $b,   e = $e<br><br>Meridian Circumference = $MeridCircum m  =  $MeridCircumMi mi<br><br></td></tr>

<tr> <td align='center'>&plusmn; Lat</td> <td align='center'>Radius of Latitude Circle</td> <td align='center'>Circumference of Latitude Circle</td></tr>
$out
<tr> <td align='center'>90&deg;</td> <td align='center'>0.0000 m  =  0.00000 mi</td> <td align='center'>0.0000 m  =  0.0000 mi</td></tr>
</table>
";

}



/*
   This arbitrary-precision function computres the hyperbolic arctangent.

   Result is rounded off.

   Values approaching very close to ±1.0 could take a
   VERY long time to evaluate and cause a time-out.
*/

   function bcATanh ($xArg, $Decimals=16)
{
   $x = trim($xArg);

// ERROR IF ABSOLUTE VALUE > 1
   if (abs($x) > 1) {return FALSE;}

   $NumSign = (substr($x, 0, 1) == '-')? '-':'';
   $x = str_replace('-','', $x);
   $x = str_replace('+','', $x);

   $d = $decimals = intval($Decimals); // Decimals precision limit.
   $d = $d + 5 + floor($d/2); // Extra decimals for internal use.

   $PrecisionLimit = '0.' . str_repeat('0', $d-1) . '1';
   $sum = $PrevSum = 0;   $CurrTerm = $diff = 1;

   for ($n=1;   bccomp($diff, $PrecisionLimit, $d+2) > 0;   $n+=2)
{
   $CurrTerm = bcdiv(bcpow($x, $n, $d), $n, $d);
   $sum      = bcadd($sum, $CurrTerm, $d);
   $diff     = str_replace('-', '', bcsub($sum, $PrevSum, $d));
   $PrevSum  = $sum;
}
// DONE.
   return $NumSign.bcadd($sum, '0.' . str_repeat('0', $d) . '5', $decimals);
}




/*
   THESE ARBITRARY TRIGONOMETRIC FUNCTIONS TAKE
   DEGREE ARGUMENTS RATHER THAN RADIANS.
*/


/*
   __________________________________________________
   This function computes  the circular sine to up to
   100 decimals precision for an angle argument given
   in decimal degrees.

   Author  : Jay Tanner - 2017
   License : Public Domain
   __________________________________________________
*/

   function bcSind ($AngDeg, $Decimals=16)
{
// ____________________________________________
// Read angular argument in degrees.  It should
// be in the form of a numeric string, such
// as '12.345', so as not to be confused
// with a normal double-precision value.

   $a = trim($AngDeg);

// __________________________________________
// Set decimals precision, not to exceed 100.

   $decimals = floor(abs($Decimals));
   if ($decimals > 100) {$decimals = 100;}
   if ($decimals <   1) {$decimals =   1;}
   $d = abs(intval($decimals)) + 8;

// ______________________________________________________
// Reasonably handle negative and excessive angle values.

   while(bccomp($a,   '0', $d) < 0) {$a = bcadd($a, '360', $d);}
   while(bccomp($a, '360', $d) > 0) {$a = bcsub($a, '360', $d);}

// ___________________________________________________________
// Define internal constant for degrees-to-radians conversion.
// This local constant could also be made global.

   $RAD_PER_DEG = '0.01745329251994329576923690768488612713442871888541725456097191440171009114603449443682241569634509482212304492';

   $x = bcmul($a, $RAD_PER_DEG, $d);

// ___________________________________________________________________
// Remove, but remember the numerical sign to be reattached to result.

   $NumSign = '+';  if (bccomp($x, '0', $decimals) < 0)
                       {$x = str_replace('-', '', $x); $NumSign = '-';}

// ______________________________________
// Compute sine value from Taylor series.

   $PrecisionLimit = '0.' . str_repeat('0', $decimals) . '1';

   $term = $n = $denom = 1;
   $nps  = -1;
   $sinx = $x;

   while (abs($term) > $PrecisionLimit)
  {
   $numer = bcmul($nps, bcpow($x, 2*$n+1, $d), $d);
   $denom = bcmul(bcmul($denom, 2*$n), 2*$n+1);
   $term  = bcdiv($numer, $denom, $d);
   $sinx  = bcadd($sinx, $term, $d);
   $nps   = -$nps;
   $n++;
  }

// _____________________________________________________
// Round-off sine value to specified number of decimals.

   $Z = '0.' . str_repeat('0', $decimals) . '5';
   $w = $NumSign.bcadd($sinx, $Z, $decimals);

// _____________________________
// Patch to fix +- sign anomaly.

   $sinx = str_replace('+-', '-', $w);

   return $sinx;

} // END OF  bcSind(...)







/*
   ___________________________________________________
   This function computes the circular cosine to up to
   100 decimals precision for an angle argument given
   in decimal degrees.

   The cosine value is actually computed from the sine
   value, where:
                cos²(x) = 1 - sin²(x)

   Author  : Jay Tanner
   License : Public Domain
   ___________________________________________________
*/

   function bcCosd ($AngDeg, $Decimals=16)
{
// _________________________________
// Read angular argument in degrees.

   $a = trim($AngDeg);

// __________________________________________
// Set decimals precision, not to exceed 100.

   $decimals = floor(abs($Decimals));
   if ($decimals > 100) {$decimals = 100;}
   if ($decimals <   1) {$decimals =   1;}
   $d = abs(intval($decimals)) + 8;

// ______________________________________________________
// Reasonably handle negative and excessive angle values.

   while(bccomp($a, '0',   $d) < 0) {$a = bcadd($a, '360', $d);}
   while(bccomp($a, '360', $d) > 0) {$a = bcsub($a, '360', $d);}

// ________________________
// Check for special cases.

   if (abs($a) ==  90 or abs($a) ==  270) {return bcadd('0', '0', $decimals);}
   if (abs($a) == 180) {return bcadd('-1', '0', $decimals);}

// ___________________________________________________________
// Define internal constant for degrees-to-radians conversion.
// This local constant could also be made global.

   $RAD_PER_DEG = '0.01745329251994329576923690768488612713442871888541725456097191440171009114603449443682241569634509482212304492';

   $x = bcmul($a, $RAD_PER_DEG, $d);

   $NumSign = '';  if (bccomp($x, '0', $d) < 0)
                      {$x = str_replace('-', '', $x); $NumSign = '-';}

// __________________________________________________
// Compute sine value from Taylor series.  The cosine
// value will then be derived from this sine value.

   $PrecisionLimit = '0.' . str_repeat('0', $decimals) . '1';

   $n = $denom = $term = 1;
   $nps  = -1;
   $sinx = $x;

   while (abs($term) > $PrecisionLimit)
  {
   $numer = bcmul($nps, bcpow($x, 2*$n+1, $d), $d);
   $denom = bcmul(bcmul($denom, 2*$n), 2*$n+1);
   $term  = bcdiv($numer, $denom, $d);
   $sinx  = bcadd($sinx, $term, $d);
   $nps   = -$nps;
   $n++;
  }
   $sinx  = $NumSign.bcadd($sinx, $PrecisionLimit, $d);

// ______________________________________
// Compute the cosine from the sine value
// cos²(x) = 1 - sin²(x) and adjust the
// numerical sign as necessary.


   $cosx = bcsqrt(bcsub('1',bcmul($sinx,$sinx, $d), $d), $d);
   $NumSign = ($a > 90 and $a < 270)? '-':'+';

// _______________________________________________________
// Round-off cosine value to specified number of decimals.

   $Z = '0.' . str_repeat('0', $decimals) . '5';
   return $NumSign.bcadd($cosx, $Z, $decimals);

} // END OF  bcCosd(...)




/*
  sin(x) = cos(90°-x) and
  cos(x) = sin(90°-x).
*/


   function bcTand ($AngDeg, $decimals=16)
{
   if ($decimals > 100) {$decimals = 100;}
   if ($decimals <=  0) {$decimals =  16;}
   $d = abs(intval($decimals)) + 5;

   $RAD_PER_DEG = '0.01745329251994329576923690768488612713442871888541725456097191440171009114603449443682241569634509482212304492';

   $x = bcmul($RAD_PER_DEG, trim($AngDeg), $d);

   $NumSign = '';  if (bccomp($x, '0', $d) < 0)
                      {$x = str_replace('-', '', $x); $NumSign = '-';}

   $PrecisionLimit = '0.' . str_repeat('0', $decimals) . '1';

   $term = $nps = 1;
   $sinx = $n   = 0;

   while (abs($term) > $PrecisionLimit)
  {
   $numer = bcmul($nps, bcpow($x, 2*$n+1, $d), $d);
   $denom = bcfact(2*$n+1);
   $term  = bcdiv($numer, $denom, $d);
   $sinx  = bcadd($sinx, $term, $d);
   $nps   = -$nps;
   $n++;
  }
   $sinx = "$NumSign$sinx";
   $cosx = bcsqrt(bcsub('1', bcmul($sinx,$sinx, $d), $d), $d);
   $tanx = bcdiv($sinx,$cosx, $d);

// DONE
   return bcadd($tanx, 0, $decimals);

} // END OF  bcTand(...)

















//  //2.30258509299404568401799145468436420760110148862877297603332790096757260967735248023599720508959829834196778403



   function bcExp ($xArg, $decimals=16)
{
   if ($decimals > 100) {$decimals = 100;}
   if ($decimals <=  0) {$decimals =  16;}
   $d = abs(intval($decimals)) + 5;

   $x = trim($xArg);

   $PrecisionLimit = '0.' . str_repeat('0', $decimals) . '1';

   $term = 1;
   $sum  = $n = 0;

   while (abs($term) > $PrecisionLimit)
  {
   $numer = bcpow($x, $n, $d);
   $denom = bcfact($n);
   $term  = bcdiv($numer, $denom, $d);
   $sum   = bcadd($sum, $term, $d);
   $n++;
}

// ROUND OFF THE SUM. DONE
   return bcadd($sum, $PrecisionLimit, $decimals);

} // END OF  bcExp(...)






/*
  This function computes the arbitrary-precision natural logarithm
  for any positive (x) argument.

  Special case:  If (0 < x < 1), then Ln(x) = -Ln(1/x)

  Error returns FALSE if (x <= 0).

*/


   function bcLn ($xArg, $decimals=16)
{
   $x = trim($xArg);
   if (strpos($x, '-') !== FALSE or $x <= 0) {return FALSE;}

   if ($decimals > 100) {$decimals = 100;}
   if ($decimals <=  0) {$decimals =   1;}

   $d = abs(intval($decimals)) + 5;

// _______________________________________________________
// Handle case of fractional (x) values where (0 < x < 1).
   $sign = ''; if ($x > 0 and $x < 1)
                  {$sign = '-';  $x = bcdiv('1', $x, $d);}

// _______________________________________________________________________________
// Natural logarithm of 10 to 110 decimals (used as internal conversion constant).
   $LN_10 = '2.30258509299404568401799145468436420760110148862877297603332790096757260967735248023599720508959829834196778403';

// _________________________________________
// Determine power of 10 in positive (x) and
// where to place the decimal in the result.
   $dpos = 1;
   if($x >= 10)
     {
      $dpos = floor(log10($x) + 1);
      $x = str_replace('.','', $x);
      $x = substr($x,0, 1).'.'.substr($x,1, strlen($x));
     }

   $PrecisionLimit = '0.' . str_repeat('0', $decimals) . '1';

   $n   = $term = 1;
   $sum = 0;
   $k   = bcdiv(bcsub($x, '1', $d), $x, $d);

   while (abs($term) > $PrecisionLimit)
  {
   $numer = bcpow($k, $n, $d);
   $denom = $n;
   $term  = bcdiv($numer, $denom, $d);
   $sum   = bcadd($sum, $term, $d);
   $n++;
  }
   $sum = bcadd($sum, bcmul($dpos-1, $LN_10, $d), $d);

// _______________________
// Round off the sum. DONE
   return $sign.bcadd($sum, str_replace('1','5', $PrecisionLimit), $decimals);

} // END OF  bcLn(...)






/*
   ========================================================================
   This function will convert an arbitrary precision positive base 10
   X value into its equivalent in any base from 2 to 36.  The X value
   is NOT restricted to integers and also allows fractional values.

   ARGUMENTS:
   $X10 = Base 10 numeric string to be to converted
          into its BaseB equivalent.

   $BaseB = Base (2 to 36) in which to interpret input digit string.

   ERRORS:
   On error FALSE is returned.  An error occurs if either
   of the arguments is non-numeric or out of valid range.

   This is a level 2 function, dependent on:
   bcBase10_Int_To_BaseB (...)

   ========================================================================
*/

   function bcBase10_To_BaseB ($X10, $BaseB, $TruncationLimit=0)
{
   $p = 1000; // Decimals precision control factor (Don't ask).

   $q =  $TruncationLimit; // Decimal truncation limit.

   $X = trim($X10);  $sign = (substr($X,0,1) == "-")? "-" : "";
   $X = str_replace("-","", $X);

   if (strpos($x, ".") === FALSE){$X .= ".0";}

   list($i, $f) = preg_split("[\.]", $X);  $f = "0.$f";

   $IntPartOut = bcBase10_Int_To_BaseB ($i, $BaseB);

   $w = bcmul(bcPow($BaseB, $p), $f);

   $fp = bcBase10_Int_To_BaseB ($w, $BaseB);

   $zeros = str_repeat("0", $p - strlen($fp));

   if ($q > 0) {$fp = substr($fp, 0, $q-0);}

   $FracPartOut = ".$zeros$fp";

   $FracPartOut = rtrim(rtrim($FracPartOut,'0'), '.');

   return "$sign$IntPartOut$FracPartOut";

} // End of  bcBase10_To_BaseB(...)





/*
   ========================================================================
   This function will convert an arbitrary precision number value in
   any base from 2 to 36 into its equivalent in base 10.  It is NOT
   restricted only to integers and applies to fractional values as well.

   ARGUMENTS:
   $BaseBIntStr = Base B numeric string to be to converted
                  into its base 10 equivalent.

   $BaseB = Base of input digit string (2 to 36).

   ERRORS:
   On error FALSE is returned.  An error occurs if either
   of the arguments is non-numeric or out of valid range.

   This is a level 2 function.
   ========================================================================
*/


   function bcX_Base_B_To_Base_10 ($BaseBNumStr, $BaseB)
{

// Compute base 10 decimal equivalent to BaseB fraction.

   $a = 0;
   $b = 1;

   $BaseDigits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

// Read and copy input arguments.
   $x = trim($BaseBNumStr);

// Remember numerical sign, if any and work with absolute
// value.  The sign is restored to output value.
   $sign = (substr($x, 0,1) != "-")? '' : '-';
   $x = str_replace("-", '', $x);
   $x = str_replace("+", '', $x);

// If argument is an integer value, then attach ".0" to it.
   if(strpos($x, ".") === FALSE) {$x .= ".0";};

// Get only integer part of the BaseB numeric string.
   $xi = substr($x, 0, strpos($x, "."));

// Compute equivalent base 10 integer value.
   $IntegerPartBase10 = bcBase_B_Int_To_Base_10($xi, $BaseB);

// Get only the fractional digits of the BaseB numeric string.
   $xf = substr($x, strpos($x, ".")+1, strlen($x));

// Count BaseB fractional digits.
   $DigitsCount = strlen($xf);

// Cycle through each digit, creating integer fraction terms
// and summing them up to final a/b.

   $out = '';

   $PowB = $BaseB;

   for($i=0;   $i < $DigitsCount;   $i++)
   {
// get digit and convert to base 10 equivalent.
   $c = strpos($BaseDigits, substr($xf, $i, 1));
   $d = $PowB;  $PowB = bcmul($PowB, $BaseB);

// Add to current (a/b).
   $a = bcadd(bcmul($a,$d), bcmul($b,$c));
   $b = bcmul($b,$d);

//   $out .= "$c/$d\n";

   }

// Compute final raw, unreduced (a/b) value.
   $c = $IntegerPartBase10;
   $d = "1";
   $a = bcadd(bcmul($a,$d), bcmul($b,$c));
   $b = bcmul($b,$d);

//   return $out;

   $w = bcreduce_frac("$sign$a/$b");

   $w = preg_replace("/\s+/", '', trim($w)); // Modified vesrion

// If single value, then put over 1.
   if (strpos($w, "/") === FALSE) {$w .= "/1";}

// Get numerator and denominator values.
   list($A,$B) = preg_split("[\/]", $w);

// Compute decimal value equivalent truncated
// at 1000 decimals.
   $w = bcdiv($A,$B, 1000);

   return rtrim(rtrim($w,'0'), '.');

} // End of  bcXBaseB_To_Base10(...)




/*
   ========================================================================
   This boolean function tests to see of the given argument is a valid
   BaseB (2 to 36) number.

   If valid, then TRUE is returned, otherwise FALSE is returned.

   NOTE:
   It is possible for a BaseB number to be valid in more than one base.

   A given number is considered valid only if none of its digits are
   invalid for the given base.  For example, a base 10 number cannot
   contain the digit 'F', but a hexadecimal (base 16) number can.

*/

   function bcIs_Base_B ($XBaseBString, $BaseB)
{
// Read numeric work string and remove any decimal or numerical sign.
   $X = str_replace(".", '', strtoupper(trim($XBaseBString)));
   $X = str_replace("-", '', $X);
   $X = str_replace("+", '', $X);

   $DigitsSpectrum = substr("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ", 0, $BaseB);

// Cycle through every digit.  Invalid if any digit is not found
// in DigitsSpectrum string, so return FALSE.
// Otherwise, TRUE is returned.
   $DigitsCount = strlen($X);
   for ($i=0;   $i < $DigitsCount;   $i++)
       {
        $digit = substr($X, $i, 1);
        if (strpos($DigitsSpectrum, $digit) === FALSE) {return FALSE;}
       }
   return TRUE;

} // End of  bcIs_Base_B(...)




/*
   ========================================================================
   This function will convert an arbitrary precision integer
   in any base from 2 to 36 into its equivalent in base 10.

   ARGUMENTS:
   $XBaseBString = Base B integer string to be converted
            into its base 10 equivalent.

   $FromBase = Base of input digit string (2 to 36).

   ERRORS:
   On error, FALSE is returned if either argument is invalid.

   ========================================================================
*/

   function bcBaseB_Int_To_Base10 ($XBaseBString, $BaseB)
{

// Read input arguments.
   $Xb = strtoupper(trim($XBaseBString));
   $B  = trim($BaseB);

// ERROR if base is outside of range 2 to 36.
   if ($B < 2 or $B > 36) {return FALSE;}

// Remember any numerical sign and work with absolute value.
// The numerical sign will be restored at the end.
   $sign = (substr($Xb, 0,1) == '-')? '-' :'';
   if ($sign == '-') {$Xb = substr($Xb, 1, strlen($Xb));}

// Count digits in base b integer X.
   $XbDigitsCount = strlen($Xb);

// Initialize base 10 summation accumulator
// and initial power (= 1) of the base.
   $X10 = 0;
   $PowB = 1;

// Define the valid digits spectrum for the given base.
   $DigitsSpectrum = substr("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ", 0, $B);

// Perform loop to compute the base 10
// equivalent of the base b integer Xb.
   for ($i=0;   $i < $XbDigitsCount;   $i++)
       {
//      Get current base b digit from Xb.
        $db = substr($Xb, $XbDigitsCount - $i - 1, 1);

//      Get base 10 numerical value (= String index) of digit.
        $d10 = strpos($DigitsSpectrum, $db);

//      If FALSE is returned, it means that the base b digit was
//      not found within the valid base b DigitsSpectrum string.
        if ($d10 === FALSE) {return FALSE;}

//      Compute and accumulate value of current base 10 term.
        $X10 = bcadd($X10, bcmul($PowB, $d10));

//      Update power of base for next cycle.
        $PowB = bcmul($PowB, $B);
       }

// Done.
   return "$sign$X10";

} // End of  bcBaseB_Int_To_Base10(...)



/*
   ========================================================================
   This function will convert an arbitrary precision base 10 integer into
   its equivalent in any given base from 2 to 36.

   ARGUMENTS:
   $x10IntStr = Base 10 integer string to be to converted
                into its Base B equivalent.

   $BaseB = Base of converted output (2 to 36).

   ERRORS:
   On error, FALSE is returned if either argument is non-numeric
   or out of valid range.
   ========================================================================
*/

   function bcBase10_Int_To_BaseB ($X10IntStr, $BaseB)
{
// Error if either argument is non-numeric
// or if base of out of range (2 to 36).

   if (!is_numeric($X10IntStr) or !is_numeric($BaseB)) {return FALSE;}

   if ($BaseB < 2 or $BaseB > 36) {return FALSE;}

// Define available digits spectrum for bases 2 to 36.
   $DigitsSpectrum = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

   $XB = "";
   $X10 = trim($X10IntStr);

// Remember any numerical sign and work with absolute value.
// The numerical sign will be restored at the end.
   $sign = (substr($X10, 0,1) == '-')? '-' :'';
   if ($sign == '-') {$X10 = substr($X, 1, strlen($X10));}

   if (bccomp($X10, 0) == 0) {return "0";} // Handle zero argument.

// Perform base 10 to base B conversion loop, digit-by-digit.
   while (bccomp($X10, 0) > 0)
  {
   $XB = substr($DigitsSpectrum, bcmod($X10, $BaseB), 1) . $XB;
   $X10 = bcdiv($X10, $BaseB);
  }
// Done.
   return "$sign$XB";

}  // End of  bcBase10_Int_To_BaseB(...)




/*
   This function converts a single integer or decimal fraction string
   into the equivalent fraction in pure integer form (A/B).

   NOTE: Numbers must be in standard form, NOT scientific E-notation.

   "12.042/3.71" would become "12042/3710"

   "1.5000/0.17" would become "150/17"

   "1.234" would become "1234/1000"

   "-0.02337/11.7346709" would become "-233700/117346709"

*/
   function bcDec_to_Int_Frac ($FractionString)
{
// Read fraction string argument and remove ALL spaces.
   $FracStr = str_replace(" ", "", trim($FractionString));

// If only a single value, put it over 1 (like A/1).
   if (strpos($FracStr, "/") === FALSE)  {$FracStr = "$FracStr/1";}

// Split fraction into numerator and denominator elements.
   list($numer, $denom) = preg_split("[\/]", $FracStr);

// If integer (no decimal) part, then append .0 to it.
   $numer .= (strpos($numer, ".") === FALSE)? ".0":"";
   $denom .= (strpos($denom, ".") === FALSE)? ".0":"";

// If any decimal numbers start with a sign followed by a decimal point
// or start with only a decimal point, then a zero must be inserted be-
// fore the decimal.  For example, -.125 would become -0.125 instead.
   $numer = str_replace("-.", "-0.", $numer);
   $numer = str_replace("+.", "+0.", $numer);
   if (substr($numer,0,1) == ".") {$numer = str_replace(".", "0.", $numer);}
   $denom = str_replace("-.", "-0.", $denom);
   $denom = str_replace("+.", "+0.", $denom);
   if (substr($denom,0,1) == ".") {$denom = str_replace(".", "0.", $denom);}



// Isolate only the decimal parts of numerator and denominator.
// This means everything from the decimal point and after.
   $NumerDec = strrchr($numer, '.');
   $DenomDec = strrchr($denom, '.');

// Length of longest decimal part determines power of
// 10 to be used as the common multiplier to replace
// the decimal values with integers.
   $Pow10 = bcpow("10", max(strlen($NumerDec)-1, strlen($DenomDec)-1));

// Multiply both numerator and denominator by
// the common power of 10 needed to convert
// them both into integer values.
   $numer = bcmul($Pow10, $numer);
   $denom = bcmul($Pow10, $denom);

// Remove any redundant common zeros from numerator and denominator.
   while(substr($numer, -1) == "0" and substr($denom, -1) == "0")
  {
   $numer = substr($numer, 0, strlen($numer)-1);
   $denom = substr($denom, 0, strlen($denom)-1);
  }

// Reduce to lowest integer terms, if possible.
// Otherwise, finished.
   $A = $numer;
   $B = $denom;

// If negative A or B argument, then convert to absolute
// value. Any numerical signs are irrelevant and removed.
   if (substr($A,0,1) == "-"){$A = substr($A,1,strlen($A));}
   if (substr($B,0,1) == "-"){$B = substr($B,1,strlen($B));}

// Swap working (A, B) values, if necessary,
// so that A > B initially.
   if (bccomp($A, $B) < 0)
      {
       $w   = $B;
       $B   = $A;
       $GCD = $w;
      }
   else
      {
       $GCD = $A;
      }

// Compute the GCD, if any, of the given arguments.  This value
// is used to reduce the integer fraction to its lowest terms.
   while ($B != 0)   {$w = $B;    $B = bcmod($GCD, $B);    $GCD = $w;}

// Now, reduce fraction to lowest terms
// by dividing both terms by their GCD.
   $numer = bcdiv($numer, $GCD);
   $denom = bcdiv($denom, $GCD);

// Done.
   return "$numer/$denom";

} // End of  Dec_to_Int_Frac(...)



















/*
   ========================================================================
   This function performs LaGrange interpolation within an XY data table.

   ========================================================================
*/

   function LaGrange_Interp ($XYDataTable, $xArg)
{

// -----------
// Initialize.

   $XDataStr = $YDataStr = '';

// --------------------------------------------------------------
// Read and split XY data pairs into a work array.  In the array,
// even number indices = X-data, odd number indices = Y-data.

   $XY = preg_split("[ ]", preg_replace("/\s+/", ' ', trim($XYDataTable)));

// ---------------------------------------------
// Count the total number of data elements. This
// value should always be an even number.

   $TotalDataCount = count($XY);

// ------------------------------------------------------------
// Number of data pairs.  This value should be an integer value
// exactly equal to 1/2 the total number of data points. If not,
// then there is an odd mismatched data point.

   $n = $TotalDataCount / 2;

// -----------------------------------------------------------
// Return error message if data vector element count mismatch.
// For every X value there must be a corresponding Y value or
// an XY data count mismatch error occurs.  An error will also
// occur if insufficient data.  There must be at least two XY
// data points given.

   if ($TotalDataCount < 4 )
      {return "ERROR: There must be at least two XY data pairs.";}

   if ($n != floor($n + 0.5))
      {return "ERROR: XY Data Count Mismatch. Odd data element.";}

// ------------------------------------------------------------
// Compute number of XY data pairs.  This value is exactly half
// the value of the total number of data elements.

   $n = $TotalDataCount / 2;

// -------------------------------------------------------
// Construct separate XY data strings from the array data.
// The XY data strings should each contain the same number
// of data elements.

   for($i=0;   $i < $TotalDataCount;   $i+=2)
      {
       $XDataStr .= $XY[$i]   . " ";
       $YDataStr .= $XY[$i+1] . " ";
      }

// --------------------------------------------------------------
// Split the created XY data vector strings into matching indexed
// arrays.  For every X value there must be a matching Y value
// and no two X values can be identical.

   $X = preg_split("[ ]", trim($XDataStr));
   $Y = preg_split("[ ]", trim($YDataStr));

// ----------------------------------------
// Read X argument for which to interpolate
// the Y value from the given XY data.

   $x = trim($xArg);  if ($x == "") {$x = "0";}

// -----------------------------------
// Initialize Y summation accumulator.
   $y = 0.0;

// -----------------------------------------------------
// Compute Lagrangian product (Li) for given X argument.

   for ($i=0;   $i < $n;   $i++)
       {
        $Li = 1.0;

        for ($j=0;   $j < $n;   $j++)
            {
             if ($j != $i) // Skip this cycle when j == i
                {
                 $Li = ($Li * ($x - $X[$j])) / ($X[$i] - $X[$j]);
                }
            } // Next j

//      --------------------------------------
//      Accumulate sum of Yi polynomial terms.

        $y += ($Y[$i] * $Li);

       } // Next i

   return $y;

} // End of  LaGrange_Interp(...)




/*
   ========================================================================
   This function simply swaps all matching XY data pairs held in a
   space delimited string argument. It was designed as a companion
   to the Lagrange_Interp() function to easily facilitate the easy
   swapping of the XY data computation roles.

   The data trend can be linear or non-linear, however
   only 2 data pairs are needed for any linear trend.

   ERRORS:
   On error FALSE is returned.  An error occurs if
   odd number of data elements or < 4 elements.
   ========================================================================
*/

   function Swap_XY_Data_Columns ($XYDataStr)
{

// -------------------------------------------------------------------------
// Read space-delimited data string and split items into a sequential array.
   $XYArray = preg_split("[ ]", preg_replace("/\s+/", " ", trim($XYDataStr)));

   $XYArrayCount = count($XYArray);

   if ($XYArrayCount % 2 != 0 or $XYArrayCount < 4) {return FALSE;}

   $XYPairsCount = $XYArrayCount / 2;

// -------------------------------------------------------
// Construct and return output string of swapped XY pairs.
   $YXDataPairsStr = "";

   for ($i=1;   $i < $XYArrayCount;   $i+=2)
       {
        $YXDataPairsStr .= $XYArray[$i] . " " . $XYArray[$i-1] . "\n";
       }

   return trim($YXDataPairsStr);
}








/*
  ----------------------------------------------------------------------
  This function returns the time of an extremum (minimum or maximum), if
  any, within a given time vs event table.  For example, this function
  can be used to find the times of perihelion, aphelion, perigee, apogee
  or any general periapsis or apoapsis times.  It is based on a 5-point
  data table and the extremum is computed from a polynomial derived from
  the given data.

  ERRORS:
  No error checking is done and the function assumes that an extremum
  exists within the given data table.

  Possible Uses:
  XY-Data = JD vs DistKm

  ----------------------------------------------------------------------
*/


   function Extremum_5 ($DataTableStr)
{

// ---------------------------------
// Read data table and parse values.

   $DataTable = preg_replace("/\s+/", ' ', trim($DataTableStr));

   list ($x1,$y1, $x2,$y2, $x3,$y3, $x4,$y4, $x5,$y5)
   = preg_split("[ ]", $DataTable);

   $interval = $x2 - $x1;

   $a = $y2 - $y1;
   $b = $y3 - $y2;
   $c = $y4 - $y3;
   $d = $y5 - $y4;

   $e = $b - $a;
   $f = $c - $b;
   $g = $d - $c;
   $h = $f - $e;
   $i = $g - $f;
   $j = $i - $h;

   $k = $j/24;
   $m = ($h + $i)/12;
   $n = $f/2 - $k;
   $p = ($b + $c)/2 - $m;

   $q = $r = 0;

   while ($r < 25)
  {
   $s = 6*($b + $c) - $h - $i + 3*$q*$q*($h + $i) + 2*$q*$q*$q*$j;
   $t = $j - 12*$f;
   $q = $s/$t;

   $r++;
  }
   return $q * $interval + $x3;

} // End of  Extremum5(...)



/*
   INTERPOLATES THE TIME IN HOURS GIVEN AN INTERPOLATION TABLE
   AND JDTT ARGUMENT. FOR ASTRONOMY, THIS CAN BE USED TO INTER-
   POLATE THE R.A. FOR A SPECIFIED JDTT ARGUMENT.
*/
   function Interp_Hours ($HoursInterpTableStr, $JDTTArg)
{
   $wArray = preg_split("[\n]", trim($HoursInterpTableStr));
   $wCount = count($wArray);

   $JDArg = floatval($JDTTArg);

   for ($i=1;   $i < $wCount;   $i++)
  {
   $CurrLine = strip_wspace($wArray[$i]);
   $PrevLine = strip_wspace($wArray[$i-1]);

   list($CurrJDTT, $CurrRAHr) = preg_split("[ ]", $CurrLine);
   list($PrevJDTT, $PrevRAHr) = preg_split("[ ]", $PrevLine);

   $diff = $CurrRAHr - $PrevRAHr;

   if ($diff < 0) {$CurrRAHr += 24;}

   $wArray[$i] = "$CurrJDTT $CurrRAHr\n";

  }

// CONSTRUCT FINAL INTERPOLATION TABLE READY FOR USE.
   $w = '';
   for ($i=0;   $i < $wCount;   $i++)
  {
   $w .= trim($wArray[$i]) . "\n";
  }
   $HoursInterpTableStr = strip_wspace($w);


// PERFORM INTERPOLATION FOR hours VALUE CORRESPONDING TO JDTT VALUE.
   $hours = Lagrange_interp($HoursInterpTableStr, $JDArg);

// IF hours > 24, THEN SUBTRACT 24.
   $hours -= ($hours > 24)? 24:0;

   return sprintf("%2.7f", $hours);

} // End of  Interp_Hours(...)





/*
   This function computes the standard deviation of a set of
   comma, semicolon or space-delimited numbers.

   $NumericDataString = String of delimited numeric values.
                        Example: "123.2  121.5  121.1  124.7"

   SampleMode = 0 or 1 = default.
   0 = Population sample randomly drawn from larger population
   1 = Sample is the total population. (= default)
*/

   function Std_Deviation ($NumericDataString, $SampleMode=1)
{
   $w = str_replace(",", " ", $NumericDataString);
   $w = str_replace(";", " ", $w);
   $w = preg_replace('/\s+/', ' ', trim($w));
   $WorkString = $w;

   $SampleMode = (abs(intval($SampleMode)) != 0)? 1:0;

   $N = $sum = 0;

   while ($WorkString != '')
  {
   $N++;

   $i = strpos($WorkString, ' ');

   if ($i === FALSE)
      {
       $sum  = $sum + $WorkString;
       $mean = $sum / $N;
       break;
      }

   $xi = trim(substr($WorkString, 0, $i));
   $sum += $xi;
   $mean = $sum / $N;

   $WorkString = trim(substr($WorkString, $i, strlen($WorkString)));
  }

   $N = $sum = 0;

   while ($w != '')
  {
   $N++;

   $i = strpos($w, ' ');

   if ($i === FALSE)
      {
       $diff = $w - $mean;
       $sum += ($diff * $diff);
       break;
      }

   $xi = trim(substr($w, 0, $i));
   $diff = $xi - $mean;
   $sum += ($diff * $diff);
   $w = trim(substr($w, $i, strlen($w)));
  }
   $variance = $sum / ($N - $SampleMode);

   return sqrt($variance);

} // End of  Std_Deviation(...)







/*
 ------------------------------------------------------------------------------
 This function computes the standard statistics for a numerical data string.
 The returned string contains the following computed statistics:
 n                  = Number of data elements (numbers to be processed)
 $sum               = Sum of all the data
 $mean              = Arithmetic mean of the data
 $SUMsquaresOfDiffs = Sum of the squares of the differences
 $variance0         = Population variance
 $variance1         = Sample variance
 $StdDev0           = Population standard deviation
 $StdDev1           = Sample standard deviation";
*/

   function Std_Dev_Stats ($NumericDataString)
{
   $w = str_replace(',',  ' ', $NumericDataString);
   $w = str_replace(';',  ' ', $w);
   $w = str_replace('\n', ' ', $w);
   $WorkString = preg_replace('/\s+/', ' ', trim($w));

   $n = $SUMdata = 0;

   while ($WorkString != '')
  {
   $n++;

   $i = strpos($WorkString, ' ');

   if ($i === FALSE)
      {
       $SUMdata += $WorkString;
       $mean = $SUMdata / $n;
       break;
      }

   $xi = trim(substr($WorkString, 0, $i));
   $SUMdata += $xi;
   $mean = $SUMdata / $n;

   $WorkString = trim(substr($WorkString, $i, strlen($WorkString)));
  }

// *************************************************************************
// Cycle through data computing the sum of squares of differences from mean.
// The mean of this summation of squares is the population variance.

   $w = str_replace(",", " ", $NumericDataString);
   $w = str_replace(";", " ", $w);
   $WorkString = preg_replace('/\s+/', ' ', trim($w));

   $n = $SUMsquaresOfDiffs = 0;

   while ($WorkString != '')
  {
   $n++;

   $i = strpos($WorkString, ' ');

   if ($i === FALSE)
      {
       $diff = $WorkString - $mean;
       $SUMsquaresOfDiffs += ($diff * $diff);
       break;
      }

   $xi = trim(substr($WorkString, 0, $i));
   $diff = $xi - $mean;
   $SUMsquaresOfDiffs += ($diff * $diff);
   $WorkString = trim(substr($WorkString, $i, strlen($WorkString)));
  }

   $variance0 = $SUMsquaresOfDiffs / ($n-0); // sigma
   $variance1 = $SUMsquaresOfDiffs / ($n-1); // s

   $StdDev0 = sqrt($variance0); // SqRt(sigma)
   $StdDev1 = sqrt($variance1); // SqRt(s)

   $StdDevStats = "$n $SUMdata $mean $SUMsquaresOfDiffs $variance0 $variance1 $StdDev0 $StdDev1";

   return $StdDevStats;

} // End of  Std_Dev_Stats(...)



/*
   ========================================================================
   Returns the compass symbol corresponding to azimuth angle in degrees and
   zero-mode.  The default zero azimuth direction is north.

   ZeroModeSymbol = 'N' (North = 0 convention = Default)
   ZeroModeSymbol = 'S' (South = 0 convention = Optional)

   ========================================================================
*/

   function Compass_Symbol ($AzimDeg, $ZeroModeSymbol='N')
{
   $a = floatval($AzimDeg) + ((strtoupper($ZeroModeSymbol) == 'S')? 180:0);

   $a -= 360*floor($a/360);

   $i = ($a < 11.25 or $a > 348.75)? 0 : floor(0.5 + $a/22.5);

   if ($ZeroModeSymbol != 'S')
   {return trim(substr("N  NNENE ENEE  ESESE SSES  SSWSW WSWW  WNWNW NNWN", 3*$i, 3));}
   else
   {return trim(substr("S  SSWSW WSWW  WNWNW NNWN  NNENE ENEE  ESESE SSES", 3*$i, 3));}

} // End of  Compass_Symbol (...)



/*
  ========================================================================
  Computes the theoretical astronomical Delta-T estimate in seconds for
  any dates between 1600 and 2200, based on the NASA polymomial
  expressions for Delta T.

  NASA Ref:
  http://eclipse.gsfc.nasa.gov/LEcat5/deltatpoly.html


  ARGUMENTS:
  Y = Year number (1600 to 2200 AD)
  mNum = Month number(01 to 12)
  EXAMPLE: Ym = 195005 (= 1950 May)

  ERRORS:
  FALSE is returned on error.  An error results if either
  the year or month number is invalid.  A zero year value
  will return an error.
  ========================================================================
*/

   function Delta_T ($yyyymnOrJDTT, $RetMode=' ')
{
// ERROR if date value is non-numeric.
// It must be a positive 6-digit integer value.
// yyyymn = 100*Y + m
//
// Modifications.
// Can also use a 7-digit JD number.
// A JD number will be used to determine the year and month values.
//
// Now has a DMS return mode to save work.
// HMS = Return XXh XXm XXs
//  :  = Return hh:mm:ss

// --------------------------------------------
// Read return mode.  Empty = Default = Seconds
// This is CASE SENSITIVE
//  H = Return XXh XXm XXs
//  h = H.hhh
//  : = Return hh:mm:ss

   $RMode = substr(trim($RetMode), 0,1);  if ($RMode == '') {$RMode = ' ';}

   $dTSeconds = _Delta_T ($yyyymnOrJDTT);

   $DeltaT = sprintf("%+3.0f", round($dTSeconds, 0));

   if ($DeltaT === FALSE){return FALSE;}

   if ($RMode == ':')
  {
   $dTHrs = $dTSeconds/3600;
   $DeltaT = Hours_to_HMS($dTHrs, 0, '+', ':');
  }

   if ($RMode == 'h')
  {
   $DeltaT = sprintf("%+1.9f", $dTSeconds/3600);
  }

   if ($RMode == 'H')
  {
   $dTHrs = $dTSeconds/3600;
   $DeltaT = Hours_to_HMS($dTHrs, 0, '+', $RMode);
  }

// Return Delta T in indicated mode.
   return $DeltaT;

} // End of  Delta_T(...)

/*
   This is a subroutine for the upper-level
   Delta T function above that calls it.
*/
   FUNCTION _Delta_T ($yyyymnOrJDTT)
{
// ERROR if date value is non-numeric.
// It must be a positive 6-digit or 7-digit integer value.
// yyyymn = 100*Y + m
// Or 7-digit JD number.
// The JD number will be used to determine the year and month values.
//
// Now has a DMS return mode to save work.
// HMS = Return XXh XXm XXs
//  :  = Return hh:mm:ss

// Read input argument.
   $w = abs(floatval(trim($yyyymnOrJDTT)));

// ERROR returns FALSE if argument is invalid value.
   if ($w == 0) {return FALSE;}

// If argument contains a decimal point, then it is assumed to be a
// Julian date value and is converted into the corresponding JD number.
   if (strpos($w, '.') !== FALSE) {$w = floor($w + 0.5);}

// AT THIS POINT, w SHOULD BE EITHER A 6 OR 7-DIGIT POSITIVE INTEGER.
//
// 6 digits = Integer-encoded date elements yyyymn = 100*yyyy + mn
// 7 digits = JD number corresponding to Julian Date argument value.

// ERROR returns FALSE if either argument is neither 6 or 7 digits.
   if (strlen($w) < 6 or strlen($w) > 7) {return FALSE;}


// HANDLE CASE OF JULIAN DATE OR JD NUMBER ARGUMENT.
// If 7-digit integer, then compute corresponding
// mn, dn, yyyy values. The returned dn is not used.
   if (strlen($w) == 7)
  {
   list($mn, $dn, $yyyy) = preg_split("[\/]", JDtoGregorian($w));

   $w = 100*$yyyy + $mn;
  }

// Otherwise, it is assumed that the argument is a 6-digit yyyymn value.
   $yyyymn = $w;

// Parse yyyymn value.

   $Y = substr($yyyymn, 0,4);
   $mNum = intval(substr($yyyymn, 4,2));

// ERROR if year value is invalid. It must
// be in the range from 1600 to 2200.

   if ($Y < 1600 or $Y == 0 or $Y > 2200) {return FALSE;}

// ERROR if month value is outside the range from 1 to 12.

   if ($mNum < 1 or $mNum > 12) {return FALSE;}

// If a negative (BC) year is given , it is converted into the
// corresponding mathematical year by adding 1.
//
// The year 100 BC (-100), becomes the mathematical year -99.
// This will never be the case here because this implementation
// of the polynomials is not meant to cover ancient dates that
// use negative year values, only years from 1600 AD onward.

   $y = $Y + (($Y < 0)? 1:0) + ($mNum - 0.5)/12;


   if ($Y >= 1600 and $Y < 1700)
      {
       $u  = $y - 1600;

       return 120 - 0.9808*$u - 0.01532*$u*$u + $u*$u*$u/7129;
      }


   if ($Y >= 1700 and $Y < 1800)
      {
       $u  = $y - 1700;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;

       return 8.83 + 0.1603*$u
                   - 0.0059285*$u2
                   + 0.00013336*$u3
                   - $u4/1174000;
      }


   if ($Y >= 1800 and $Y < 1860)
      {
       $u  = $y - 1800;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;
       $u6 = $u * $u5;
       $u7 = $u * $u6;

       return 13.72 - 0.332447*$u
                    + 0.0068612*$u2
                    + 0.0041116*$u3
                    - 0.00037436*$u4
                    + 0.0000121272*$u5
                    - 0.0000001699*$u6
                    + 0.000000000875*$u7;
      }


   if ($Y >= 1860 and $Y < 1900)
      {
       $u  = $y - 1860;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;

       return 7.62 + 0.5737*$u
                   - 0.251754*$u2
                   + 0.01680668*$u3
                   - 0.0004473624*$u4
                   + $u5 / 233174;
      }


   if ($Y >= 1900 and $Y < 1920)
      {
       $u  = $y - 1900;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;

       return 1.494119*$u - 2.79
                          - 0.0598939*$u2
                          + 0.0061966*$u3
                          - 0.000197*$u4;
      }


   if ($Y >= 1920 and $Y < 1941)
      {
       $u  = $y - 1920;
       $u2 = $u * $u;
       $u3 = $u * $u2;

       return 21.20 + 0.84493*$u - 0.076100*$u2 + 0.0020936*$u3;
      }


   if ($Y >= 1941 and $Y < 1961)
      {
       $u  = $y - 1950;
       $u2 = $u * $u;
       $u3 = $u * $u2;

       return 29.07 + 0.407*$u - $u2/233 + $u3/2547;
      }


   if ($Y >= 1961 and $Y < 1986)
      {
       $u = $y - 1975;
       $u2 = $u * $u;
       $u3 = $u * $u2;

       return 45.45 + 1.067*$u - $u2/260 - $u3/718;
      }


   if ($Y >= 1986 and $Y < 2005)
      {
       $u  = $y - 2000;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;

       return 63.86 + 0.3345*$u
                    - 0.060374*$u2
                    + 0.0017275*$u3
                    + 0.000651814*$u4
                    + 0.00002373599*$u5;
      }


   if ($Y >= 2005 and $Y < 2050)
      {
       $u  = $y - 2000;

       return 62.92 + 0.32217*$u + 0.005589*$u*$u;
      }


   if ($Y >= 2050 and $Y <= 2150)
      {
       $u  = ($y - 1820) / 100;

       return 32*$u*$u - 0.5628*(2150 - $y) - 20;
      }


   if ($Y > 2150 and $Y <= 3000)
      {
       $u  = ($y - 1820) / 100;

       return 32*$u*$u - 20;
      }

   return FALSE;

} // End of  _Delta_T(...)



/*
   NASA Delta T function for the years 1600 TO 2200.

   DATE FORMATS = (yyyymn, yyyymndn, xxxxxxx, xxxxxxx.xxx)
   xxxxxxx.xxx  = JD TT value = Converted to nearest JD number
   xxxxxxx      = 7 digit integer = JD number = Compute (yyyymn)
   yyyymndn     = 8 digit integer = Converted to yyyymn
   yyyymn       = 6 digit integer

   ReturnMode   = 0, 1, 2
   0 = Default  = Raw full seconds value  (+430.90045)
   1 = Seconds rounded to nearest second (+431)
   2 = hh:mm:ss mode to nearest second   (+00:07:11)
*/

   function NASA_Delta_T ($yyyymnOrJDTT, $ReturnMode=0)
{
   $w = trim($yyyymnOrJDTT);

   if (is_numeric($w) and ($w != floor($w)))
  {
   $w = floor($w + 0.5);
  }

   if (is_numeric($w) and strlen($w) == 7)
  {
   list($mn, $dn, $yyyy) = preg_split("[\/]", JDtoGregorian($w));
   $w = 100*$yyyy + $mn;
  }

   if (is_numeric($w) and strlen($w) == 8)
  {
   $w = substr($w, 0,6);
  }

   if (is_numeric($w) and strlen($w) != 6)
      {return FALSE;}
   else
      {$yyyymn = $w;}

   $Y    = substr($yyyymn, 0,4);
   $mNum = intval(substr($yyyymn, 4,2));

   if ($Y < 1600 or $Y == 0 or $Y > 2200) {return FALSE;}

   if ($mNum < 1 or $mNum > 12) {return FALSE;}

   $y = $Y + ($mNum - 0.5)/12;



   if ($Y >= 1600 and $Y < 1700)
      {
       $u  = $y - 1600;

       $DeltaT = 120 - 0.9808*$u - 0.01532*$u*$u + $u*$u*$u/7129;
      }


   if ($Y >= 1700 and $Y < 1800)
      {
       $u  = $y - 1700;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;

       $DeltaT = 8.83 + 0.1603*$u
                   - 0.0059285*$u2
                   + 0.00013336*$u3
                   - $u4/1174000;
      }


   if ($Y >= 1800 and $Y < 1860)
      {
       $u  = $y - 1800;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;
       $u6 = $u * $u5;
       $u7 = $u * $u6;

       $DeltaT = 13.72 - 0.332447*$u
                    + 0.0068612*$u2
                    + 0.0041116*$u3
                    - 0.00037436*$u4
                    + 0.0000121272*$u5
                    - 0.0000001699*$u6
                    + 0.000000000875*$u7;
      }


   if ($Y >= 1860 and $Y < 1900)
      {
       $u  = $y - 1860;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;

       $DeltaT = 7.62 + 0.5737*$u
                   - 0.251754*$u2
                   + 0.01680668*$u3
                   - 0.0004473624*$u4
                   + $u5 / 233174;
      }


   if ($Y >= 1900 and $Y < 1920)
      {
       $u  = $y - 1900;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;

       $DeltaT = 1.494119*$u - 2.79
                          - 0.0598939*$u2
                          + 0.0061966*$u3
                          - 0.000197*$u4;
      }


   if ($Y >= 1920 and $Y < 1941)
      {
       $u  = $y - 1920;
       $u2 = $u * $u;
       $u3 = $u * $u2;

       $DeltaT = 21.20 + 0.84493*$u - 0.076100*$u2 + 0.0020936*$u3;
      }


   if ($Y >= 1941 and $Y < 1961)
      {
       $u  = $y - 1950;
       $u2 = $u * $u;
       $u3 = $u * $u2;

       $DeltaT = 29.07 + 0.407*$u - $u2/233 + $u3/2547;
      }


   if ($Y >= 1961 and $Y < 1986)
      {
       $u = $y - 1975;
       $u2 = $u * $u;
       $u3 = $u * $u2;

       $DeltaT = 45.45 + 1.067*$u - $u2/260 - $u3/718;
      }


   if ($Y >= 1986 and $Y < 2005)
      {
       $u  = $y - 2000;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;

       $DeltaT = 63.86 + 0.3345*$u
                    - 0.060374*$u2
                    + 0.0017275*$u3
                    + 0.000651814*$u4
                    + 0.00002373599*$u5;
      }


   if ($Y >= 2005 and $Y < 2050)
      {
       $u  = $y - 2000;

       $DeltaT = 62.92 + 0.32217*$u + 0.005589*$u*$u;
      }


   if ($Y >= 2050 and $Y <= 2150)
      {
       $u  = ($y - 1820) / 100;

       $DeltaT = 32*$u*$u - 0.5628*(2150 - $y) - 20;
      }


   if ($Y > 2150 and $Y <= 3000)
      {
       $u  = ($y - 1820) / 100;

       $DeltaT = 32*$u*$u - 20;
      }

// Return Delta T value according to (ReturnMode).

   $NumSign = ($DeltaT < 0)? '-':'+';  $DeltaT = abs($DeltaT);

   if ($ReturnMode <= 0) {return "$NumSign$DeltaT";}

   if ($ReturnMode == 1) {return  $NumSign.round($DeltaT, 0);}

   if ($ReturnMode >= 2)
      {
       $hours = ($DeltaT/3600);
       $hh = floor($hours);
       $minutes = 60*($hours - $hh);
       $mm = floor($minutes);
       $ss = round(60*($minutes - $mm), 0);
       if ($ss == 60){$mm++; $ss=0;}
       if ($mm == 60){$hh++; $mm=0;}
       $hh = sprintf("%02d", $hh);
       $mm = sprintf("%02d", $mm);
       $ss = sprintf("%02d", $ss);
       return "$NumSign$hh:$mm:$ss";
      }

   return FALSE;

} // END OF  NASA_Delta_T(...)



/*
   Return mode 1 returns full polynomial result in seconds (444.413605555556).
   Return mode 2 (default) returns result resolved to nearest second (+444).
   Return mode 3 returns result as ±HH:MM:SS string (+00:07:24).
   Return mode 4 returns result as ±h m s string (+00h 07m 24s).
*/

   function xNASA_Delta_T ($JDTT_or_SignedYmn, $RetMode=2)
{
// _________________________________________________
// Get date elements corresponding to JDTT argument.
// Only the year and month values are needed or used.

   $w = trim($JDTT_or_SignedYmn);

   $dT = FALSE;

   if (trim($RetMode) == '' or intval(trim($RetMode)) < 1) {$RetMode = 1;}

   if (trim($RetMode) > 4) {$RetMode = 4;}

// ___________________________________________________________________
// If there is no numerical sign, then assume it is a JD (Julian Date)
// value and compute the calendrical JD number and date elements in
// accordance with whether Julian or Gregorian calendar date mode.

   if(substr($w,0,1) <> '-' and substr($w,0,1) <> '+')
  {
// Determine if Julian or Gregorian year
   $JDNum = floor($w + 0.5);
   $JorGmode = ($JDNum < 2299161)? 'J':'G';

// Compute year and month elements from the JD number,
// according to whether Julian or Gregorian date mode.
   $mdYStr = ($JorGmode == 'J')? JDtoJulian($JDNum) : JDtoGregorian($JDNum);

   list($m, $d, $Y) = preg_split("[\/]", $mdYStr);

   $Y  = intval($Y);
   $mn = ($m < 10)? "0$m" : $m;
  }

// __________________________________________________________________
// If there is a numerical sign, assume it is an integer-encoded date
// and compute the corresponding JDTT.  It may be ±yyyymndd ±yyyymn.

   if(substr($w,0,1) == '-' or substr($w,0,1) == '+')
  {
   $Y  = intval(substr($w, 0, strlen($w)-2));
   $mn = substr($w, -2);
   $m  = intval($mn);

// Determine if Julian or Gregorian calendar date mode.
   $JorGmode = ($Y < 1582 or ($Y == 1582 and $m < 10))? 'J':'G';
  }

// At this point, we have the arguments (Y, m) required
// to compute the Delta T estimate for the given year
// and month.  The day of the month is not needed or
// used for this Delta T estimate.
//
// The computed  Delta T estimate is for mid-month and
// should be resolved to no better than to the nearest
// second.

// ___________________________________________
// ERROR if year value is invalid.  It must be
// in the range from -2000 to +3000.

   if ($Y < -2000 or $Y == 0 or $Y > 3000) {return FALSE;}

// _______________________________________________________
// ERROR if month value is outside the range from 1 to 12.

   if ($m < 1 or $m > 12) {return FALSE;}

// ___________________________________________________________
// If a negative (BC) year is given , it is converted into the
// corresponding mathematical year by adding 1 to (Y).
//
// The year 100 BC (-100), becomes the mathematical year -99
// for proper use within internal computations.

// Compute working (y) value for mid-month (m).
   $y = $Y + (($Y < 0)? 1:0) + ($m - 0.5)/12;

// _________________________________________________________________
// Perform NASA Delta T polynomial computations using (Y, y) values.

   if ($Y >= -2000 and $Y < -500)
      {
       $u  = ($Y - 1820) / 100;

     $dTSec = 32*$u*$u - 20;
      }


   if ($Y >= -500 and $Y < 500)
      {
       $u  = $y/100;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;
       $u6 = $u * $u5;

     $dTSec = 10583.6 - 1014.41*$u
                        + 33.78311*$u2
                        -  5.952053*$u3
                        -  0.1798452*$u4
                        +  0.022174192*$u5
                        +  0.0090316521*$u6;
      }


   if ($Y >= 500 and $Y < 1600)
      {
       $u  = ($y - 1000)/100;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;
       $u6 = $u * $u5;

     $dTSec = 1574.2 - 556.01*$u
                     +  71.23472*$u2
                     +   0.319781*$u3
                     -   0.8503463*$u4
                     -   0.005050998*$u5
                     +   0.0083572073*$u6;
      }


   if ($Y >= 1600 and $Y < 1700)
      {
       $u  = $y - 1600;

     $dTSec = 120 - 0.9808*$u - 0.01532*$u*$u + $u*$u*$u/7129;
      }


   if ($Y >= 1700 and $Y < 1800)
      {
       $u  = $y - 1700;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;

     $dTSec = 8.83 + 0.1603*$u
                   - 0.0059285*$u2
                   + 0.00013336*$u3
                   - $u4/1174000;
      }


   if ($Y >= 1800 and $Y < 1860)
      {
       $u  = $y - 1800;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;
       $u6 = $u * $u5;
       $u7 = $u * $u6;

     $dTSec = 13.72 - 0.332447*$u
                    + 0.0068612*$u2
                    + 0.0041116*$u3
                    - 0.00037436*$u4
                    + 0.0000121272*$u5
                    - 0.0000001699*$u6
                    + 0.000000000875*$u7;
      }


   if ($Y >= 1860 and $Y < 1900)
      {
       $u  = $y - 1860;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;

     $dTSec = 7.62 + 0.5737*$u
                   - 0.251754*$u2
                   + 0.01680668*$u3
                   - 0.0004473624*$u4
                   + $u5 / 233174;
      }


   if ($Y >= 1900 and $Y < 1920)
      {
       $u  = $y - 1900;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;

     $dTSec = 1.494119*$u - 2.79
                          - 0.0598939*$u2
                          + 0.0061966*$u3
                          - 0.000197*$u4;
      }


   if ($Y >= 1920 and $Y < 1941)
      {
       $u  = $y - 1920;
       $u2 = $u * $u;
       $u3 = $u * $u2;

     $dTSec = 21.20 + 0.84493*$u - 0.076100*$u2 + 0.0020936*$u3;
      }


   if ($Y >= 1941 and $Y < 1961)
      {
       $u  = $y - 1950;
       $u2 = $u * $u;
       $u3 = $u * $u2;

     $dTSec = 29.07 + 0.407*$u - $u2/233 + $u3/2547;
      }


   if ($Y >= 1961 and $Y < 1986)
      {
       $u = $y - 1975;
       $u2 = $u * $u;
       $u3 = $u * $u2;

     $dTSec = 45.45 + 1.067*$u - $u2/260 - $u3/718;
      }


   if ($Y >= 1986 and $Y < 2005)
      {
       $u  = $y - 2000;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;

     $dTSec = 63.86 + 0.3345*$u
                    - 0.060374*$u2
                    + 0.0017275*$u3
                    + 0.000651814*$u4
                    + 0.00002373599*$u5;
      }


   if ($Y >= 2005 and $Y < 2050)
      {
       $u  = $y - 2000;

     $dTSec = 62.92 + 0.32217*$u + 0.005589*$u*$u;
      }


   if ($Y >= 2050 and $Y <= 2150)
      {
       $u  = ($y - 1820) / 100;

     $dTSec = 32*$u*$u - 0.5628*(2150 - $y) - 20;
      }


   if ($Y > 2150 and $Y <= 3000)
      {
       $u  = ($y - 1820) / 100;

     $dTSec = 32*$u*$u - 20;
      }

// Format returned value according to return mode.
   if ($RetMode == 1) {return $dTSec;}

   if ($RetMode == 2) {return sprintf("%+d", round($dTSec, 0));}

   if ($RetMode == 3 or $RetMode == 4)
      {
       $NumSign = ($dTSec < 0)? '-':'+';
       $hours = ($dTSec/3600);
       $hh = floor($hours);
       $minutes = 60*($hours - $hh);
       $mm = floor($minutes);
       $ss = round(60*($minutes - $mm), 0);
       if ($ss == 60){$mm++; $ss=0;}
       if ($mm == 60){$hh++; $mm=0;}
       $hh = sprintf("%02d", $hh);
       $mm = sprintf("%02d", $mm);
       $ss = sprintf("%02d", $ss);
       if ($RetMode == 4)
       {$dThhmmss = "$NumSign$hh"."h $mm"."m $ss"."s";}
       else
       {$dThhmmss = "$NumSign$hh:$mm:$ss";}

       return $dThhmmss;
      }

   return $dTSec;

} // END OF  NASA_Delta_T (...)











/*
  ========================================================================
  Computes the nutation in obliquity (delta epsilon) in
  degrees according to the IAU 2000B nutation series.

  This value is applied to the mean obliquity of the ecliptic to
  obtain the true, apparent obliquity.

  The series consists of 77 terms.

  ARGUMENT:
  $JD = JD number at the moment (TDB) for which the
        nutation in obliquity is required.

  ERRORS:
  No error checking.

  ========================================================================
*/


// THIS WILL BE NEW FUTURE VERSION TO REPLACE
// THE DEPRECATED dEps() FUNCTION VERSION.
// BOTH ARE FUNCTIONALLY IDENTICAL.
   function Delta_Eps ($JDTT)
  {
   return dEps ($JDTT);
  }

   FUNCTION dEps ($JDTT)
{
   $T = ($JDTT - 2451545) / 36525;

   $T2 = $T * $T;
   $T3 = $T * $T2;
   $T4 = $T * $T3;

// -------------------------------------------
// Compute mean anomaly of the Moon in radians

   $L  = deg2rad((485868.249036 + 1717915923.2178*$T + 31.8792*$T2
       + 0.051635*$T3 - 0.0002447*$T4) / 3600);


// ------------------------------------------
// Compute mean anomaly of the Sun in radians

   $Lp = deg2rad((1287104.79305 + 129596581.0481*$T
       - 0.5532*$T2  + 0.000136*$T3 - 0.00001149*$T4) / 3600);


// ------------------------------------------------------------
// Compute mean argument of the latitude of the Moon in radians

   $F  = deg2rad((335779.526232 + 1739527262.8478*$T
       - 12.7512*$T2 - 0.001037*$T3 + 0.00000417*$T4) / 3600);


// -----------------------------------------------------------
// Compute mean elongation of the Moon from the Sun in radians

   $D  = deg2rad((1072260.70369 + 1602961601.2090*$T
       - 6.3706*$T2  + 0.006593*$T3 - 0.00003169*$T4) / 3600);


// ------------------------------------------------------------------------
// Compute mean longitude of the mean ascending node of the Moon in radians

   $Om = deg2rad((450160.398036 - 6962890.5431*$T
       + 7.4722*$T2  + 0.007702*$T3 - 0.00005939*$T4) / 3600);

// --------------------------------------------------------------
// Sum series for nutation in obliquity (dEps) in arc sec * 10E+7

$s = 0;
$s += (92052331 + 9086*$T)*cos($Om) + 15377*sin($Om);
$s += (5730336 - 3015*$T)*cos(2*($F - $D + $Om)) - 4587*sin(2*($F - $D + $Om));
$s += (978459 - 485*$T)*cos(2*($F + $Om)) + 1374*sin(2*($F + $Om));
$s += (-897492 + 470*$T)*cos(2*$Om) - 291*sin(2*$Om);
$s += (73871 - 184*$T)*cos($Lp) - 1924*sin($Lp);
$s += (224386 - 677*$T)*cos($Lp + 2*($F - $D + $Om)) - 174*sin($Lp + 2*($F - $D + $Om));
$s -= 6750*cos($L) - 358*sin($L);
$s += (200728 + 18*$T)*cos(2*$F + $Om) + 318*sin(2*$F + $Om);
$s += (129025 - 63*$T)*cos($L + 2*($F + $Om)) + 367*sin($L + 2*($F + $Om));
$s += (-95929 + 299*$T)*cos(2*($F - $D + $Om) - $Lp) + 132*sin(2*($F - $D + $Om) - $Lp);
$s += (-68982 - 9*$T)*cos(2*($F - $D) + $Om) + 39*sin(2*($F - $D) + $Om);
$s += (-53311 + 32*$T)*cos(2*($F + $Om) - $L) - 4*sin(2*($F + $Om) - $L);
$s -= 1235*cos(2*$D - $L) - 82*sin(2*$D - $L);
$s -= 33228*cos($L + $Om) + 9*sin($L + $Om);
$s += 31429*cos($Om - $L) - 75*sin($Om - $L);
$s += (25543 - 11*$T)*cos(2*($F + $D + $Om) - $L) + 66*sin(2*($F + $D + $Om) - $L);
$s += 26366*cos($L + 2*$F + $Om) + 78*sin($L + 2*$F + $Om);
$s += (-24236 - 10*$T)*cos(2*($F - $L) + $Om) + 20*sin(2*($F - $L) + $Om);
$s -= 1220*cos(2*$D) - 29*sin(2*$D);
$s += (16452 - 11*$T)*cos(2*($F + $D + $Om)) + 68*sin(2*($F + $D + $Om));
$s -= 13870*cos(-2*($Lp - $F + $D - $Om));
$s += 477*cos(2*($D - $L)) - 25*sin(2*($D - $L));
$s += (13238 - 11*$T)*cos(2*($L + $F + $Om)) + 59*sin(2*($L + $F + $Om));
$s += (-12338 + 10*$T)*cos($L + 2*($F - $D + $Om)) - 3*sin($L + 2*($F - $D + $Om));
$s -= 10758*cos(2*$F + $Om - $L) + 3*sin(2*$F + $Om - $L);
$s -= 609*cos(2*$L) - 13*sin(2*$L);
$s -= 550*cos(2*$F) - 11*sin(2*$F);
$s += (8551 - 2*$T)*cos($Lp + $Om) - 45*sin($Lp + $Om);
$s -= 8001*cos(-$L + 2*$D + $Om) + sin(-$L + 2*$D + $Om);
$s += (6850 - 42*$T)*cos(2*($Lp + $F - $D + $Om)) - 5*sin(2*($Lp + $F - $D + $Om));
$s -= 167*cos(2*($D - $F)) - 13*sin(2*($D - $F));
$s += 6953*cos($L - 2*$D + $Om) - 14*sin($L - 2*$D + $Om);
$s += 6415*cos($Om - $Lp) + 26*sin($Om - $Lp);
$s += 5222*cos(2*($F + $D) + $Om - $L) + 15*sin(2*($F + $D) + $Om - $L);
$s += (168 - $T)*cos(2*$Lp) + 10*sin(2*$Lp);
$s += 3268*cos($L + 2*($F + $D + $Om)) + 19*sin($L + 2*($F + $D + $Om));
$s += 104*cos(2*($F - $L)) + 2*sin(2*($F - $L));
$s -= 3250*cos($Lp + 2*($F + $Om)) + 5*sin($Lp + 2*($F + $Om));
$s += 3353*cos(2*($F + $D) + $Om) + 14*sin(2*($F + $D) + $Om);
$s += 3070*cos(2*($F + $Om) - $Lp) + 4*sin(2*($F + $Om) - $Lp);
$s += 3272*cos(2*$D + $Om) + 4*sin(2*$D + $Om);
$s -= 3045*cos($L + 2*($F - $D) + $Om) + sin($L + 2*($F - $D) + $Om);
$s -= 2768*cos(2*($L + $F - $D + $Om)) + 4*sin(2*($L + $F - $D + $Om));
$s += 3041*cos(2*($D - $L) + $Om) - 5*sin(2*($D - $L) + $Om);
$s += 2695*cos(2*($L + $F) + $Om) + 12*sin(2*($L + $F) + $Om);
$s += 2719*cos(2*($F - $D) + $Om - $Lp) - 3*sin(2*($F - $D) + $Om - $Lp);
$s += 2720*cos($Om - 2*$D) - 9*sin($Om - 2*$D);
$s -= 51*cos(-$L - $Lp + 2*$D) - 4*sin(-$L - $Lp + 2*$D);
$s -= 2206*cos(2*($L - $D) + $Om) - sin(2*($L - $D) + $Om);
$s -= 199*cos($L + 2*$D) - 2*sin($L + 2*$D);
$s -= 1900*cos($Lp + 2*($F - $D) + $Om) - sin($Lp + 2*($F - $D) + $Om);
$s -= 41*cos($L - $Lp) - 3*sin($L - $Lp);
$s += 1313*cos(-2*($L - $F - $Om)) - sin(-2*($L - $F - $Om));
$s += 1233*cos(3*$L + 2*($F + $Om)) + 7*sin(3*$L + 2*($F + $Om));
$s -= 81*cos(-$Lp + 2*$D) - 2*sin(-$Lp + 2*$D);
$s += 1232*cos($L - $Lp + 2*($F + $Om)) + 4*sin($L - $Lp + 2*($F + $Om));
$s -= 20*cos($D) + 2*sin($D);
$s += 1207*cos(2*($F + $D + $Om) - $L - $Lp) + 3*sin(2*($F + $D + $Om) - $L - $Lp);
$s += 40*cos(2*$F - $L) - 2*sin(2*$F - $L);
$s += 1129*cos(-$Lp + 2*($F + $D + $Om)) + 5*sin(-$Lp + 2*($F + $D + $Om));
$s += 1266*cos($Om - 2*$L) - 4*sin($Om - 2*$L);
$s -= 1062*cos($L + $Lp + 2*($F + $Om)) + 3*sin($L + $Lp + 2*($F + $Om));
$s -= 1129*cos(2*$L + $Om) + 2*sin(2*$L + $Om);
$s -= 9*cos($Lp + $D - $L);
$s += 35*cos($L + $Lp) - 2*sin($L + $Lp);
$s -= 107*cos($L + 2*$F) - sin($L + 2*$F);
$s += 1073*cos(2*($F - $D) + $Om - $L) - 2*sin(2*($F - $D) + $Om - $L);
$s += 854*cos($L + 2*$Om);
$s -= 553*cos($D - $L) + 139*sin($D - $L);
$s -= 710*cos(2*($F + $Om) + $D) + 2*sin(2*($F + $Om) + $D);
$s += 647*cos(2*($F + 2*$D + $Om) - $L) + 4*sin(2*($F + 2*$D + $Om) - $L);
$s -= 700*cos($Lp + $D + $Om - $L);
$s += 672*cos(-2*($Lp - $F + $D) + $Om);
$s += 663*cos($L + 2*($F + $D) + $Om) + 4*sin($L + 2*($F + $D) + $Om);
$s -= 594*cos(-2*($L - $F - $D - $Om)) + 2*sin(-2*($L - $F - $D - $Om));
$s -= 610*cos(2*$Om - $L) - 2*sin(2*$Om - $L);
$s -= 556*cos($L + $Lp + 2*($F - $D + $Om));

// ----------------------------
// Return nutation in obliquity
// expressed in degrees.

   return $s / 36000000000;


} // End of  dEps(...)



/*

   ========================================================================
   Computes the nutation in ecliptical longitude (Delta psi)
   in degrees according to the IAU 2000B nutation series.

   This value is applied to the mean ecliptical longitude to obtain the
   true, apparent ecliptical longitude.

   The series consists of 77 terms.

   ARGUMENT:
   $JDTT = Astronomical JD number corresponding to date and time (TT).

   ERRORS:
   No error checking.
   ========================================================================
*/


   function Delta_Psi($JDTT)
  {
   return dPsi ($JDTT);
  }

   function dPsi ($JDTT)
{
// ---------------------------------------------
// Compute time (T) in Julian centuries reckoned
// from J2000.0 and 4 powers of T.

   $T  = ($JDTT - 2451545) / 36525;

   $T2 = $T * $T;
   $T3 = $T * $T2;
   $T4 = $T * $T3;

// -------------------------------------------
// Compute mean anomaly of the Moon in radians

   $L  = deg2rad((485868.249036 + 1717915923.2178*$T + 31.8792*$T2
       + 0.051635*$T3 - 0.0002447*$T4) / 3600);


// ------------------------------------------
// Compute mean anomaly of the Sun in radians

   $Lp = deg2rad((1287104.79305 + 129596581.0481*$T
       - 0.5532*$T2  + 0.000136*$T3 - 0.00001149*$T4) / 3600);


// ------------------------------------------------------------
// Compute mean argument of the latitude of the Moon in radians

   $F  = deg2rad((335779.526232 + 1739527262.8478*$T
       - 12.7512*$T2 - 0.001037*$T3 + 0.00000417*$T4) / 3600);


// -----------------------------------------------------------
// Compute mean elongation of the Moon from the Sun in radians

   $D  = deg2rad((1072260.70369 + 1602961601.2090*$T
       - 6.3706*$T2  + 0.006593*$T3 - 0.00003169*$T4) / 3600);


// ------------------------------------------------------------------------
// Compute mean longitude of the mean ascending node of the Moon in radians

   $Om = deg2rad((450160.398036 - 6962890.5431*$T
       + 7.4722*$T2  + 0.007702*$T3 - 0.00005939*$T4) / 3600);

// -----------------------------------------------------------------------
// Sum 2000B series for nutation in longitude (dPsi) in arc sec * 10000000

$s = 0;
$s += (-172064161 - 174666*$T)*sin($Om) + 33386*cos($Om);
$s += (-13170906 - 1675*$T)*sin(2*($F - $D + $Om)) - 13696*cos(2*($F - $D + $Om));
$s += (-2276413 - 234*$T)*sin(2*($F + $Om)) + 2796*cos(2*($F + $Om));
$s += (2074554 + 207*$T)*sin(2*$Om) - 698*cos(2*$Om);
$s += (1475877 - 3633*$T)*sin($Lp) + 11817*cos($Lp);
$s += (-516821 + 1226*$T)*sin($Lp + 2*($F - $D + $Om)) - 524*cos($Lp + 2*($F - $D + $Om));
$s += (711159 + 73*$T)*sin($L) - 872*cos($L);
$s += (-387298 - 367*$T)*sin(2*$F + $Om) + 380*cos(2*$F + $Om);
$s += (-301461 - 36*$T)*sin($L + 2*($F + $Om)) + 816*cos($L + 2*($F + $Om));
$s += (215829 - 494*$T)*sin(2*($F - $D + $Om) - $Lp) + 111*cos(2*($F - $D + $Om) - $Lp);
$s += (128227 + 137*$T)*sin(2*($F - $D) + $Om) + 181*cos(2*($F - $D) + $Om);
$s += (123457 + 11*$T)*sin(2*($F + $Om) - $L) + 19*cos(2*($F + $Om) - $L);
$s += (156994 + 10*$T)*sin(2*$D - $L) - 168*cos(2*$D - $L);
$s += (63110 + 63*$T)*sin($L + $Om) + 27*cos($L + $Om);
$s += (-57976 - 63*$T)*sin($Om - $L) - 189*cos($Om - $L);
$s += (-59641 - 11*$T)*sin(2*($F + $D + $Om) - $L) + 149*cos(2*($F + $D + $Om) - $L);
$s += (-51613 - 42*$T)*sin($L + 2*$F + $Om) + 129*cos($L + 2*$F + $Om);
$s += (45893 + 50*$T)*sin(2*($F - $L) + $Om) + 31*cos(2*($F - $L) + $Om);
$s += (63384 + 11*$T)*sin(2*$D) - 150*cos(2*$D);
$s += (-38571 - $T)*sin(2*($F + $D + $Om)) + 158*cos(2*($F + $D + $Om));
$s += 32481*sin(2*($F - $Lp - $D + $Om));
$s -= 47722*sin(2*($D - $L)) + 18*cos(2*($D - $L));
$s += (-31046 - $T)*sin(2*($L + $F + $Om)) + 131*cos(2*($L + $F + $Om));
$s += 28593*sin($L + 2*($F - $D + $Om)) - cos($L + 2*($F - $D + $Om));
$s += (20441 + 21*$T)*sin(2*$F - $L + $Om) + 10*cos(2*$F - $L + $Om);
$s += 29243*sin(2*$L) - 74*cos(2*$L);
$s += 25887*sin(2*$F) - 66*cos(2*$F);
$s += (-14053 - 25*$T)*sin($Lp + $Om) + 79*cos($Lp + $Om);
$s += (15164 + 10*$T)*sin(2*$D - $L + $Om) + 11*cos(2*$D - $L + $Om);
$s += (-15794 + 72*$T)*sin(2*($Lp + $F - $D + $Om)) - 16*cos(2*($Lp + $F - $D + $Om));
$s += 21783*sin(2*($D - $F)) + 13*cos(2*($D - $F));
$s += (-12873 - 10*$T)*sin($L - 2*$D + $Om) - 37*cos($L - 2*$D + $Om);
$s += (-12654 + 11*$T)*sin($Om - $Lp) + 63*cos($Om - $Lp);
$s -= 10204*sin(2*($F + $D) - $L + $Om) - 25*cos(2*($F + $D) - $L + $Om);
$s += (16707 - 85*$T)*sin(2*$Lp) - 10*cos(2*$Lp);
$s -= 7691*sin($L + 2*($F + $D + $Om)) - 44*cos($L + 2*($F + $D + $Om));
$s -= 11024*sin(-2*$L + 2*$F) + 14*cos(2*($F - $L));
$s += (7566 - 21*$T)*sin($Lp + 2*($F + $Om)) - 11*cos($Lp + 2*($F + $Om));
$s += (-6637 - 11*$T)*sin(2*($F + $D) + $Om) + 25*cos(2*($F + $D) + $Om);
$s += (-7141 + 21*$T)*sin(2*($F + $Om) - $Lp) + 8*cos(2*($F + $Om) - $Lp);
$s += (-6302 - 11*$T)*sin(2*$D + $Om) + 2*cos(2*$D + $Om);
$s += (5800 + 10*$T)*sin($L + 2*($F - $D) + $Om) + 2*cos($L + 2*($F - $D) + $Om);
$s += 6443*sin(2*($L + $F - $D + $Om)) - 7*cos(2*($L + $F - $D + $Om));
$s += (-5774 - 11*$T)*sin(2*($D - $L) + $Om) - 15*cos(2*($D - $L) + $Om);
$s -= 5350*sin(2*($L + $F) + $Om) - 21*cos(2*($L + $F) + $Om);
$s += (-4752 - 11*$T)*sin(2*($F - $D) - $Lp + $Om) - 3*cos(2*($F - $D) - $Lp + $Om);
$s += (-4940 - 11*$T)*sin($Om - 2*$D) - 21*cos($Om - 2*$D);
$s += 7350*sin(2*$D - $L - $Lp) - 8*cos(2*$D - $L - $Lp);
$s += 4065*sin(2*($L - $D) + $Om) + 6*cos(2*($L - $D) + $Om);
$s += 6579*sin($L + 2*$D) - 24*cos($L + 2*$D);
$s += 3579*sin($Lp + 2*($F - $D) + $Om) + 5*cos($Lp + 2*($F - $D) + $Om);
$s += 4725*sin($L - $Lp) - 6*cos($L - $Lp);
$s -= 3075*sin(2*($F - $L  + $Om)) + 2*cos(2*($F - $L  + $Om));
$s -= 2904*sin(3*$L + 2*($F + $Om)) - 15*cos(3*$L + 2*($F + $Om));
$s += 4348*sin(2*$D - $Lp) - 10*cos(2*$D - $Lp);
$s -= 2878*sin($L - $Lp + 2*($F + $Om)) - 8*cos($L - $Lp + 2*($F + $Om));
$s -= 4230*sin($D) - 5*cos($D);
$s -= 2819*sin(2*($F + $D + $Om) - $L - $Lp) - 7*cos(2*($F + $D + $Om) - $L - $Lp);
$s -= 4056*sin(2*$F - $L) - 5*cos(2*$F - $L);
$s -= 2647*sin(2*($F + $D + $Om) - $Lp) - 11*cos(2*($F + $D + $Om) - $Lp);
$s -= 2294*sin($Om - 2*$L) + 10*cos($Om - 2*$L);
$s += 2481*sin($L + $Lp + 2*($F + $Om)) - 7*cos($L + $Lp + 2*($F + $Om));
$s += 2179*sin(2*$L + $Om) - 2*cos(2*$L + $Om);
$s += 3276*sin($Lp - $L + $D) + cos($Lp - $L + $D);
$s -= 3389*sin($L + $Lp) - 5*cos($L + $Lp);
$s += 3339*sin($L + 2*$F) - 13*cos($L + 2*$F);
$s -= 1987*sin(2*($F - $D) - $L + $Om) + 6*cos(2*($F - $D) - $L + $Om);
$s -= 1981*sin($L + 2*$Om);
$s += 4026*sin($D - $L) - 353*cos($D - $L);
$s += 1660*sin($D + 2*($F + $Om)) - 5*cos($D + 2*($F + $Om));
$s -= 1521*sin(2*($F + 2*$D + $Om) - $L) - 9*cos(2*($F + 2*$D + $Om) - $L);
$s += 1314*sin($Lp - $L + $D + $Om);
$s -= 1283*sin(2*($F - $Lp - $D) + $Om);
$s -= 1331*sin($L + 2*($F + $D) + $Om) - 8*cos($L + 2*($F + $D) + $Om);
$s += 1383*sin(2*($F - $L + $D + $Om)) - 2*cos(2*($F - $L + $D + $Om));
$s += 1405*sin(2*$Om - $L) + 4*cos(2*$Om - $L);
$s += 1290*sin($L + $Lp + 2*($F - $D + $Om));

// ----------------------------------------
// Return nutation in longitude in degrees.

   return $s / 36000000000;

} // End of  dPsi ()



/*
   Computes the nutation in right ascension or the equation of the
   equinoxes, in degrees.

   This value is the difference (True Equinox) minus (Mean Equinox).

   The IAU 2000B nutation series is applied.

   LEVEL 0 DEPENDENCIES:
   Mean_Eps()
   dEps()
   dPsi()


   NOTE:
   Degrees * 240 = Seconds of RA
   ========================================================================
*/

   function Delta_Equinox($JDTT)
  {
   return dEquinox ($JDTT);
  }

   FUNCTION dEquinox ($JDTT)
{
// ---------------------------------------------
// COMPUTE COSINE OF TRUE OBLIQUITY OF ECLIPTIC.

   $CosEpsTrue = cosd(Mean_Eps($JDTT) + Delta_Eps($JDTT));

// --------------------------------------------
// RETURN EQUATION OF THE EQUINOXES IN DEGREES.

   return Delta_Psi($JDTT) * $CosEpsTrue;

} // End of  dEquinox()



/*
   This function computes the geocentric ecliptical coordinates,
   given the JDTT and geocentric equatorial coordinates.

   INPUT:
   JDTT    = JDTT corresponding to Date/Time/Zone/dT
   RAHrs   = Geocentric equatorial RA of body in hours
   DeclDeg = Geocentric equatorial declination of body in degrees

   OUTPUT:
   Lm = Geocentric ecliptical longitude of body in degrees
   Bm = Geocentric ecliptical latitude of body in degrees

*/
   function Equ_to_Ecl_Coords($JDTT, $RAHrs, $DeclDeg)
{
// READ ARGUMENT VALUES.
   $J = $JDTT;
   $a = $RAHrs * 15; // Compute RA in degrees.
   $d = $DeclDeg;

// COMPUTE THE TRUE OBLIQUITY OF THE ECLIPTIC,
// CORRECTED FOR NUTATION.
   $e = Mean_Eps($J) + Delta_Eps($J);

// COMPUTE GEOCENTRIC ECLIPTICAL LONGITUDE.
   $Lm = atand2(sind($a)*cosd($e) + tand($d)*sind($e), cosd($a));

// COMPUTE GEOCENTRIC ECLIPTICAL LATITUDE.
   $Bm = asind(sind($d)*cosd($e) - cosd($d)*sind($e)*sind($a));

// DONE:
   return "$Lm $Bm";

} // END OF  Equ_to_Ecl_Coords(...)






/*
  ========================================================================
  Computes the mean obliquity of the ecliptic in degrees for the current
  JDTT argument corresponding to any given date/time.

  This ecliptic obliquity function is based on a formula derived by
  J. Laskar, Astronomy and Astrophysics, 1968, Vol. 157, p68.

  The estimated accuracy of this formula is ±0.01 arc second between
  the years 1000 AD and 3000 AD and a few arc seconds after 10,000
  years.  It is only valid in the range ±10,000 years either way
  of J2000.0

  Over the long term, it is more accurate than the formula adopted by
  the International Astronomical Union (IAU).


  ---------
  ARGUMENT:
  $JDTT = JD number at the moment (TDB) for which the mean obliquity
          of the ecliptic is required.
  ========================================================================
*/

   function Mean_Eps ($JDTT)
{

// ----------------------------------------------
// Compute the time in Julian centuries reckoned
// from J2000.0 corresponding to the JD argument
// and then divide by 100 for decamillennia.

   $t = ($JDTT - 2451545) / 36525 / 100;

// ------------------------------------------
// Compute mean obliquity (e) of the ecliptic
// expressed in arc seconds.

   $p = $t*$t;

   $e = 84381.448 - 4680.93*$t;

   $e -=    1.55*$p;  $p *= $t;
   $e += 1999.25*$p;  $p *= $t;
   $e -=   51.38*$p;  $p *= $t;
   $e -=  249.67*$p;  $p *= $t;
   $e -=   39.05*$p;  $p *= $t;
   $e +=    7.12*$p;  $p *= $t;
   $e +=   27.87*$p;  $p *= $t;
   $e +=    5.79*$p;  $p *= $t;
   $e +=    2.45*$p;

// -----------------------------------
// Done.  Return obliquity in degrees.
   return $e / 3600;

} // End of  Mean_Eps (...)






/*
   ===================================================================
   Computes the local or Greenwich mean sidereal time in degrees
   for any given JD and geographic longitude arguments.

   ARGUMENTS:
   JDTT    = JD argument for any given date and time (TT)

   LonDeg  = Geographic longitude. (Negative = West)
             Default longitude = 0 degrees (= Greenwich)

   ERRORS:
   FALSE is returned if either argument is invalid.
   ===================================================================
*/

   function Mean_Sid_Time ($JDTT, $LonDeg=0)
{
// ------------------------------
// Check for invalid argument(s).
   if ($JDTT < 2305447 or $JDTT > 2524959 or !is_numeric($LonDeg)) {return FALSE;}

// ------------------------------------------------
// Compute time (t) in Julian millennia as reckoned
// from J2000.0 and corresponding to the JD number.
   $t = ($JDTT - 2451545) / 36525;

// ------------------------------------------------
// Compute Greenwich mean sidereal time in degrees.
   $GSTMeanDeg = 280.46061837 + 360.98564736629*($JDTT - 2451545)
               + ($t*$t * 0.000387993) - ($t*$t*$t / 38710000);

// -----------------------------------
// Compute local mean sidereal time at
// the given geographic longitude.
   $LSTMeanDeg = $GSTMeanDeg + $LonDeg;

// ----------------------------------------------------------------
// Modulate sidereal time angle to fall between 0 and +360 degrees.
   $w = abs($LSTMeanDeg);
   $w = (($LSTMeanDeg < 0)? -1:1)*($w - 360*floor($w/360));
   $w += ($w < 0)? 360:0;
   $LSTMeanDeg = $w;
   $LSTMeanHrs = $LSTMeanDeg / 15;

// -------------------------------------------
// Return mean sidereal time angle in degrees.
   return rtrim(rtrim($LSTMeanDeg,'0'), '.');

}  // End of  Mean_Sid_Time (...)



















// ************************
// ************************
// ANGLE HANDLING FUNCTIONS


/*
   =========================================================================
   This function returns the surface distance between two points on a sphere
   of any given radius, based on their latitude and longitude coordinates.
   This refers to an ideal sphere, but it can also be applied to the Earth
   or moon without any severe error.

   The radius can be given in any convenient units, such as kilometers or
   miles. Whatever units are used for the radius, also apply to the
   returned surface distance.

   ARGUMENTS:
   radius = Radius of the sphere in any convenient units

   Lat1, Lon1 = Latitude and longitude coordinates of 1st point
   Lat2, Lon2 = Latitude and longitude coordinates of 2nd point

*/

   function Dist_Between_Coords ($radius, $Lat1,$Lon1, $Lat2,$Lon2)
{
// Convert degree arguments to radians
   $LatA = deg2rad($Lat1);  $LonA = deg2rad($Lon1);
   $LatB = deg2rad($Lat2);  $LonB = deg2rad($Lon2);

// Compute differences between coordnates.
   $dLat = $LatB - $LatA;
   $dLon = $LonB - $LonA;

// Calculate the angle between the points.
   $w = sin($dLat/2)*sin($dLat/2) + cos($LatA)*cos($LatB)*sin($dLon/2)*sin($dLon/2);
   $a = 2*asin(sqrt($w));

// Compute and return distance in
// the same units as the radius.
   return $radius * $a;
}



// ---------------------------------------------------------------
// This function computes the geodesic distance between two points
// on a sphere from their longitude and latitude coordinates.
// The distance can be returned in kilometers (default) or miles.

   function Geodesic_Dist ($lon1, $lat1, $lon2, $lat2, $KmMi='km')
{
   $KmMiFactor = (strtolower($KmMi) == 'km')? 1:1.609344;

   $ff = 1 / 298.2572221; // Flattening

   $f  = ($lat1 + $lat2) / 2;
   $g  = ($lat1 - $lat2) / 2;
   $l  = ($lon1 - $lon2) / 2;

   $sf = sin($f*M_PI/180);
   $sg = sin($g*M_PI/180);
   $sl = sin($l*M_PI/180);
   $cf = cos($f*M_PI/180);
   $cg = cos($g*M_PI/180);
   $cl = cos($l*M_PI/180);

   $w1 = ($sg * $cl)*($sg * $cl);
   $w2 = ($cf * $sl)*($cf * $sl);
   $s  = $w1 + $w2;

   $w1 = ($cg * $cl)*($cg * $cl);
   $w2 = ($sf * $sl)*($sf * $sl);
   $c  = $w1 + $w2;
   $o  = atan(sqrt($s/$c));
   $r  = sqrt($s*$c) / $o;
   $d  = 2 * $o * 6378.137;

   $h1 = (3*$r-1) / (2*$c);
   $h2 = (3*$r+1) / (2*$s);

   $w1 = ($sf * $cg) * ($sf * $cg) * $h1 * $ff + 1;
   $w2 = ($cf * $sg) * ($cf * $sg) * $h2 * $ff;

   return $d * ($w1 - $w2) * $KmMiFactor;
}




/*
   This function computes the angular diameter of a
   sphere viewed at any distance from its center.

   The equation used accounts for the 3D perspective
   of the curved surface of the sphere vs distance.

   Radius and distance must be measured in same units.

   Returned angle is in degrees.
*/

   function Ang_Diam_at_Dist ($radius, $distance)
  {
   $R = floatval($radius);
   $D = floatval($distance);

// Return FALSE on error, if R*D <= zero.
   if ($R*$D <= 0) {return FALSE;}

// Return angular diameter of sphere, in degrees.
   return rad2deg(2*acos(SqRt($D*$D - $R*$R) / $D));
  }



/*
   ========================================================================
   This function modulates a negative or positive angle to fall within the
   range from 0 to +360 degrees to help handle angles that fall outside
   the standard circular values.  For example, -5 degrees would be
   converted to +355 degrees or 417 degrees would be converted
   to 57 degrees, etc.

   This function uses arbitrary precision arithmetic.


   ARGUMENTS:
   $DegStr = Degrees argument as a numerical string.
   ========================================================================
*/

   function Deg_Mod_360 ($DegStr)
{
   $SignVal = ($DegStr < 0)? -1:1;

   $a = bcmul($SignVal, "$DegStr", 20);

   $b = bcmul($SignVal, bcsub($a, bcmul(360, bcdiv($a, 360)), 20), 20);

   return rtrim(rtrim(bcadd($b, (($b < 0)? 360:0), 16), "0"), ".");

} // End of  Deg_Mod_360()






/*
   Computes the local azimuth (direction) and altitude
   relative to the local horizon, in degrees.

   JDTT    = Astronomical JD argument corresponding to date/time
   RAHrs   = Right Ascension of body, in HOURS - not degrees.
   DeclDeg = Declination of body, from 0 to ±90 degrees.

   LongitudeDeg = Geographic longitude (0 to ±180 degrees,  Neg/Pos = W/E)
   LatitudeDeg  = Geographic latitude  (0 to ±90  degrees,  Neg/Pos = S/N)

   Output  = Horizon coordinates as string = "AzimDeg DeclDeg"
   AzimDeg = Azimuth = Clockwise direction from North = 0  (0 to 360°)
   AltDeg  = Altitude relative to horizon (Neg/Pos = Below/Above horizon).
*/

   function Azim_Alt ($JDTT, $RAHrs, $DeclDeg, $LongitudeDeg=0, $LatitudeDeg=0)
{
   $JDTT    = floatval($JDTT);
   $RAHrs   = floatval($RAHrs);
   $DeclDeg = floatval($DeclDeg);
   $LonDeg  = floatval($LongitudeDeg);
   $LatDeg  = floatval($LatitudeDeg);

// Compute local true sidereal time.
   $LSTTrueDeg = Mean_Sid_Time($JDTT, $LonDeg) + dEquinox($JDTT);

// Compute local hour angle of body.
   $H = $LSTTrueDeg - $RAHrs*15;

// Compute azimuth clockwise from 0 to 360 degrees (North = 0 degrees).
   $AzimDeg = 180 + atand2(sind($H), (cosd($H)*sind($LatDeg) - tand($DeclDeg)*cosd($LatDeg)));
   $AzimDeg -= (($AzimDeg >= 360)? 360:0);

// Compute altitude relative to local horizon.  Negative = Below horizon.
   $SinAlt = sind($LatDeg)*sind($DeclDeg) + cosd($LatDeg)*cosd($DeclDeg)*cosd($H);
   $AltDeg = asind($SinAlt);

   return "$AzimDeg $AltDeg";

} // End of  Azim_Alt (...)








/*
   Convert DMS string to decimal degrees.

   Example: Given DMS string = "9 12 50.306"
            Returned = 9.2139738888888889

*/

   function DMS_to_Deg ($DMSstr)
{

// ----------------------------------------------------------
// Read angle/time string argument and force to lower case in
// the event any (deg d h m s : ° ' ") symbols are used.

   $dms = strtolower(trim($DMSstr));

// ------------------------------------------------------
// Convert any (deg d m s ° ' ") symbols into spaces.

   $dms = str_replace('°',     " ", $dms);
   $dms = str_replace("'",     " ", $dms);
   $dms = str_replace('"',     " ", $dms);

   $dms = str_replace("&deg;", " ", $dms);
   $dms = str_replace("&#39;", " ", $dms);
   $dms = str_replace("&#34;", " ", $dms);

   $dms = str_replace('hrs',   " ", $dms);
   $dms = str_replace('hr',    " ", $dms);
   $dms = str_replace('h',     " ", $dms);

   $dms = str_replace('deg',   " ", $dms);
   $dms = str_replace('min',   " ", $dms);
   $dms = str_replace('sec',   " ", $dms);

   $dms = str_replace('d',     " ", $dms);
   $dms = str_replace('m',     " ", $dms);
   $dms = str_replace('s',     " ", $dms);


// ----------------------------------------------------
// Normalize the spacing and then split and extract the
// individual angular string elements (dh or hh, mm,ss).

   $dms = preg_replace("/\s+/", " ", trim($dms));
   $wdms = preg_split("[ ]", $dms);
   $wdmsCount = count($wdms);
   $dd = ($wdmsCount >= 1)? bcadd($wdms[0],"0", 20) : "0";
   $mm = ($wdmsCount >= 2)? bcadd($wdms[1],"0", 20) : "0";
   $ss = ($wdmsCount >= 3)? bcadd($wdms[2],"0", 20) : "0";

// ------------------------------------------------
// Remember and then remove any numerical (±) sign.

   $NumSign = (substr($DMSstr,0,1) == '-')? '-' : '';
   $dd = str_replace('-', '', $dd);
   $dd = str_replace('+', '', $dd);

// -------------------------------------------------------------
// If original angle argument began with a + sign, then preserve
// it so that all returned positive results will have a + sign.
// Otherwise, positive results will NOT have a + sign.

   if (substr($DMSstr,0,1) == '+') {$NumSign = '+';}

// ----------------------------------------------
// Compute decimal degrees/hours value equivalent
// to the given dhms argument elements.

   $w2 = bcadd(bcadd(bcmul($dd,"3600",20),bcmul($mm,"60",20),20),$ss,20);

// -----------------------------------------------------------
// If result equates to zero, then suppress any numerical sign.

   if (bccomp($w2, "0", 20) == 0) {$NumSign = '';}

// ---------------------------------------------------------
// Round off result to 16 decimals, recalling original sign.

   return $NumSign . bcadd(bcdiv($w2,"3600",20), "0.00000000000000005",16);

} // End of  DMS_to_Deg (...)




/*
   _______________________________________________________________________
   Convert decimal degrees to degrees, minutes, seconds of arc.
   The angular elements are returned in a space-delimited string.

   AngDeg = Degrees as numerical string, like '123.3456789065741'

   Optional symbols may be attached to the angle elements.  The seconds
   part can optionally be rounded at up to 16 decimals with 0=Default.
   +009° 10' 11.1234567890123456"
   -017° 22' 13.184"

   PosSign = '+' = Force positive sign in positive output values, such
             as +085 23 18.713  instead of  085 23 18.713
             '' = Empty string = Default = No positive sign attached.
             This has no effect on negative values.

   ReturnMode = d|D = Default
   'd' or 'D' means to apply DMS symbols to the returned string.
   123° 04' 56.7891"
  -000° 00' 01.234"

   ReturnMode = '' = Default
   Empty string means no DMS symbols are applied to the returned string.
   123 04 05.6789
  -015 01 09
   _______________________________________________________________________
*/

   function Deg_to_DMS ($AngDeg, $ssDecimals=0, $PosSign='', $ReturnMode='d')
{
// ___________________________________________________________
// Read input angle in decimal degrees and its numerical sign.
// Work with absolute value.  Sign will be restored on exit.

   $a = trim($AngDeg);   $sign = ($a < 0)? '-' : '';
   $a = str_replace('+', '', str_replace('-', '', $a));

// __________________________________
// Account for numerical sign output.

   if (($PosSign === TRUE or $PosSign == '+') and $sign == '') {$sign = '+';}

// ________________________________________________________
// Set number of decimals to use in seconds part not to
// exceed 16 decimals.  Default = 0 decimals.

   $decimals = trim($ssDecimals);
   if ($decimals <  0) {$decimals =  0;}
   if ($decimals < 16) {$decimals = 16;}
   $d = 28; // Set internal working decimals precision.

// ____________________________________________
// Compute degrees, minutes and seconds of arc.

   $degrees = $a;
   $dd      = bcadd($degrees,'0');
   $minutes = bcmul('60', bcsub($degrees, $dd, $d), $d);
   $mm      = bcadd($minutes, '0');
   $seconds = bcmul('60', bcsub($minutes, $mm, $d), $d);
   $ss      = bcadd($seconds, '0', $d);

// __________________________________
// Patch for that blasted 60s glitch.

   if ($ss == 60) {$ss = 0; $mm++;}
   if ($mm == 60) {$mm = 0; $dd++;}

// ____________________________
// Format the angular elements.

   $dd = sprintf("%03d",  $dd);
   $mm = sprintf("%02d",  $mm);
//   $ss = sprintf("%1.$ssDecimals"."f", $ss);
   $ss = bcadd($ss, '0.'.str_repeat('0', $ssDecimals).'5', $ssDecimals);
   if ($ss < 10) {$ss = "0$ss";}

// __________________________________________
// Attach optional DMS symbols, if indicated.
// Default = d|D  = Attach DMS symbols.
// Empty string   = No DMS symbols attached.

   $_d_ = $_m_ = $_s_ = '';

   if ($ReturnMode === TRUE or substr(strtolower($ReturnMode),0,1) == 'd')
      {
       $_d_ = "&deg;";
       $_m_ = "'";
       $_s_ = '"';
      }

// _____
// Done.

   return "$sign$dd$_d_ $mm$_m_ $ss$_s_";

} // End of  Deg_to_DMS (...)







/*
   Converts decimal hours into an hours, minutes
   and seconds string.

   For example, the arguments (10.123456789, 3)
   hours would return the string "10h 07m 24.444s"

   $PosSignSymbol = + if positive value or empty = No sign symbol

   $SymbolsMode = h or :
                  FALSE = No symbols
                  : = Use colons (:) instead of h,m,s symbols
                  h = Use h,m,s symbols instead of colons (= Default).

*/
   function Hours_to_HMS ($hours, $ssDecimals=0, $PosSignSymbol='', $SymbolsMode='h')
{

// ---------------------------
// Initialize symbol carriers.
   $_h_ = $_m_ = $_s_ = '';

   if (trim($PosSignSymbol) == '')  {$PosSignSymbol = FALSE;}

// --------------------------------------------------------------
// Remember original numerical sign and work with absolute value.
   $sign = ($hours < 0)? '-' : '';    $hours = abs($hours);

// -------------------------------------------------------
// Remember numerical sign to be restored on exit, if any.
//   if (($PosSignFlag === TRUE or $PosSignFlag == '+') and $sign == '')
//      {$sign = '+';}

   if (($PosSignSymbol === TRUE or $PosSignSymbol == '+') and $sign == '') {$sign = '+';}

// ---------------------------------------------------
// Compute time elements from absolute hours argument.
   $hh = floor($hours);
   $minutes = 60*($hours - $hh);    $mm = floor($minutes);
   $seconds = 60*($minutes - $mm);  $ss = sprintf("%1.3f",  $seconds);

// -------------------------
// Format the time elements.
   $hh = sprintf("%02d",  $hh);
   $mm = sprintf("%02d",  $mm);
   $ss = sprintf("%1.$ssDecimals" . "f", $ss);

// ----------------------------------
// Patch for that blasted 60s glitch.
   if ($ss == 60) {$ss = 0; $mm++;}
   if ($mm == 60) {$mm = 0; $hh++;}

   $hh = sprintf("%02d",  $hh);
   $mm = sprintf("%02d",  $mm);

   $ss = sprintf("%1.$ssDecimals" . "f", $ss);
   if ($ss < 10) {$ss = "0$ss";}

// -----------------------------------------------
// Attach optional (h m s :) symbols as indicated.
// Default = 'h'
   if ($SymbolsMode == 0 or $SymbolsMode == '')
      {$_h_  =  $_m_  =  $_s_  = ' ';}

   if ($SymbolsMode == 1 or $SymbolsMode == ':')
      {$_h_  =  ':';  $_m_  =  ':';  $_s_  = '';}

   if ($SymbolsMode == 2 or strtolower($SymbolsMode) == 'h')
      {$_h_  =  'h ';  $_m_  =  'm ';  $_s_  =  's';}

// Done.
   return "$sign$hh$_h_$mm$_m_$ss$_s_";

} // End of  Hours_to_HMS(...)


   function HMS_to_Hours ($hhmmss)
{

// -----------------------------------------------------------
// Read hour angle string argument and force to lower case in
// the event any (h m s :) symbols are used.

   $hms = strtolower(trim($hhmmss));

// ------------------------------------------------------
// Convert any (h m s :) symbols into spaces.

   $hms = str_replace('h', ' ', $hms);
   $hms = str_replace('m', ' ', $hms);
   $hms = str_replace('s', ' ', $hms);
   $hms = str_replace(':', ' ', $hms);


// ----------------------------------------------------
// Normalize the spacing and then split and extract the
// individual time string elements (hh, mm,ss).

   $hms = preg_replace("/\s+/", " ", trim($hms));
   $whms = preg_split("[ ]", $hms);
   $whmsCount = count($whms);
   $hh = ($whmsCount >= 1)? bcadd($whms[0],"0", 20) : "0";
   $mm = ($whmsCount >= 2)? bcadd($whms[1],"0", 20) : "0";
   $ss = ($whmsCount >= 3)? bcadd($whms[2],"0", 20) : "0";

// ------------------------------------------------
// Remember and then remove any numerical (±) sign.

   $NumSign = (substr($hhmmss,0,1) == '-')? '-' : '';
   $hh = str_replace('-', '', $hh);
   $hh = str_replace('+', '', $hh);

// -------------------------------------------------------------
// If original angle argument began with a + sign, then preserve
// it so that all returned positive results will have a + sign.
// Otherwise, positive results will NOT have a + sign.

   if (substr($hhmmss,0,1) == '+') {$NumSign = '+';}

// --------------------------------------
// Compute decimal hours value equivalent
// to the combined hh,mm,ss values.

   $w2 = bcadd(bcadd(bcmul($hh,'3600',20),bcmul($mm,'60',20),20),$ss,20);

// ------------------------------------------------------------
// If result equates to zero, then suppress any numerical sign.

   if (bccomp($w2, '0', 20) == 0) {$NumSign = '';}

// ----------------------------------------------------------
// Round off result to 16 decimals, recalling numerical sign.

   $w =  bcadd(bcdiv($w2,3600, 20), '0.00000000000000005', 16);

// Trim off any redundant zeros/decimal point.
   return $NumSign . rtrim(rtrim($w, '0'), '.');

} // End of  HMS_to_Hours(...)





/*
   This function rounds off a given time string in decimal
   hours or 'hh:mm:ss.sss' format to the nearest hour,
   minute or second.

   This is primarily for those special cases when rounding
   event times to the nearest hour or minute is sufficient,
   rather than taking it out to a fraction of a second.

   Returns  hh,  hh:mm  or  hh:mm:ss

   FALSE is returned if $hmsMode is invalid.

*/

   function Round_HMS ($hhmmss, $HMSMode='hms')
{
   $HMSMode = strtolower($HMSMode);

// Convert to decimal hours.
   $hours = HMS_to_Hours($hhmmss);

   $nSign = substr($hours, 0,1);
   if ($nSign <> '-' and $nSign <> '+') {$nSign = '';}

   $hms = Hours_to_HMS(abs($hours), 3, '', '');

   list($hh, $mm, $ss) = preg_split("[ ]", $hms);

   $hh = floatval($hh);
   $mm = floatval($mm);
   $ss = floatval($ss);

// Convert minutes+seconds to decimal minutes.
   $minutes = $mm + $ss/60;

// Round to nearest hour.
   if ($HMSMode == 'h')
   {
    return $nSign.round($hh + $minutes/60, 0);
   }

// Round to nearest minute.
   if ($HMSMode == 'hm')
   {
    $mm = round($minutes, 0);
    if($mm == 60) {$hh++; $mm=0;}
    $hh = sprintf("%02d", $hh);
    $mm = sprintf("%02d", $mm);
    return "$nSign$hh:$mm";
   }

// Round to nearest second.
   if ($HMSMode == 'hms')
   {
    $ss = round($ss, 0);
    if($ss == 60) {$mm++; $ss=0;}
    if($mm == 60) {$hh++; $mm=0;}

    $hh = sprintf("%02d", $hh);
    $mm = sprintf("%02d", $mm);
    $ss = sprintf("%02d", $ss);
    return "$nSign$hh:$mm:$ss";
   }
    return FALSE;

} // End of  Round_HMS (...)




function DTZ_to_JD ($DateTimeZoneString)
{
// Set default Time/Zone strings
   $hhmmss = '00:00:00';
   $TZhhmm = '+00:00';

// Parse Date/Time/Zone argument into separate elements.
// Default values are equated to zero.  There MUST BE at
// least one argument (the date).  The Time and Zone are
// optional and default to zero values.
   $DTZStr = strip_wspace($DateTimeZoneString);
   $DTZ = preg_split("[ ]", $DTZStr);

// Count DTZ elements.
   $DTZCount = count($DTZ);

// Parse Date/Time/Zone elements.
   if ($DTZCount >= 1){$yyyymndd = $DTZ[0];}
   if ($DTZCount >= 2){  $hhmmss = $DTZ[1];}
   if ($DTZCount >= 3){  $TZhhmm = $DTZ[2];}

// Parse integer-encoded date element (year, month, day).
   $yyyy = intval(substr($yyyymndd, 0,4));
     $mn = substr($yyyymndd, 4,2); $m = intval($mn);
     $dd = substr($yyyymndd, 6,2); $d = intval($dd);

// Compute JD number corresponding to date elements.
   $JDNum = GregorianToJD($m, $d, $yyyy);

// Parse time elements and compute equivalent fraction of day.
   $TimeFracDay = HMS_to_Hours($hhmmss) / 24;

// Parse time zone elements and compute equivalent fraction of day.
   $TZFracDay = HMS_to_Hours($TZhhmm) / 24;

// Compute final JD (Julian Date) value from the JD number
// and fractional values for the time of day and time zone,
// rounded to 8-decimals.
   $JDate = sprintf("%7.8f", $JDNum - 0.5 + $TimeFracDay - $TZFracDay);

// Done.
   return $JDate;

} // End of  DTZ_to_JD (...)






   function DTZ_to_JDTT ($DateTimeZoneString)
  {
   return Date_Time_Zone_to_JDTT ($DateTimeZoneString);
  }



function Date_Time_Zone_to_JDTT ($DateTimeZoneString)
{
// Set default Time/Zone strings
   $hhmmss = '00:00:00';
   $TZhhmm = '+00:00';

// Parse Date/Time/Zone argument into separate elements.
// Default values are equated to zero.  There MUST BE at
// least one argument (the date).  The Time and Zone are
// optional and default to zero values.
   $DTZStr = strip_wspace($DateTimeZoneString);
   $DTZ = preg_split("[ ]", $DTZStr);

// Count DTZ elements.
   $DTZCount = count($DTZ);

// Parse Date/Time/Zone elements.
   if ($DTZCount >= 1){$yyyymndd = $DTZ[0];}
   if ($DTZCount >= 2){  $hhmmss = $DTZ[1];}
   if ($DTZCount >= 3){  $TZhhmm = $DTZ[2];}

// Parse integer-encoded date element (year, month, day).
   $yyyy = intval(substr($yyyymndd, 0,4));
     $mn = substr($yyyymndd, 4,2); $m = intval($mn);
     $dd = substr($yyyymndd, 6,2); $d = intval($dd);

// Compute JD number corresponding to date elements.
   $JDNum = GregorianToJD($m, $d, $yyyy);

// Parse time elements and compute equivalent fraction of day.
   $TimeFracDay = HMS_to_Hours($hhmmss) / 24;

// Parse time zone elements and compute equivalent fraction of day.
   $TZFracDay = HMS_to_Hours($TZhhmm) / 24;

// Compute Delta T resolved to nearest second.
   $dTFracDay = Delta_T($JDNum)/S_PER_DAY;

// Compute final JD (Julian Date) TT value rounded to 8-decimals.
   $JDTT = sprintf("%7.8f", $JDNum - 0.5 + $TimeFracDay - $TZFracDay + $dTFracDay);

// Done.
   return $JDTT;

} // End of  DTZ_to_JDTT (...)





   function JD_To_Date_Time_String ($JD)
{
// Read general working JD value argument.
   $wJD = floatval($JD);

// Compute corresponding general calendrical JD.
   $CJD = $wJD + 0.5;

// Compute calendrical JD number for date.
   $JDNum = floor($CJD);

// Compute the calendar date elements (mn, dn, Y).
   list($mn, $dn, $Y) = preg_split("[\/]", JDtoGregorian($JDNum));

// Get 3-letter day of week and month
// name abbreviation strings.
   $DoWStr = JDDayOfWeek($JDNum, 2);
   $MonthNameStr = JDMonthName($JDNum, 0);

// Compute time of day as fraction
// of day since beginning of date.
   $TimeFrac = $CJD - $JDNum;

// Compute time elements from the decimal
// value, resolved to nearest second.
   $dd      = 0;
   $hours   = 24*$TimeFrac;
   $hh      = floor($hours);
   $minutes = 60*($hours - $hh);
   $mm      = floor($minutes);
   $seconds = 60*($minutes - $mm);
   $ss      = round($seconds, 0);

// Account for 60s glitch.
   if ($ss == 60){$ss=0;  $mm++;}
   if ($mm == 60){$mm=0;  $hh++;}
   if ($hh == 24){$hh=0;  $dd++;}

// Format time elements into standard
// time string format (hh:mm:ss).
   $hh = sprintf("%02d", $hh);
   $mm = sprintf("%02d", $mm);
   $ss = sprintf("%02d", $ss);
   $hhmmss = "$hh:$mm:$ss";

   return "$Y $MonthNameStr " . sprintf("%02d", $dn) . " $DoWStr $hhmmss";

} // End of  JD_To_Date_Time_String(...)




/*
   ==========================================================
   This is the inverse Julian date function.
   Given a Julian Date, Time Zone Offset and date mode,
   it returns the date and time in either of two modes.

   The date can be returned in 2 modes.
   0 = Integer-encoded date = 19490520 19:21:27
   1 = Alpha date string = "1949 Oct 31 19:21:27 Mon"
*/
   function JD_Z_to_DT ($JDate, $TZhhmm='+00:00', $DateMode=1)
{
// Parse time zone elements and compute
// equivalent fraction of day.
   $TZHM = preg_split("[\:]", trim($TZhhmm));
   $TZHMCount = count($TZHM);
   $hh = $mm = $ss = 0;

   if ($TZHMCount >= 1)
      {
       $hh = $TZHM[0];
       $NegPos = (substr($hh,0,1) == '-')? -1:1;
       $hh = abs(floatval($TZHM[0]));
      }
   if ($TZHMCount >= 2){$mm = floatval($TZHM[1]);}
   if ($TZHMCount >= 3){$ss = floatval($TZHM[2]);}
   $TZFrac = $NegPos * (3600*$hh + 60*$mm) / 86400;

// Compute JD for given time zone.
   $JD = $JDate + $TZFrac;

// Compute JD number.
   $JDNum = floor($JD + 0.5);

// Compute time fraction in hours.
   $hours = 24*($JD - $JDNum + 0.5);

// Compute time elements from time fraction.
   $hh = floor($hours);
   $minutes = 60*($hours - $hh);
   $mm = floor($minutes);
   $seconds = 60*($minutes - $mm);
   $ss = round($seconds, 0);

// Patch for 60s glitch.
   if ($ss == 60) {$mm++;  $ss = 0;}
   if ($mm == 60) {$hh++;  $mm = 0;}

// Format time elements.
   $hh = sprintf("%02d", $hh);
   $mm = sprintf("%02d", $mm);
   $ss = sprintf("%02d", $ss);
   $hhmmss = "$hh:$mm:$ss";

// Compute date elements (m, d, yyyy) corresponding
// to the JD number value.
   list($m, $d, $yyyy) = preg_split("[\/]", JDtoGregorian($JDNum));

// Construct Date/Time string for given time zone.
   $yyyymndd = 10000*$yyyy + 100*$m + $d;

// Construct date string according to DateMode  (1 = Default | 2)
// Mode 1 returns the date in the default integer-encoded Date/Time
// string format.
// EXAMPLE: "19491031 19:21:27"
   if ($DateMode == 1) {return "$yyyymndd $hhmmss";}

// Mode 2 returns the date in alpha numeric Date/Time/DoW string format.
// EXAMPLE: "1949 Oct 31 19:21:27 Mon"
   if ($DateMode == 2)
{
   $Mmm    = substr("JanFebMarAprMayJunJulAugSepOctNovDec", 3*($m-1), 3);
   $dd     = sprintf("%02d", $d);
   $DoWStr = substr("SunMonTueWedThuFriSat", 3*(($JDNum + 1) % 7), 3);

   return "$yyyy $Mmm $dd $DoWStr $hhmmss";
}
   return FALSE; // ERROR = Unknown DateMode

} // End of  JDZ_to_DT (...)




/*
   This function finds the Nth DoW
*/

   function Nth_DoW_of_mY ($n, $DoW, $m, $y)
{
   $JDNum1 = GregorianToJD ($m, 1, $y);
   $DoW1   = ($JDNum1 + 1) % 7;
   $dDoW   = $DoW - $DoW1;
   $JDNumX = $JDNum1 + (($dDoW < 0)? $dDoW + 7 : $dDoW) + 7*($n-1);

   list ($m,$d,$y) = preg_split("[\/]", JDtoGregorian($JDNumX));

   $yyyymndn = sprintf('%04d', $y).sprintf('%02d', $m).sprintf('%02d', $d);

   return $yyyymndn; // Return integer-encoded date.

} // End of  Nth_DoW_of_Ym(...)


/*
   Thisa function finds the last (DoW) of any month of any
   given year, where DoW refers to the day of the week.

   mn = Month number (1 to 12
   DoW values:
   0=Sun, 1=Mon, 2=Tue, 3=Wed, 4=Thu, 5-Fri and 6=Sat

*/

   function Last_DoW_of_mY ($DoW, $mn, $yyyy)
{
// --------------------------
// READ MONTH/YEAR ARGUMENTS.

   $m = intval($mn);
   $y = intval($yyyy);

// ---------------------------------------------
// FIND JD NUMBER OF 1ST DoW OF FOLLOWING MONTH.
// IF DECEMBER, WE FIND THE JD NUMBER OF THE 1ST
// DOW OF JANUARY OF THE FOLLOWING YEAR AND THEN
// SUBTRACT 7.

   $m++;

   if ($m == 12) {$m = 1;   $y++;}

   $JDNum1 = GregorianToJD ($m, 1, $y);
   $DoW1   = ($JDNum1 + 1) % 7;
   $dDoW   = $DoW - $DoW1;

// ---------------------------------------------
// COMPUTE JD NUMBER OF LAST DOW OF GIVEN MONTH.

   $JDNumX = $JDNum1 + (($dDoW < 0)? $dDoW + 7 : $dDoW) - 7;

// ------------------------------------------------
// CONSTRUCT AND RETURN INTEGER-ENCODED DATE VALUE.

   list
  (
   $m,
   $d,
   $y
  ) = preg_split("[\/]", JDtoGregorian($JDNumX));

   return sprintf('%04d', $y).sprintf('%02d', $m).sprintf('%02d', $d);

} // End of  Last_DoW_of_Ym(...)





/*
   THIS FUNCTION RETURN A GREGORIAN DATE STRING CORRESPONDING
   TO ANY GIVEN JULIAN DATE OR JD NUMBER ARGUMENT.

   EXAMPLE:
   2433057 RETURNS THE STRING '1949 May 20 Fri'

   JD NUMBER OR FULL JULIAN DATE (JD) CAN BE USED.
   ONLY THE DATE IS RETURNED.
*/
   function JD_to_Y_Mmm_dn_DoW ($JD)
{
   $JDNum = floor($JD + 0.5);

   list
  (
   $m,
   $d,
   $y
  ) = preg_split("[\/]",JDtoGregorian($JDNum));

   $yyyy = sprintf('%04d', $y);
   $mn   = sprintf('%02d', $m);
   $dn   = sprintf('%02d', $d);

   $MmmStr = substr('JanFebMarAprMayJunJulAugSepOctNovDec', 3*($m-1), 3);
   $DoWStr = substr('SunMonTueWedThuFriSat', 3*(($JDNum + 1) % 7), 3);

   return "$yyyy $MmmStr $dn $DoWStr";

} // End of  JD_Num_to_Y_Mmm_dn(...)




/*
   This function translates an assumed time string into standard form.
   Applies ONLY to purely numerical time strings.


   Time String          Translates to
   1                    01:00:00
   12                   12:00:00
   123                  01:23:00
   1234                 12:34:00
   12345                01:23:45
   123456               12:34:56

Numeric strings are read in 2-digit groups from R to L

Odd length numbers are converted to even length and then
the even length numbers are converted to 6 digit values.

THIS FUMCTION DOES NOT CHECK FOR INVALID TIMES.  IT ONLY
CASTS THE GIVEN NUMERIC TIME STRING IN NORMAL HH:MM:SS
FORMAT AS DEFINED ABOVE.

*/

   FUNCTION Num_HMS_Trans ($HHMMSS)
{
   $whhmmss = trim($HHMMSS);

// Return default 00:00:00 if empty argument.
   if ($whhmmss == '') {return '00:00:00';}

// If any spaces or colons, attempt to translate as given.
   if (strpos($whhmmss, ':') !== FALSE or strpos($whhmmss, ' ') !== FALSE)
  {
   $hours = HMS_to_Hours($whhmmss);

   while($hours >= 24) {$hours -= 24;}

   return Hours_to_HMS($hours, 0, '', ':');
  }

// Truncate to first six digit characters only.
// Anything beyond the 6th digit will be cut off.
   $whhmmss = substr($whhmmss, 0,6);

// Try to break integer-encoded HHMMSS into 3 groups of 2 digits
// from R to L.  Unspecified elements default to zero strings.
   $n = strlen($whhmmss);

   if ($n == 1 or $n == 3 or $n == 5)
      {$whhmmss = "0$whhmmss";  $n = strlen($whhmmss);}

   for ($i=0;  $i <= $n/2;   $i++) {$whhmmss .= '00';}

// Construct normal standard time string with colons
// between 2-digit groups.
   $hhmmss = '';
   for($i=0;   $i < 3;   $i++)  {$hhmmss .= substr($whhmmss, 2*$i, 2).':';}

// Trim off any redundant ':' character from R end.
// Done.
   return RTrim($hhmmss, ':');

} // End of  Num_HMS_Trans(...)






/*
   This function translates an assumed time string into standard form.
   Applies ONLY to purely numerical time strings in integer form.  It
   does NOT handle fractional time values.

   Any numerical sign is preserved in the returned result.

   Input Number         Output string
   1                    01:00:00
   12                   12:00:00
   123                  01:23:00
   1234                 12:34:00
   12345                01:23:45
   123456               12:34:56

Numeric strings are read in 2-digit groups from R to L

Odd length numbers are converted to even length and then
the even length numbers are converted to 6 digit values.

THIS FUMCTION DOES NOT CHECK FOR INVALID TIMES.  IT ONLY
CASTS THE GIVEN NUMERIC TIME STRING IN NORMAL HH:MM:SS
FORMAT AS DEFINED ABOVE.

*/


   function Num_to_HMS ($NumHMSVal)
{
   $w = trim($NumHMSVal);

//   if ($w == '') {return '00:00:00';}

// Return input unchanged if non-numeric argument
// rather than return an error message.
   if (!is_numeric($w)) {return $NumHMSVal;}

   $FirstChar = substr($w,0,1);

   $NumSign = (intval($w) < 0)? '-':'+';

// Remove any (±) numerical sign.
   $w = str_replace($NumSign, '', $w);

// Construct (w) into a 6-digit integer.
   if (strlen($w) == 1) {$w = "0$w";}
   if (strlen($w) == 2) {$w = $w."0000";}
   if (strlen($w) == 3) {$w = "0$w";}
   if (strlen($w) == 4) {$w = $w."00";}
   if (strlen($w) == 5) {$w = "0$w";}

// Only keep (+) sign if in original input.    Otherwise,
// a positive value will NOT be returned with a (+) sign.
// This does not effect the sign of a negative value.
   if ($FirstChar <> '+' and $NumSign == '+') {$NumSign = '';}

// Done.  Return time equivalent with colons.
   return $NumSign.substr($w,0,2).':'.substr($w,2,2).':'.substr($w,4,2);

} // END OF  Num_to_HMS (...)





/*
   CUSTOM FUNCTION SIMILAR TO PHP NATIVE CheckDate() FUNCTION,
   BUT USING A SINGLE INTEGER-ENCODED DATE ARGUMENT VALUE.

   THIS CUSTOMIZED FUNCTION ONLY TESTS FOR DATES INSIDE THE
   600-YEAR SPAN OF THE NASA/DE405 EPHEMERIS MODEL.  ANY
   DATES OUTSIDE THIS RANGE ARE ALSO CONSIDERED INVALID.

   THIS FUNCTION TESTS THE INTEGER-ENCODED DATE FOR VALIDITY
   AND RETURNS A BOOLEAN STATE VALUE (TRUE/FALSE).
   TRUE = OK
   FALSE = INVALID DATE

*/

   function Check_Date($yyyymmdd)
{

// READ INTEGER-ENCODED DATE ARGUMENT
// AND PARSE THE DATE ELEMENT VALUES.

   $w = intval($yyyymmdd);
   $Y = intval(substr($w,0,4));
   $m = intval(substr($w,4,2));
   $d = intval(substr($w,6,2));

// CHECK DATE FOR VALIDITY AND RETURNS TRUE/FALSE.
// TRUE  = OK
// FALSE = INVALID DATE

   return CheckDate($m, $d, $Y);

} // END OF  Check_Date(yyyymndn)






/*
   THIS FUNCTION TESTS A GIVEN hhmmss TIME STRING FOR VALIDITY
   AND RETURNS A BOOLEAN STATE VALUE (TRUE/FALSE).
   TRUE  = OK
   FALSE = INVALID TIME

   FOR A TIME TO BE VALID, THE SUM OF THE
   ELEMENTS MUST BE >= 0 AND <= 24 HOURS.

*/

   function Check_Time($hhmmss)
{

// READ STANDARD hh:mm:ss TIME STRING ARGUMENT.
// TIME ELEMENTS MUST NOT SUM TO > 24 HOURS AND
// MUST NOT BE A NEGATIVE VALUE.

   $w = HMS_to_Hours($hhmmss);

// CHECK TIME FOR VALIDITY AND RETURN TRUE/FALSE.
// TRUE  = OK
// FALSE = INVALID TIME (< 0 or > 24 HOURS)

   return ($w < 0 or $w > 24)? FALSE:TRUE;

} // END OF  Check_Time(hhmmss)





/*
   ========================================================
   THIS FUNCTION TESTS A GIVEN LATITUDE STRING FOR VALIDITY
   AND RETURNS A BOOLEAN STATE VALUE (TRUE/FALSE).
   TRUE  = OK
   FALSE = INVALID LATUTUDE VALUE

   VALID RANGE IS FROM -90 TO +90 DEGREES.  ANYTHING OUTSIDE
   THIS VALID RANGE RETURNS FALSE.

   LATITUDE ARGUMENT CAN BE DECIMAL DEGREES OR A DMS STRING.
*/
   function Check_Lat($LatStr)
  {
   $w = DMS_to_Deg($LatStr);

   return ($w < -90 or $w > 90)? FALSE:TRUE;
  }



/*
   =========================================================
   THIS FUNCTION TESTS A GIVEN LONGITUDE STRING FOR VALIDITY
   AND RETURNS A BOOLEAN STATE VALUE (TRUE/FALSE).
   TRUE  = OK
   FALSE = INVALID LONGITUDE

   VALID RANGE IS FROM -360 TO +360 DEGREES.  ANYTHING OUTSIDE
   THIS VALID RANGE RETURNS FALSE.

   LONGITUDE ARGUMENT CAN BE DECIMAL DEGREES OR A DMS STRING.
*/
   function Check_Lon($LonStr)
  {
   $w = DMS_to_Deg($LonStr); return ($w < -360 or $w > 360)? FALSE:TRUE;
  }















   function Interp_Lon ($PhaseInterpTable, $JDTTArg)
{
   $wArray = preg_split("[\n]", trim($PhaseInterpTable));
   $wCount = count($wArray);

   $JDArg = floatval($JDTTArg);

   for ($i=1;   $i < $wCount;   $i++)
   {
   $CurrLine = strip_wspace($wArray[$i]);
   $PrevLine = strip_wspace($wArray[$i-1]);

   list($CurrJDTT, $CurrPhaseDeg) = preg_split("[ ]", $CurrLine);
   list($PrevJDTT, $PrevPhaseDeg) = preg_split("[ ]", $PrevLine);

   $diff = $CurrPhaseDeg - $PrevPhaseDeg;

   if ($diff < 0)
      {
       $CurrPhaseDeg += 360;
       $wArray[$i] = "$CurrJDTT $CurrPhaseDeg\n";
      }
   }

// Construct final interpolation table ready for use.
   $w = '';
   for ($i=0;   $i < $wCount;   $i++)
   {
   $w .= trim($wArray[$i]) . "\n";
   }
   $PhaseInterpTableStr = trim($w);

// Perform interpolation for phase angle corresponding to JD argument.
   $PhaseDeg = Lagrange_interp($PhaseInterpTableStr, $JDArg);

   $PhaseDeg -= ($PhaseDeg > 360)? 360:0;

   return sprintf("%1.5f", $PhaseDeg);

} // End of  Interp_Lon(...)



// -----------------------------------------------------------
// IDENTICAL TO Interp_Lon() TO REPLACE IT IN FUTURE PROGRAMS.

   function Interp_Angle ($PhaseInterpTable, $JDTTArg)
{
   $wArray = preg_split("[\n]", trim($PhaseInterpTable));
   $wCount = count($wArray);

   $JDArg = floatval($JDTTArg);

   for ($i=1;   $i < $wCount;   $i++)
   {
   $CurrLine = strip_wspace($wArray[$i]);
   $PrevLine = strip_wspace($wArray[$i-1]);

   list($CurrJDTT, $CurrPhaseDeg) = preg_split("[ ]", $CurrLine);
   list($PrevJDTT, $PrevPhaseDeg) = preg_split("[ ]", $PrevLine);

   $diff = $CurrPhaseDeg - $PrevPhaseDeg;

   if ($diff < 0)
      {
       $CurrPhaseDeg += 360;
       $wArray[$i] = "$CurrJDTT $CurrPhaseDeg\n";
      }
   }

// CONSTRUCT INTERPOLATION TABLE.
   $w = '';
   for ($i=0;   $i < $wCount;   $i++)
   {
   $w .= trim($wArray[$i]) . "\n";
   }
   $PhaseInterpTableStr = trim($w);

// -------------------------------------------------------------------
// PERFORM INTERPOLATION FOR PHASE ANGLE CORRESPONDING TO JD ARGUMENT.

   $PhaseDeg = Lagrange_Interp($PhaseInterpTableStr, $JDArg);

   $PhaseDeg -= ($PhaseDeg > 360)? 360:0;

   return sprintf("%1.5f", $PhaseDeg);

} // End of  Interp_Angle(...)






/*
  THIS FUNCTION RETURNS AN HTML TEXTAREA READY FOR DISPLAY
  ACCORDING TO THE GIVEN PARAMETERS.

*/
   function Text_Area ($TextBlock, $TxtRows='', $TxtCols=81, $ColorStr='', $BGColorStr='')
 {
   $rows = intval($TxtRows);
   $cols = intval($TxtCols);
   $C    = trim($ColorStr);
   $B    = trim($BGColorStr);

   if ($cols <= 0) {$cols       = 1;}
   if  ($C == '')  {$ColorStr   = 'black';}
   if  ($B == '')  {$BGColorStr = 'LightYellow';}

   if (intval($rows) == 0) {$rows = 2 + substr_count($TextBlock, "\n");}

// -----------------------------
// CONSTRUCT HTML TEXTAREA BLOCK

   return
"<textarea rows='$rows' cols='$cols' readonly='readonly'
style='color:$ColorStr; background-color:$BGColorStr; font-family:monospace;
font-size:16px; font-weight:bold; border:1px solid black; padding:6px;'>
$TextBlock
</textarea>";

 } // end of  Text_Area(...)



/*
  THIS FUNCTION RETURNS AN HTML TEXTAREA TO DISPLAY PHP
  PROGRAMMING CODE ACCORDING TO THE GIVEN PARAMETERS.

*/
   function Text_Area_PHP ($TextBlock, $TxtRows='', $TxtCols=81, $ColorStr='', $BGColorStr='')
 {
   $rows = abs(intval($TxtRows));
   $cols = abs(intval($TxtCols));
   $C    = trim($ColorStr);
   $B    = trim($BGColorStr);

   if ($cols == 0) {$cols       = 1;}
   if  ($C == '')  {$ColorStr   = 'black';}
   if  ($B == '')  {$BGColorStr = 'white';}

   if (intval($rows) == 0) {$rows = 2 + substr_count($TextBlock, "\n");}

// -----------------------------
// CONSTRUCT HTML TEXTAREA BLOCK

   return
"<textarea rows='$rows' cols='$cols' readonly='readonly'
style='color:$ColorStr; background-color:$BGColorStr; font-family:monospace;
font-size:16px; font-weight:bold; border:2px solid red; padding:6px;'>
$TextBlock
</textarea>";

 } // end of  Text_Area_PHP(...)







/*

COMPUTE USA DAYLIGHT SAVING TIME START/END DATES.

VALID YEAR RANGE: 1975 to 9999

RETURNS A STRING LIKE:
2017 Mar 12 Sun to 2017 Nov 05 Sun = 238 days = 34 weeks = 2457825 to 2458063

DEPENDING ON THE YEAR, THREE DIFFERENT RULES ARE USED.

*/

   function USA_DST ($year)
{
   $y = intval($year);

   if ($y < 1975 or $y > 9999)
      {return "ERROR: Invalid Year '$y'.  Valid range: 1975 to 9999";}
   if ($y > 1974 and $y < 1987)
  {
   $start = Last_DoW_of_mY (0,  4, $y); // Last Sunday in April
   $end   = Last_DoW_of_mY (0, 10, $y); // Last Sunday in October
  }
   if ($y > 1986 and $y < 2007)
  {
   $start = Nth_DoW_of_mY  (1,  0, 4, $y); // First Sunday in April
   $end   = Last_DoW_of_mY (0, 10, $y);    // Last Sunday in October
  }
   if ($y > 2006)
  {
   $start = Nth_DoW_of_mY (2,  0,  3, $y); // Second Sunday in March
   $end   = Nth_DoW_of_mY (1,  0, 11, $y); // First Sunday in November
  }

   $JDNumStart = 0 + DTZ_to_JD("$start 12");
   $StartDate  = JD_to_Y_Mmm_dn_DoW($JDNumStart);
   $JDNumEnd   = 0 + DTZ_to_JD("$end 12");
   $EndDate    = JD_to_Y_Mmm_dn_DoW($JDNumEnd);
   $DaysSpan   = $JDNumEnd - $JDNumStart;
   $WeeksSpan  = $DaysSpan / 7;

   return "$StartDate to $EndDate = $DaysSpan days = $WeeksSpan weeks = $JDNumStart to  $JDNumEnd ";

} // end of  USA_DST(...)





/*

COMPUTE EUROPEAN UNION SUMMER TIME START/END DATES.

RETURNS A STRING LIKE:
2017 Mar 26 Sun to 2017 Oct 29 Sun = 217 days = 31 weeks = 2457839 to 2458056

*/
   function EU_ST ($year)
{
   $y = intval($year);

   if ($y < 2002 or $y > 9999)
      {return "ERROR: Invalid Year '$y'.  Valid range: 2002 to 9999";}

   $start = Last_DoW_of_mY (0,  3, $y); // Last Sunday in March
   $end   = Last_DoW_of_mY (0, 10, $y); // Last Sunday in October

   $JDNumStart = 0 + DTZ_to_JD("$start 12");
   $StartDate  = JD_to_Y_Mmm_dn_DoW($JDNumStart);
   $JDNumEnd   = 0 + DTZ_to_JD("$end 12");
   $EndDate    = JD_to_Y_Mmm_dn_DoW($JDNumEnd);
   $DaysSpan   = $JDNumEnd - $JDNumStart;
   $WeeksSpan  = $DaysSpan / 7;

   return "$StartDate to $EndDate = $DaysSpan days = $WeeksSpan weeks = $JDNumStart to  $JDNumEnd ";
}




/*
   THIS FUNCTION RETURNS A PLAIN-TEXT (STRING) LIST
   OF ALL DEFINED TIME ZONE IDENTIFIERS IN PHP.
*/
   function Time_Zones_ID_List()
{
   $TZListArray = DateTimeZone::ListIdentifiers();
   $TZListCount = count($TZListArray);

   $TZList = '';

   for ($i=0;  $i < $TZListCount;  $i++)  {$TZList .= $TZListArray[$i]."\n";}

   return trim($TZList);
}




/*
   ================================================
   THIS FUNCTION RETURNS A PLAIN-TEXT (STRING) LIST
   OF ALL TIME ZONE IDENTIFIERS DEFINED IN PHP AND
   THE CORRESPONDING LOCAL TIMES AND LOCALES WITH
   EXTERNAL WIKI-LINKS TO GENERAL LOCALE INFO.
*/

   function Current_World_Times_Table()
{
   $wArray = DateTimeZone::ListIdentifiers();
   $wCount = count($wArray);

   $TZList = '';

   for($i=1;   $i < $wCount;   $i++)
      {
       $locale  = trim($wArray[$i]);
       $localex = '';
       $j = strpos($locale, ',');

       if($j !== FALSE)
      {
       $localex = substr($locale, $j, strlen($locale));
       $locale = substr($locale, 0, $j);
      }


// process only data lines with / characters.
// -666 = Process ALL lines (assumes PHPSL standard raw data format)
//      $LocPointer = strpos($locale, '/');

      $LocPointer = strrpos($locale, '/');

      if ($LocPointer !== -666)
     {
      $DTZ           = new DateTimeZone($locale);
      $LocaleTime    = new DateTime('now', $DTZ);
      $UTOffsetHours = ($DTZ -> getOffset($LocaleTime))/3600;
      $UTOffsetHMS   = substr(hours_to_hms($UTOffsetHours, 0, '+', ':'),0,6);
      $UThhmmss      = gmdate('H:i:s');

//      $LocationName = strrchr($locale, '/');
     $LocationName  = substr($locale, $LocPointer+((strrpos($locale, '/') !== FALSE)? 1:0), strlen($locale));


// APPLY SPECIAL FILTERS TO WIKI-LINK
   $LocationName  = str_replace('New_Salem'      , 'New_Salem,_North_Dakota', $LocationName);
   $LocationName  = str_replace('Beulah'         , 'Beulah,_North_Dakota', $LocationName);
   $LocationName  = str_replace('New_York'       , 'New_York_City', $LocationName);
   $LocationName  = str_replace('Center'         , 'Center,_North_Dakota', $LocationName);
   $LocationName  = str_replace('Knox'           , 'Knox,_Indiana', $LocationName);
   $LocationName  = str_replace('Palmer'         , 'Palmer_Land', $LocationName);
   $LocationName  = str_replace('Troll'          , 'Troll_(research_station)', $LocationName);
   $LocationName  = str_replace('Canary'         , 'Canary_Islands', $LocationName);
   $LocationName  = str_replace('Resolute'       , 'Resolute,_Nunavut', $LocationName);
   $LocationName  = str_replace('Chihuahua'      , 'Chihuahua_(state)', $LocationName);
   $LocationName  = str_replace('Petersburg'     , 'Petersburg,_Indiana', $LocationName);
   $LocationName  = str_replace('Stanley'        , 'Stanley,_Falkland_Islands', $LocationName);
   $LocationName  = str_replace('Thule'          , 'Qaanaaq', $LocationName);
   $LocationName  = str_replace('Goose_Bay'      , 'Happy_Valley-Goose_Bay', $LocationName);
   $LocationName  = str_replace('Halifax'        , 'Halifax,_Nova_Scotia', $LocationName);
   $LocationName  = str_replace('La_Roija'       , 'La_Rioja,_Argentina', $LocationName);
   $LocationName  = str_replace('Mendoza'        , 'Mendoza,_Argentina', $LocationName);
   $LocationName  = str_replace('San_Juan'       , 'San_Juan,_Argentina', $LocationName);
   $LocationName  = str_replace('San_Luis'       , 'San_Luis,_Argentina', $LocationName);
   $LocationName  = str_replace('Boa_Vista'      , 'Boa_Vista,_Roraima', $LocationName);
   $LocationName  = str_replace('Grand_Turk'     , 'Grand_Turk_Island', $LocationName);
   $LocationName  = str_replace('Marigot'        , 'Marigot,_Saint_Martin', $LocationName);
   $LocationName  = str_replace('Nassau'         , 'Nassau,_Bahamas', $LocationName);
   $LocationName  = str_replace('Easter'         , 'Easter_Island', $LocationName);
   $LocationName  = str_replace('Oral'           , 'Oral,_Kazakhstan', $LocationName);
   $LocationName  = str_replace('Christmas'      , 'Christmas_Island', $LocationName);
   $LocationName  = str_replace('Davis'          , 'Davis_Station', $LocationName);
   $LocationName  = str_replace('Ho_Chi_Minh'    , 'Ho_Chi_Minh_City', $LocationName);
   $LocationName  = str_replace('Hovd'           , 'Khovd,_Khovd', $LocationName);
   $LocationName  = str_replace('Adak'           , 'Adak_Island', $LocationName);
   $LocationName  = str_replace('Gambier'        , 'Gambier_Islands', $LocationName);
   $LocationName  = str_replace('Midway'         , 'Midway_Atoll', $LocationName);
   $LocationName  = str_replace('Lome'           , 'Lomé', $LocationName);
   $LocationName  = str_replace('Faroe'          , 'Faroe_Islands', $LocationName);
   $LocationName  = str_replace('Casey'          , 'Casey_Station', $LocationName);
   $LocationName  = str_replace('South_Georgia'  , 'South_Georgia_and_the_South_Sandwich_Islands', $LocationName);
   $LocationName  = str_replace('St_Johns'       ,  "St._John&#39;s,_Newfoundland_and_Labrador", $LocationName);
   $LocationName  = str_replace('Cordoba'        , 'Córdoba,_Argentina', $LocationName);
   $LocationName  = str_replace('Norfolk'        , 'Norfolk_Island', $LocationName);
   $LocationName  = str_replace('Wake'           , 'Wake_Island', $LocationName);
   $LocationName  = str_replace('McMurdo'        , 'McMurdo_Station', $LocationName);
   $LocationName  = str_replace('Lord_Howe'      , 'Lord_Howe_Island', $LocationName);
   $LocationName  = str_replace('Bougainville'   , 'Bougainville_Island', $LocationName);
   $LocationName  = str_replace('Lindeman'       , 'Lindeman_Islands_National_Park', $LocationName);
   $LocationName  = str_replace('Comoro'         , 'Comoro_Islands', $LocationName);
   $LocationName  = str_replace('Gaza'           , 'Gaza_City', $LocationName);
   $LocationName  = str_replace('Reunion'        , 'Réunion', $LocationName);
   $LocationName  = str_replace('Chita'          , 'Chita_Peninsula', $LocationName);
   $LocationName  = str_replace('Dili'           , 'Dili_District', $LocationName);
   $LocationName  = str_replace('Darwin'         , 'Darwin,_Northern_Territory', $LocationName);
   $LocationName  = str_replace('DumontDUrville' , 'Antarctica/DumontDUrville', $LocationName);
   $LocationName  = str_replace('Chuuk'          , 'Chuuk_State', $LocationName);
   $LocationName  = str_replace('Currie'         , 'Currie,_Tasmania', $LocationName);
   $LocationName  = str_replace('Macquarie'      , 'Antarctica/Macquarie', $LocationName);
   $LocationName  = str_replace('Wallis'         , 'Wallis_(island)', $LocationName);
   $LocationName  = str_replace('Phoenix'        , 'Phoenix,_Arizona', $LocationName);
   $LocationName  = str_replace('St_Thomas'      , 'St._Thomas,_Ontario', $LocationName);
   $LocationName  = str_replace('St_Vincent'     , 'Meaford,_Ontario', $LocationName);
   $LocationName  = str_replace('Noronha'        , 'Fernando_de_Noronha', $LocationName);
   $LocationName  = str_replace('La_Rioja'       , 'La_Rioja,_Argentina', $LocationName);
   $LocationName  = str_replace('Vevay'          , 'Vevay,_Indiana', $LocationName);
   $LocationName  = str_replace('Vincennes'      , 'Vincennes,_Indiana', $LocationName);
// $LocationName  = str_replace('xxxxxxxxx'      , 'xxxxxxxxxxx', $LocationName);
// $LocationName  = str_replace('xxxxxxxxx'      , 'xxxxxxxxxxx', $LocationName);
// $LocationName  = str_replace('xxxxxxxxx'      , 'xxxxxxxxxxx', $LocationName);
// $LocationName  = str_replace('xxxxxxxxx'      , 'xxxxxxxxxxx', $LocationName);
// $LocationName  = str_replace('xxxxxxxxx'      , 'xxxxxxxxxxx', $LocationName);
// $LocationName  = str_replace('xxxxxxxxx'      , 'xxxxxxxxxxx', $LocationName);
// $LocationName  = str_replace('xxxxxxxxx'      , 'xxxxxxxxxxx', $LocationName);
// $LocationName  = str_replace('xxxxxxxxx'      , 'xxxxxxxxxxx', $LocationName);
// $LocationName  = str_replace('xxxxxxxxx'      , 'xxxxxxxxxxx', $LocationName);
// $LocationName  = str_replace('xxxxxxxxx'      , 'xxxxxxxxxxx', $LocationName);

    $LocationName  = str_replace(' ', '%20', $LocationName);
    $WikiLink      = "<a href='https://en.wikipedia.org/wiki/$LocationName' target='_blank'>" . "$locale</a>";


//    -------------------------------------
//    GET CURRENT DATE AND TIME ELEMENTS UT
      $Y             = gmdate('Y'); $Y = intval($Y);
      $mn            = gmdate('m'); $m = intval($mn);
      $dn            = gmdate('d'); $d = intval($dn);
      $hhmmss        = gmdate('H:i:s');
      $JDNumUT       = GregorianToJD($m, $d, $Y);
      $DoW           = ($JDNumUT + 1) % 7;
      $DoWString     = trim(substr(DOW_NAMES, 10*$DoW, 10));
      $MonthString   = trim(substr(MONTH_NAMES, 10*($m-1), 10));
      $DoWStr        = substr($DoWString, 0, 3);
      $MonthStr      = substr($MonthString, 0, 3);

      $UTFracDay     = hms_to_hours($UThhmmss) / 24;
      $TZFracDay     = hms_to_hours($UTOffsetHours) / 24;

// COMPUTE JD (JULIAN DATE) FOR UT.
   $JDUT = $JDNumUT + $UTFracDay - 0.5;
   $DateStringUT = JD_To_Date_Time_String ($JDUT);


// COMPUTE LOCAL JD (JULIAN DATE) FOR LT.
   $JDLT = $JDNumUT + $UTFracDay + $TZFracDay - 0.5;
   $DateStringLT = JD_To_Date_Time_String ($JDLT);
   $hhmmssLT = HMS24_to_HMS12(substr($DateStringLT, -8));
   $DateStringLT = substr($DateStringLT, 0,15)." $hhmmssLT";

   $TZList .= "UTC$UTOffsetHMS   $DateStringLT   $WikiLink\n";
  }
  }

   $TZList = str_replace('_', ' ', trim($TZList));

// DONE.
   return trim($TZList);

} // END OF  Current_World_Times_Table()






/*
   This function returns a 24-hour time string to an AM/PM time string.

   Rules:
   Set default label to PM
   If time is <  12:00:00 then label it AM
   If time is >= 12:00:00 then subtract 12 hours

*/
   function HMS24_to_HMS12 ($hhmmss24Str)
   {
   $AMPM  = 'PM';
   $hours = HMS_to_Hours(trim($hhmmss24Str));

   if ($hours <  12) {$AMPM = 'AM';}
   if ($hours >= 12) {$AMPM = 'PM'; $hours -= 12;}
   if ($hours < 1)   {$hours += 12;}

   return Hours_to_HMS($hours, 0, '', ':') . " $AMPM";
   }







   function XYZ_For_Ellipsoid ($Re, $Rp, $LatDeg, $LonDeg, $Elevation)
{
   $a   = trim($Re);
   $b   = trim($Rp);
   $lat = deg2rad($LatDeg);
   $lon = deg2rad($LonDeg);
   $H   = floatval(trim($Elevation));

   $e2  = ($a*$a - $b*$b) / ($a*$a);

// _______________________
// COMPUTE CIRCUMFERENCES.

   $PolarCircum   = Ellipse_Circum ($a, $b);
   $EquatorCircum = Ellipse_Circum ($a, $a);

// COMPUTE ELLIPSOID STATISTICS
   $e = sqrt($e2);
   $u = $a / SqRt(1 - $e2 * sin($lat) * sin($lat));
   $x = ($u + $H) * cos($lat) * cos($lon);
   $y = ($u + $H) * cos($lat) * sin($lon);
   $z = ($u * (1 - $e2) + $H) * sin($lat);
   $a = $a;
   $b = $b;
   $e = $e;
   $H = $H;

// DONE.
   return "$x $y $z $a $b $e $H $EquatorCircum $PolarCircum";
}








/*
   This function computes the static time dilation for any given
   CONSTANT speed (m/s) or beta = (v/c).

   It is called static in reference to the rest frame, where the
   speed is constant, without acceleration.

   T = 1 = Time interval, according to Earth-based clock, during
       which the spacefraft has coasted at the given speed.

   t = Time interval, according to spacecraft clock, during which
       the spacefraft has coasted at the given speed.

   The rules for speed arguments:
   Any speed >= 1 and < 299792458 is taken as meters per second.
   Any speed <= 0 and < 1 is taken as relativistic speed beta (c = 1).

*/

   function Static_Time_Dilation ($Speed, $Decimals=16)
{
   $c = '299792458';
   $v = trim($Speed);

   $decimals = abs(intval(trim($Decimals)));
   if ($decimals > 100) {$decimals = 100;}
   $d = $decimals + 5 + floor($decimals / 4);

   if ($v >= 0 and $v < 1){$v = bcmul($c, $v, $d);}

   if (bccomp($c, $v, $d) < 0) {return 'ERROR: Invalid speed argument.';}

   $beta  = bcdiv($v,$c, $d);
   $beta2 = bcmul($beta,$beta, $d);

   $t = bcsqrt(bcsub('1', $beta2, $d), $d);

// Round-off time result to the given number of decimals.
   $dilation = bcadd($t, '0.' . str_repeat('0', $decimals) . '5', $decimals);

   return $dilation;
}



/*
   ____________________________________________________________
   This function simply performs a chain of basic BC arithmetic
   operations to the specified number of decimals precision.
   Default precision = 0 decimals

   IMPORTANT NOTE:
   Only the basic arithmetic (+ - * /) operators are allowed.
   All operations are from left-to-right without precedence.

   '12.3456 * 2.3741589 + 0.0116547'

   There MUST be a space on each side of the operators so that
   they can be distinguished from the numerical signs of the
   numerical string values.
   ____________________________________________________________
*/
   function bcChain_Arith($ArithChainString, $Decimals=0)
{
// ___________________________________________________________
// Read arithmetic chain string and pad operators with spaces
// on both ends and then remove all extra white space from
// within the string.

   $w = trim($ArithChainString);
   $w = str_replace('*', ' * ', $w);
   $w = str_replace('/', ' / ', $w);
   $w = str_replace('- ', ' - ', $w);
   $w = str_replace('+ ', ' + ', $w);

   $ArithStr = preg_replace("/\s+/", " ", trim($w));

// _________________________________________________
// Read decimals argument as positive integer value.
// Default = 0 decimals.

   $d = trim($Decimals);  if (!is_numeric($d)) {$d = 0;}
   $d = floor(abs($d));

// _______________________________________________
// Store arithmetic chain string in working array.

   $wArray = preg_split("[ ]", $ArithStr);
   $wCount = count($wArray);

// _____________________________________________________
// Within the working array:
//
// Even elements 0, 2, 4, ..., = Numerical value strings
// Odd elements  1, 3, 5, ..., = Operators    +  -  *  /

// ____________________________________
// Count numeric and operator elements.

   $NumCount = $wCount / 2;
   $OpCount  = $NumCount - 1;

// _______________________________
// Construct numeric string array.

   $NumArray = array();

   $j=0;  for($i=0;   $i < $wCount;   $i+=2)
  {
   $NumArray[$j] = "'" . $wArray[$i] . "'";  $j++;
  }

// _____________________________________
// Construct arithmetic operators array.

   $wArray  = array_reverse($wArray);
   $OpArray = array();

   $j=0;  for($i=1;   $i < $wCount;   $i+=2)
  {
   $OpArray[$j] = $wArray[$i];  $j++;
  }

   $w = $OpString = '';

   for ($i=0;   $i < $OpCount;  $i++)
       {
        $OpString .= $OpArray[$i].'(';
       }

   $w = $OpString . $NumArray[0] . ', ';

// _______________________________
// Construct numeric string array.

   for ($i=1;   $i < $NumCount;   $i++)
       {
//        $u  = ($i < $NumCount-1)? '+5':'';
        $w .= $NumArray[$i] . ", $d" . (($i < $NumCount-1)? '+5':'') . "), ";
       }
// _______________________________________________
// Construct sequential BC arithmetic chain string
// ready for evaluation.

   $w = substr($w, 0, strlen($w)-2);

   $w = str_replace('+(', 'bcadd(', $w);
   $w = str_replace('-(', 'bcsub(', $w);
   $w = str_replace('*(', 'bcmul(', $w);

   $bcArithString = str_replace('/(', 'bcdiv(', $w);

// Done.
   return "$bcArithString;";
}





/*
   This function computes the properties of a sphere
   given any one of 4 values.

   The (R,C,A,V) symbols are NOT case sensitive.

   R = Radius value
   C = Circumference value
   A = Surface area value
   V = Volume value

   Given any one known variable, the other three can be
   directly computed from it.

   EXAMPLE 1:
   The arguments ('R = 1.62125', 10)
   Should return:
   '1.6212500000 10.1866141792 33.0300964762 17.8500146373'

   EXAMPLE 2:
   The single argument ('V = 2.5')
   Should return:
   '0.8419451504803148 5.2900973989490201 8.9079437012273097 2.5000000000000000'
   Default decimals = 16

   Returned values will be an 'R C A V' string with the
   numerical 'R C A V' values separated by spaces.

*/

   function Spherical_Properties ($KnownRCAV, $Decimals=16)
{
   $KVStr = strtoupper(str_replace(' ', '', trim($KnownRCAV)));

// _____________________________________________
// Define local  constant pi to 110 decimals and
// related derived values. These constants could
// also be made global.

   $PI   = '3.141592653589793238462643383279502884197169399375105820974944592307816406286208998628034825342117067982148';
   $PIx2 = bcmul('2', $PI, 110);
   $PIx4 = bcmul('4', $PI, 110);

// __________________________________________
// Read decimals value from 0 to 100 maximum.

   $decimals = abs(intval(trim($Decimals)));
   if ($decimals > 100) {$decimals = 100;}
   $d = $decimals + 5 + floor($decimals / 4);

// _________________________________________
// Assume initial ERROR state by default.  A
// successful operation will set it to FALSE.

   $ERROR_FLAG_SET = TRUE;

// ____________________________________
// Handle the case of known radius (R).

   if (substr($KVStr,0, 2) == 'R=')
  {
   $R  = substr($KVStr,2, strlen($KVStr));
   $R2 = bcmul($R,$R, $d);
   $R3 = bcmul($R, bcmul($R,$R, $d), $d);
   $C  = bcmul($PIx2, $R, $d);
   $A  = bcmul($PIx4, $R2, $d);
   $V  = bcdiv(bcmul($PIx4, $R3, $d), '3', $d);
   $R  = bcadd($R, '0', $d);
   $ERROR_FLAG_SET = FALSE;
  }

// ___________________________________________
// Handle the case of known circumference (C).

   if (substr($KVStr,0,2) == 'C=')
  {
   $C  = substr($KVStr,2, strlen($KVStr));
   $R  = bcdiv($C, bcmul('2', $pi, $d), $d);
   $R2 = bcmul($R,$R, $d);
   $R3 = bcmul($R, bcmul($R,$R, $d), $d);
   $A  = bcmul($PIx4, $R2, $d);
   $V  = bcdiv(bcmul($PIx4, $R3, $d), '3', $d);
   $C  = bcadd($C, '0', $d);
   $ERROR_FLAG_SET = FALSE;
  }

// __________________________________________
// Handle the case of known surface area (A).

   if (substr($KVStr,0,2) == 'A=')
  {
   $A  = substr($KVStr,2, strlen($KVStr));
   $R  = bcsqrt(bcdiv($A, $PIx4, $d), $d);
   $R2 = bcmul($R,$R, $d);
   $R3 = bcmul($R, bcmul($R,$R, $d), $d);
   $C  = bcmul($PIx2, $R, $d);
   $V  = bcdiv(bcmul($PIx4, $R3, $d), '3', $d);
   $A  = bcadd($A, '0', $d);
   $R  = bcadd($R, '0', $d);
   $ERROR_FLAG_SET = FALSE;
  }

// ____________________________________
// Handle the case of known volume (V).

   if (substr($KVStr,0,2) == 'V=')
  {
   $V = substr($KVStr,2, strlen($KVStr));
   $x = bcdiv(bcmul('3', $V, $d), $PIx4, $d);
   $r = sprintf("%1.16f", pow($x, 1/3));

   for ($i=1;  $i <= 100;   $i++)
  {
   $R = bcdiv(bcadd(bcmul('2',$r,$d),bcdiv($x, bcpow($r,'2',$d),$d),$d),'3',$d);
        if (bccomp($r,$R,$d) == 0) {break;} else {$r = $R;}
  }
   $C  = bcmul($PIx2, $R, $d);
   $A  = bcmul($PIx4, bcmul($R,$R, $d), $d);
   $R  = bcadd($R, '0', $d);
   $V  = bcadd($V, '0', $d);
   $ERROR_FLAG_SET = FALSE;
  }

// _______________________
// Check error flag state.

   if ($ERROR_FLAG_SET) {return "ERROR: Invalid or missing R, C, A or V argument.";}

// _________________________________
// Round-off all values to the given
// number of output decimals.

   $Z = '0.' . str_repeat('0', $decimals) . '5';
   $R = bcadd($R, $Z, $decimals);
   $C = bcadd($C, $Z, $decimals);
   $A = bcadd($A, $Z, $decimals);
   $V = bcadd($V, $Z, $decimals);

   return "$R $C $A $V";

} // END OF  Spherical_Properties (...)





/*
  This function simply rounds off a given BC number
  string to the given number of decimals.

  Default = 16 decimals.

*/
   function bcRound ($NumericStr, $Decimals=16)
  {
   $x = trim($NumericStr);

   $x1 = substr($x,0,1);

   if ($x1 == '-') {$sign = '-';} else {$sign = '+';}

   $sign = ($x1 == '-')? '-' : '';
   $sign = ($x1 == '+')? '+' : $sign;

   $x = str_replace('-', '', $x);
   $x = str_replace('+', '', $x);

   $decimals = floor(abs(trim($Decimals)));

   return $sign.bcadd($x, '0.'.str_repeat('0', $decimals).'5', $decimals);
  }









/*
   ___________________________________________________
   This function computes the Nth root of any positive
   (x) argument to up to 100 decimals precision.

   N = Root = Positive integer >= 2
   xArgStr  = X argument as numeric string
   Decimals = Decimals precision (default = 16, maximum = 100)
*/

   function bcNth_Root ($N, $xArgStr, $Decimals=16)
{
   $N = floor(trim($N));
   if (!is_numeric($N) or $N < 2) {return FALSE;}

   $decimals = abs(intval(trim($Decimals)));
   if ($decimals > 100) {$decimals = 100;}
   $d = $decimals + floor($decimals / 4);

   $x = trim($xArgStr);
   $a = sprintf("%1.16f", pow($x, 1/$N));
   $k = 0;  for ($i=1;  $i <= 20;   $i++)
  {
   $b = bcdiv(bcadd(bcmul($N-1,$a,$d),bcdiv($x,bcpow($a,$N-1,$d),$d),$d),$N,$d);

   if (bccomp($a,$b,$d) == 0) {$k++;  break;} else {$a = $b;  $k++;}
  }

// _____________________________________________
// Round off result to given number of decimals.

   $NthRoot = Rtrim(Rtrim(bcadd($b, '0.' . str_repeat(0, $decimals) . '5', $decimals), '0'), '.');

// _____
// Done.
   return "$NthRoot";
}





/*
   This function computes the difference between 2 times
   or times of day in the context TimeB minus TimeA.

   Input times are positive values, but the result could
   be a negative value.

   TimeA = Earlier time.
   TimeB = Later time.

   EXAMPLE:
   01:39       Moonrise - Waning crescent moon
   07:36       Sunrise
   12:48       Moonset
   16:54       Sunset

   $MoonInSky = HMS_Diff_24 ("01:39", "12:48"); // 11 09 00
   $SunInSky  = HMS_Diff_24 ("01:39", "12:48"); // 09 18 00

*/
   function HMS_Diff_24 ($TimeStrA, $TimeStrB, $Decimals=0)
{
   $a = strtolower(trim($TimeStrA));
   $b = strtolower(trim($TimeStrB));

   $decimals = floor(abs(trim($Decimals)));
   $d = 32;

   $a = str_replace(':', ' ', $a);
   $a = str_replace('h', ' ', $a);
   $a = str_replace('m', ' ', $a);
   $a = str_replace('s', ' ', $a);

   $b = str_replace(':', ' ', $b);
   $b = str_replace('h', ' ', $b);
   $b = str_replace('m', ' ', $b);
   $b = str_replace('s', ' ', $b);

   $a = preg_replace("/\s+/", " ", trim($a));
   $b = preg_replace("/\s+/", " ", trim($b));

   $hha = $hhb = $mma = $mmb = $ssa = $ssb = '';

// Parse Time A:
   $wArray = preg_split("[ ]", $a);
   $wCount = count($wArray);
   if ($wCount >= 1) {$hha = $wArray[0];}
   if ($wCount >= 2) {$mma = $wArray[1];}
   if ($wCount >= 3) {$ssa = $wArray[2];}

// Parse Time B:
   $wArray = preg_split("[ ]", $b);
   $wCount = count($wArray);
   if ($wCount >= 1) {$hhb = $wArray[0];}
   if ($wCount >= 2) {$mmb = $wArray[1];}
   if ($wCount >= 3) {$ssb = $wArray[2];}

// Convert TimeA to hours
   $U = bcmul('3600',$hha, $d);
   $V = bcmul('60',$mma, $d);
   $W = $ssa;
   $aHours = bcdiv(bcadd(bcadd($U,$V, $d), $W, $d), '3600', $d);

// Convert TimeB to hours
   $U = bcmul('3600',$hhb, $d);
   $V = bcmul('60',$mmb, $d);
   $W = $ssb;
   $bHours = bcdiv(bcadd(bcadd($U,$V, $d), $W, $d), '3600', $d);

   $hhDiff = bcsub($bHours, $aHours, $d);
   $NumSign = ($hhDiff < 0)? '-':'';
   $hours = str_replace('-','', $hhDiff);

// Convert hours difference into hh:mm:ss
   $hh = floor($hours);

   $minutes = bcmul('60', bcsub($hours, $hh, $d), $d);
   $mm = floor($minutes);

   $seconds = bcmul('60', bcsub($minutes, $mm, $d), $d);
   $ss = bcadd($seconds, '0', $d);

   $hh = sprintf("%02d", $hh);
   $mm = sprintf("%02d", $mm);
   $ss = bcadd($ss, '0.' . str_repeat('0', $decimals) . '5', $decimals);
   if ($ss < 10){$ss = "0$ss";}

   if ($ss == 60) {$ss = 0; $mm++;}
   if ($mm == 60) {$mm = 0; $hh++;}

   $hh = sprintf("%02d", $hh);
   $mm = sprintf("%02d", $mm);
   $ss = bcadd($ss, '0.' . str_repeat('0', $decimals) . '5', $decimals);
   if ($ss < 10){$ss = "0$ss";}

  return "$NumSign$hh $mm $ss";

}



/*
  This function computes the geocentric (xyz-coordinates) given
  the RA, declination and distance of the body.

*/
   function RA_Decl_Dist_to_XYZ ($RAHours, $DeclDeg, $Distance)
{
   $RA   = deg2rad(trim($RAHours)*15);
   $decl = deg2rad(trim($DeclDeg));
   $R    = trim($Distance);

   $x = $R * cos($RA) * cos($decl);
   $y = $R * sin($RA) * cos($decl);
   $z = $R * sin($decl);

   return "$x $y $z";
}















// ****************************************************************
// ****************************************************************
// ****************************************************************
// ****************************************************************
// ****************************************************************

// TERMINUS LINE.

// ****************************************************************
// ****************************************************************
// ****************************************************************
// ****************************************************************
// ****************************************************************


   FUNCTION PHPSCILABS_Info()
{
   $cYear = gmdate("Y");

/*
  USE GLOB TO CONSTRUCT LIST OF AVAILABLE MODULES

  Define a list of module names and then
  store them in a work array. Modules may
  be added or removed from this list, but
  note that in certain cases, some modules
  may require functions in other modules.

*/

   $modules = "PHPSCILABS.php\n";

   ForEach (glob("*_Module.php") as $FileName)
  {
   $modules .= "$FileName\n";
  }



// Convert module names listing into an indexed work array
// with the same name.
   $modules = preg_split("[ ]", strip_wspace($modules));

// =========================================================================

// Get modules count.
   $ModulesCount = count($modules);

   $out =
"
<!doctype html public '-//W3C//DTD HTML 4.01 Transitional//EN' 'http://www.w3.org/TR/html4/loose.dtd'>

<html>

<head>

<title>PHP Science Labs Module Contents</title>

<style type='text/css'>
BODY {background-color:white; color:black; font-family:Verdana; font-size:100%;}

TEXTAREA{font-family:monospace; font-size:125%; font-weight:bold; background-color:LightYellow; color:black; border:1px solid black; border-radius:8px; box-shadow:2px 3px 4px #666666; white-space:pre;}
</style>
</head>

<b><span style='font-family:Verdana; font-size:90%;'>
A quick listing of the current contents of the PHPSCILABS function module(s).</span></b>
<br><br>
";

   for ($i=0;   $i < $ModulesCount;   $i++)
       {
        $out .= Module_Info($modules[$i]);
       }
   return $out . "<br><br>\n</body>\n</html>\n";
   }

/*
   Scans and returns a listing of all custom definitions
   and functions found within the given module.

*/

   FUNCTION Module_Info ($ModuleFile='PHPSCILABS.php')
{
// Define path to PHP modules folder.
//   $ModuleFilePath = "modules/$ModuleFile";
   $ModuleFilePath = "$ModuleFile";

// Load module file into work array and count
// the total number of text lines in the file.
   $ModuleLines = file("$ModuleFilePath");
   $RevisedStr  = gmdate("Y F d  H:i:s l", FileMTime($ModuleFilePath)) . ' GMT';
   $ModuleLinesCount = count($ModuleLines);

// This shouldn't be here, if it is, so it won't be.
   $ModuleFileName = str_replace("../",   "", $ModuleFilePath);


// ----------------------------------------------------
// Generate list of all custom constants within module.

   $ConstantsList  = '';
   $ConstantsCount = 0;

   for ($i=0;   $i < $ModuleLinesCount;   $i++)
       {
        $ModuleLine = LTrim($ModuleLines[$i]);
        if (substr($ModuleLine,0,6) == 'define')
       {
        list($NamePartStr, $ValPartStr) = preg_split("[,]", $ModuleLine);
         $NamePartStr = strip_wspace(str_replace('"', '', $NamePartStr));
         $ValPartStr  = LTrim(str_replace('"', '', $ValPartStr));
         $NamePartStr = strip_wspace(str_replace('define (', '', $NamePartStr));
         $ValPartStr  = substr($ValPartStr, 0, strpos($ValPartStr, ';'));

         $w = str_replace(')', '"', "$NamePartStr = &#34;$ValPartStr");

         $ConstantsList .= " $w\n";
         $ConstantsCount++;
        } // END OF  if(...)
       } // END OF  for(i=0; ...)


// ____________________________________________________
// GENERATE LIST OF ALL CUSTOM FUNCTIONS WITHIN MODULE.
// TO PREVENT A FUNCTION FROM BEING LISTED, START THE
// FUNCTION PREFIX WITH UPPERCASE, LIKE  "Function".

   $FunctionsList  = '';
   $FunctionsCount = 0;

   for ($i=0;   $i < $ModuleLinesCount;   $i++)
       {
        $ModuleLine = trim($ModuleLines[$i]);

        if (substr($ModuleLine, 0, 8) == 'function')
           {
            $ModuleLine = trim(substr($ModuleLine, 8, strlen($ModuleLine)));

            $FunctionsList .= " $ModuleLine\n\n";

            $FunctionsCount++;
           }
       }

   $FunctionsList = trim($FunctionsList);

   $s1 = $s2 = '';

   $IsAre1 = $IsAre2 = 'is';

   if ($ConstantsCount <> 1)
  {
   $s1 = 's';   $IsAre1 = 'are';
   if ($ConstantsCount == 0) {$ConstantsCount = 'No';}
  }

   if ($FunctionsCount <> 1)
  {
   $s2 = 's';   $IsAre2 = 'are';
   if ($FunctionsCount == 0) {$FunctionsCount = 'No';}
  }

   $ConstantsList = trim($ConstantsList);

// -------------------------------------------------
// Construct return final html TEXTAREA code string.


   $ModuleFileName = str_replace('modules/', '', $ModuleFileName);

   $_BAR_ = str_repeat("&#151;", 76);

   $w =
"
 $ModuleFileName

 Revised:
 $RevisedStr


 MODULE CONTENTS LISTING:
 $_BAR_
 $ConstantsCount constant$s1 $IsAre1 defined in this module.

 $ConstantsList


 $_BAR_
 $FunctionsCount function$s2 $IsAre2 defined in this module.

 $FunctionsList
 ";

   $NumRows = 2 + substr_count($w, "\n");

 $w =
"
<textarea cols='81' rows=\"$NumRows\" readonly='readonly' style='font-size:14px; white-space:pre;'>
$w
</textarea>
<br><br>
";



return $w;

} // End of  Module_Info (...)





?>




